package com.perdemia.permissionanalyzer.application.handlers.menu;

import gnu.trove.list.array.TIntArrayList;

import java.lang.reflect.InvocationTargetException;
import java.nio.file.FileSystems;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.attribute.UserPrincipal;
import java.nio.file.attribute.UserPrincipalLookupService;
import java.sql.PreparedStatement;

import javax.inject.Named;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.e4.ui.workbench.modeling.ESelectionService;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.logging.GUILogHandler;
import com.perdemia.permissionanalyzer.application.parts.FilePart;
import com.perdemia.permissionanalyzer.application.util.FilterHelper;
import com.perdemia.permissionanalyzer.application.util.FilterHelper.ObjectType;
import com.perdemia.permissionanalyzer.data.FileInfo;
import com.sun.jna.platform.win32.Advapi32Util;
import com.sun.jna.platform.win32.WinNT;
import static com.sun.jna.platform.win32.WinNT.SE_DACL_PROTECTED;

public class ChangeACLInheritanceHandler {
	private static final Logger logger = LoggerFactory.getLogger(ChangeACLInheritanceHandler.class.getName());
		
	@Execute
	public void execute(final Shell shell, EPartService partService, ESelectionService selectionService,
			@Optional @Named("com.perdemia.permissionanalyzer.application.commands.changeaclinheritance.param.allfiles") final String allFiles){
		StructuredSelection selection = (StructuredSelection) selectionService.getSelection(FilePart.ID);
		final FileInfo file = (FileInfo) selection.getFirstElement();
		MPart filePart = partService.findPart(FilePart.ID);
		final FilePart filePartObject = (FilePart) filePart.getObject();
		
		if (allFiles.equalsIgnoreCase("true")) {
			// Change the ACL Inheritance of all the files
			// determine the number of files
			// start with a progress monitor
			ProgressMonitorDialog progressDialog = new ProgressMonitorDialog(Display.getCurrent().getActiveShell());
			try {
				progressDialog.run(true, true, new IRunnableWithProgress() {

					@Override
					public void run(final IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
						try {
							monitor.setTaskName("Getting all files from the database...");
													
							// execute the query and extract all the applicable file ids
							PreparedStatement ps = FilterHelper.getInstance().buildFilterQuery(ObjectType.FILE, false, false, 0, "FILEINFO.PARENT_FILE_IDS ASC, FILEINFO.NAME_LOWERCASE ASC", "FILEINFO.IS_DACL_PROTECTED = true");
							final TIntArrayList uniqueFileIds = FilterHelper.getInstance().getApplicableFileIds(ps, false, false);
							final int fileCount = uniqueFileIds.size();
							
							Display.getDefault().syncExec(new Runnable(){
								public void run() {
									try {
										if (fileCount == 0) {
											SyncMessageDialog.showMessage(MessageDialog.INFORMATION, "No files found", "All flies and directories in this overview have already ACL Inheritance enabled.");
										} else {
											if (SyncMessageDialog.showMessage(MessageDialog.CONFIRM, "Change ACL file inheritance", "Are you sure you want to enable ACL Inheritance for " + fileCount + " files/directories from this overview?")) {
												ProgressMonitorDialog progressDialog = new ProgressMonitorDialog(Display.getCurrent().getActiveShell());
												try {
													progressDialog.run(true, true, new IRunnableWithProgress() {
														@Override
														public void run(final IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
															monitor.setTaskName("Applying ACL inheritance");
															monitor.beginTask("Setting ACL inheritance on " + fileCount + " directories/files", fileCount);
															try {
																for (int counter = 0; counter < uniqueFileIds.size(); counter++) {
																	int fileId = uniqueFileIds.getQuick(counter);
																	final FileInfo file = FileInfo.getById(fileId);
																	monitor.subTask(file.determineFullPath());
																	
																	final Path path = Paths.get(file.determineFullPath());
																	
																	// get the security descriptor for this file
																	// NOTE: The Advapi32Util.getFileSecurityDescriptor method has been modified! In case the path startsWith \\ and 
																	// the length is longer than 248 characters it will convert to path to a short notation (~1 etc). Otherwise the 
																	// getNamedSecurityInfo will throw an error "The filename, directory name, or volume label syntax is incorrect."
																	final WinNT.SECURITY_DESCRIPTOR_RELATIVE secDescriptor = Advapi32Util.getFileSecurityDescriptor(path.toFile(), false);;
																	
																	// clear the SE_DACL_PROTECTED bit
																	secDescriptor.Control &= ~SE_DACL_PROTECTED;
																	
																	Advapi32Util.setFileSecurityDescriptor(path.toFile(), secDescriptor, false, false, false, false, true, false);
																	
																	file.setDaclProtected(false);
																	file.save();
																	
																	Display.getDefault().syncExec(new Runnable(){
																		public void run() {
																			// refresh the UI
																			if (filePartObject.getTreeViewer().getTree().isVisible()) {
																				filePartObject.getTreeViewer().refresh(file);
																			} else {
																				filePartObject.getTableViewer().refresh(file);
																			}
																		}
																	});																
																	
																	monitor.worked(1);
																}
															} catch (Exception e) {
																logger.error("Error while applying the new ACL inheritance: " + e.getMessage(), e);
																SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while applying the new ACL inheritance, make sure you run Permission Analyzer as Administrator: \r\n\r\n" + GUILogHandler.getNestedMessage(e));
															}
														}
													});
												} catch (InvocationTargetException | InterruptedException e) {
													SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while applying ACL inheritance: " + e.getMessage());
													logger.error("Error while applying ACL inheritance", e);
												}
											}
										}
										
									} catch (Exception ex) {
										logger.error("Error while changing the ACL inheritance", ex);
										SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while changing the ACL inheritance: " + ex.getMessage());
									}
								}
							});
							
						} catch (Exception e) {
							logger.error("Error while applying the new owner: " + e.getMessage(), e);
							SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while applying the new ACL inheritance: " + GUILogHandler.getNestedMessage(e));
						}
					}
				});
			} catch (InvocationTargetException | InterruptedException e) {
				SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while applying ACL inheritance: " + e.getMessage());
				logger.error("Error while applying owner", e);
			}
			
		} else {
			// Change the owner of one particular file
			if (file != null) {
				try {
					final Path path = Paths.get(file.determineFullPath());
					
					// get the security descriptor for this file
					// NOTE: The Advapi32Util.getFileSecurityDescriptor method has been modified! In case the path startsWith \\ and 
					// the length is longer than 248 characters it will convert to path to a short notation (~1 etc). Otherwise the 
					// getNamedSecurityInfo will throw an error "The filename, directory name, or volume label syntax is incorrect."
					final WinNT.SECURITY_DESCRIPTOR_RELATIVE secDescriptor = Advapi32Util.getFileSecurityDescriptor(path.toFile(), false);;
					
					// clear the SE_DACL_PROTECTED bit
					secDescriptor.Control &= ~SE_DACL_PROTECTED;
					
					Advapi32Util.setFileSecurityDescriptor(path.toFile(), secDescriptor, false, false, false, false, true, false);
					
					file.setDaclProtected(false);
					file.save();
					Thread.sleep(800);
					
					Display.getDefault().syncExec(new Runnable(){
						public void run() {
							// refresh the UI
							if (filePartObject.getTreeViewer().getTree().isVisible()) {
								filePartObject.getTreeViewer().refresh(file);
							} else {
								filePartObject.getTableViewer().refresh(file);
							}
						}
					});		
					
				} catch (Exception ex) {
					logger.error("Error while changing the ACL inheritance", ex);
					SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while changing the ACL inheritance, make sure you run Permission Analyzer as Administrator: \r\n\r\n" + ex.getMessage());
				}		
			}
		}		
	}
	
	@CanExecute
	public boolean canExecute(EPartService partService, ESelectionService selectionService,
			@Optional @Named("com.perdemia.permissionanalyzer.application.commands.changeaclinheritance.param.allfiles") final String allFiles){
		
		StructuredSelection selection = (StructuredSelection) selectionService.getSelection(FilePart.ID);
		FileInfo file = (FileInfo) selection.getFirstElement();
		
		return allFiles.equals("true") || (file != null && file.isDaclProtected());
	}
}
