package com.perdemia.permissionanalyzer.application.handlers.menu;

import javax.inject.Named;

import org.eclipse.core.commands.ParameterizedCommand;
import org.eclipse.e4.core.commands.ECommandService;
import org.eclipse.e4.core.commands.EHandlerService;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.export.ExportHelper;
import com.perdemia.permissionanalyzer.application.jface.dialogs.ReportDialog;
import com.perdemia.permissionanalyzer.application.parts.FilterFolderPart;
import com.perdemia.permissionanalyzer.application.parts.FilterMemberPart;
import com.perdemia.permissionanalyzer.application.parts.FilterPermissionPart;
import com.perdemia.permissionanalyzer.application.util.FilterHelper;
import com.perdemia.permissionanalyzer.application.util.LicenseHelper;
import com.perdemia.permissionanalyzer.application.util.LicenseHelper.Edition;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.Member;
import com.perdemia.permissionanalyzer.data.Report;

public class ManageReportsHandler {
	private static Logger logger = LoggerFactory.getLogger(ManageReportsHandler.class.getName());
		
	@Execute
	public void execute(Shell shell, MApplication application, EPartService partService, ECommandService commandService, EHandlerService handlerService,
			@Optional @Named("com.perdemia.permissionanalyzer.application.commands.managereports.param.ispolicy") final String isPolicy){
		ReportDialog dialog = new ReportDialog(shell, isPolicy != null, false);
    	int result = dialog.open();
    	
    	Report selectedReport = dialog.getSelectedReport();
    	
    	if (result == ReportDialog.LOAD_REPORT_ID) {
    		MPart part = partService.findPart(FilterMemberPart.ID);
    		partService.activate(part, false); // make sure the Part is loaded, otherwise the getObject() will be null!
    		FilterMemberPart filterMemberPart = (FilterMemberPart) part.getObject();
    		
    		try {
				if (selectedReport.getMemberFilterSet() != null) {
					filterMemberPart.loadNewFilterSet(selectedReport.getMemberFilterSet());
				} else {
					filterMemberPart.loadNewFilters(selectedReport.getMemberFilters());
				}
				
				part = partService.findPart(FilterPermissionPart.ID);
				partService.activate(part, false); // make sure the Part is loaded, otherwise the getObject() will be null!
				FilterPermissionPart filterPermissionPart = (FilterPermissionPart) part.getObject();
				if (selectedReport.getPermissionFilterSet() != null) {
					filterPermissionPart.loadNewFilterSet(selectedReport.getPermissionFilterSet());
				} else {
					filterPermissionPart.loadNewFilters(selectedReport.getPermissionFilters());
				}
				
				part = partService.findPart(FilterFolderPart.ID);
				partService.activate(part, false); // make sure the Part is loaded, otherwise the getObject() will be null!
				FilterFolderPart filterFolderPart = (FilterFolderPart) part.getObject();
				if (selectedReport.getFolderFilterSet() != null) {
					filterFolderPart.loadNewFilterSet(selectedReport.getFolderFilterSet());
				} else {
					filterFolderPart.loadNewFilters(selectedReport.getFolderFilters());
				}
    		} catch (DBException ex) {
    			logger.error("Error while loading the report filter sets from the database", ex);
				MessageDialog.openError(null, "Error", "Error while loading the report filter sets from the database: " + ex.getMessage());
    		}
    		
			part = partService.findPart(FilterMemberPart.ID);
    		partService.activate(part);
    		
    		
			FilterHelper.getInstance().setSelectedReport(dialog.getSelectedReport());
    		
    		// refresh the data to display the report data
			ParameterizedCommand applyFilterCommand = commandService.createCommand("com.perdemia.permissionanalyzer.application.commands.applyfilter", null);
			handlerService.executeHandler( applyFilterCommand );
    					
    		// get the menu item "Save report" and change the label
    		/* Label change is not reflected in the GUI menu :(
    		List<MMenuContribution> menus = application.getMenuContributions();
    		for (MMenuContribution menu : menus) {
    			if (menu.getElementId().equals("com.perdemia.permissionanalyzer.application.menu.main")) {
    				for (MMenuElement menuItem : menu.getChildren()) {
    					if (menuItem.getElementId().equals("com.perdemia.permissionanalyzer.application.menu.report")) {
    						MMenu reportMenu = (MMenu) menuItem;
    						for (MMenuElement reportMenuItem : reportMenu.getChildren()) {
    							if (reportMenuItem.getElementId().equals("com.perdemia.permissionanalyzer.application.menu.savereport")) {
    								reportMenuItem.setLabel("Save report " + dialog.getSelectedReport().getName());
    								reportMenuItem.updateLocalization();
    	    					}
    						}
    					}    					
    				}
    			}
    		}*/
    		
    	}
	}
	
	@CanExecute
	public boolean canExecute() {
		return Member.isUserGroupCountAllowed() && !LicenseHelper.getEdition().equals(Edition.SCAN_AGENT);
	}
}
