package com.perdemia.permissionanalyzer.application.handlers.toolbar;

import gnu.trove.list.array.TIntArrayList;

import java.util.List;

import com.perdemia.permissionanalyzer.application.parts.FilePart;
import com.perdemia.permissionanalyzer.application.swt.TreeViewerWithMonitor;
import com.perdemia.permissionanalyzer.application.util.FilterHelper;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.FileInfo;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.viewers.AbstractTreeViewer;
import org.eclipse.jface.viewers.ILazyTreeContentProvider;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.swt.widgets.Display;

public class ExpandAllHandler {
	
	@Execute
	public void execute(MApplication app, EPartService partService, EModelService modelService){
		MPart part = partService.findPart(FilePart.ID);
		final FilePart fileTreePart = (FilePart) part.getObject();
		final TreeViewerWithMonitor viewer = fileTreePart.getTreeViewer();
				
			// configure a progress monitor dialog
			final ProgressMonitorDialog progressDialog = new ProgressMonitorDialog(Display.getCurrent().getActiveShell());
			progressDialog.setBlockOnOpen(true);
			progressDialog.setCancelable(true);
			viewer.setMonitor(progressDialog.getProgressMonitor());
								
			// start the expanding process
			Display.getCurrent().asyncExec(new Runnable(){
				@Override
				public void run() {
					// now determine how many items we are going to expand
					int numberOfItemsToExpand = 0;
					
					if (fileTreePart.getTreeViewer().getTree().getSelection().length > 0) {
						FileInfo selectedFile = (FileInfo)((ITreeSelection)viewer.getSelection()).getFirstElement();
						
						// in order to use the progress bar we need to know how many tree items we are about to expand
						// we have a list of all file ids in the tree, so we need to know the file id of the next sibling in the tree
						// the number of files between the selected file id and the next sibling file id is the number of tree items to display
						
						// determine the next sibling (when there is no next sibling use the sibling of the parent!)
						TIntArrayList allFileIds = FilterHelper.getInstance().getApplicableFileIdsIncludingParents();
						TreePath path = ((ITreeSelection)viewer.getSelection()).getPaths()[0];
						int segmentIndex = path.getSegmentCount() - 1;
						int expandUntilFileId = -1;
						while (expandUntilFileId < 0) {
							int indexOf;
							FileInfo file = (FileInfo) path.getSegment(segmentIndex);
							
							// reached root element?
							if (segmentIndex == 0) {
								@SuppressWarnings("unchecked")
								List<FileInfo> rootFiles = (List<FileInfo>) ((ILazyTreeContentProvider)viewer.getContentProvider()).getParent(null); 
								indexOf = rootFiles.indexOf(file);
								
								// last root file?
								if (indexOf == rootFiles.size() - 1) {
									// we have reached the last root file, meaning we expand util the end of the tree
									expandUntilFileId = Integer.MAX_VALUE;
								} else {
									// expand until the next root file
									expandUntilFileId = rootFiles.get(indexOf + 1).getId();
								}
							} else {
								
								FileInfo parentFile = (FileInfo) path.getSegment(segmentIndex - 1);
								indexOf = parentFile.getChildren().indexOf(file);
								
								// is there a next sibling?
								if (indexOf != parentFile.getChildren().size() - 1) {
									// expand the tree until the next sibling
									expandUntilFileId = parentFile.getChildren().get(indexOf + 1).getId();
								} else {
									// the current file is the last sibling, try the parent
								}
							}
							
							// no next sibling found, try the next parent
							segmentIndex--;
						}
												
						// do we have a filtered list with file id?
						if (allFileIds != null) {
							int selectedFileIndex = 0;
							int expandUntilIndex = 0;
							for (int counter = 0; counter < allFileIds.size(); counter++) {
								if (allFileIds.get(counter) == selectedFile.getId()) {
									selectedFileIndex = counter;
									
									// is expandUntilFileId the last file id? Then store the index and break
									if (expandUntilFileId == Integer.MAX_VALUE) {
										expandUntilIndex = allFileIds.size() - 1;
										break;
									}
								}
								
								if (allFileIds.get(counter) == expandUntilFileId) {
									expandUntilIndex = counter;
									break;
								}
							}
							numberOfItemsToExpand = expandUntilIndex - selectedFileIndex;
						} else {
							// no filter has been applied, just use the difference between the selected file id and the expandUntilFileId
							try {
								if (expandUntilFileId == Integer.MAX_VALUE) {
									expandUntilFileId = FileInfo.getCount();
								}
								numberOfItemsToExpand = expandUntilFileId - selectedFile.getId();
							} catch (DBException e) {
								numberOfItemsToExpand = IProgressMonitor.UNKNOWN;
							}
						}
					} else {
						// no item is selected, just expand all the items
						if (FilterHelper.getInstance().getApplicableFileIdsIncludingParents() != null) {
							numberOfItemsToExpand = FilterHelper.getInstance().getApplicableFileIdsIncludingParents().size();
						} else {
							try {
								numberOfItemsToExpand = FileInfo.getCount();
							} catch (DBException e) {
								numberOfItemsToExpand = IProgressMonitor.UNKNOWN;
							}
						}
					}
					
					progressDialog.getProgressMonitor().beginTask("Expanding " + (numberOfItemsToExpand != -1 ? numberOfItemsToExpand + " " : "") + "tree nodes...", numberOfItemsToExpand);
				
					viewer.getTree().setRedraw(false);
					
					if (fileTreePart.getTreeViewer().getTree().getSelection().length > 0) {
						viewer.expandToLevel(((ITreeSelection)viewer.getSelection()).getFirstElement(), AbstractTreeViewer.ALL_LEVELS);
					} else {
						viewer.expandAll();
					}
					viewer.getTree().setRedraw(true);
					progressDialog.close();
					
				}
			});
			
			// the expanding has started async, now open the progress dialog
			progressDialog.open();
		
	}
	
	@CanExecute
	public boolean canExecute(EPartService partService){
		MPart part = partService.findPart(FilePart.ID);
		FilePart fileTreePart = (FilePart) part.getObject();
		return fileTreePart.getTreeViewer().getTree().isVisible();
	}
}
