package com.perdemia.permissionanalyzer.application.jface.dialogs;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

import com.perdemia.permissionanalyzer.application.Activator;
import com.perdemia.permissionanalyzer.application.swt.ImageComposite;
import com.perdemia.permissionanalyzer.application.util.ApplicationUtil;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;

public class AboutDialog extends DialogWithTopBar {
	
	public AboutDialog(Shell shell) {
		super(shell);
	}

	@Override
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setImage(ImageFactory.getInstance().getImage(AppImage.APP_ICON_16X16));
		shell.setSize(450, 420);
		shell.setText("Permission Analyzer");
	}
	
	@Override
	protected Control createDialogArea(Composite container) {
		Composite composite = (Composite) super.createDialogArea(container, "Permission Analyzer", ImageFactory.getInstance().getImage(AppImage.APP_ICON_48X48));
		container.setBackgroundMode(SWT.INHERIT_DEFAULT);
				
		GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.marginTop = 30;
		gridLayout.marginLeft = 30;
		gridLayout.marginRight = 30;
		composite.setLayout(gridLayout);
		
		Label descriptionLabel = new Label(composite, SWT.NONE);
		GridData gridData = new GridData();
		gridData.horizontalSpan = 2;
		gridData.heightHint = 100;
		descriptionLabel.setLayoutData(gridData);
		descriptionLabel.setText("Protect your data and get in control!\r\nScan your network, filter NTFS permissions,\r\nvalidate your access control design and\r\ntrace user or group access.");
		
		Label versionLabel = new Label(composite, SWT.NONE);
		versionLabel.setText("Version:");
		gridData = new GridData();
		gridData.widthHint = 80;
		versionLabel.setLayoutData(gridData);
		
		Label versionValueLabel = new Label(composite, SWT.NONE);
		versionValueLabel.setText(Activator.getContext().getBundle().getVersion().toString());
		
		Label websiteLabel = new Label(composite, SWT.NONE);
		websiteLabel.setText("Website:");
		gridData = new GridData();
		gridData.widthHint = 80;
		websiteLabel.setLayoutData(gridData);
		
		Link websiteLink = new Link(composite, SWT.NONE);
		websiteLink.setText("<a>http://www.perdemia.com</a>");
		websiteLink.addListener(SWT.Selection,new Listener(){
		    public void handleEvent(Event event) {
		    	ApplicationUtil.setBusyCursor();
		    	Program.launch(event.text);
		    	ApplicationUtil.setDefaultCursor();
		    }
		});
		
		Label emailLabel = new Label(composite, SWT.NONE);
		emailLabel.setText("E-mail:");
		gridData = new GridData();
		gridData.widthHint = 80;
		emailLabel.setLayoutData(gridData);
		
		Link emailLink = new Link(composite, SWT.NONE);
		emailLink.setText("<a>supportdesk@perdemia.com</a>");
		emailLink.addListener(SWT.Selection,new Listener(){
		    public void handleEvent(Event event) {
		    	ApplicationUtil.setBusyCursor();
		    	Program.launch("mailto:" + event.text);
		    	ApplicationUtil.setDefaultCursor();
		    }
		});
		
		return composite;
	}

	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		GridLayout gridLayout = (GridLayout) parent.getLayout();
		gridLayout.makeColumnsEqualWidth = false;
		((GridData)parent.getLayoutData()).horizontalAlignment = GridData.FILL;
		
		ImageComposite logo = new ImageComposite(parent, ImageFactory.getInstance().getImage(AppImage.LOGO));
		((GridLayout) parent.getLayout()).numColumns++; //also triggered by the createButton calls
		
		// filler
		Composite fillComposite = new Composite(parent, SWT.NONE);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.heightHint = 10;
		fillComposite.setLayoutData(gridData);
		((GridLayout) parent.getLayout()).numColumns++; //also triggered by the createButton calls
		
		createButton(parent, IDialogConstants.CANCEL_ID, "Close", true);
	}
}
