package com.perdemia.permissionanalyzer.application.jface.dialogs;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import com.perdemia.permissionanalyzer.application.util.ColorFactory;
import com.perdemia.permissionanalyzer.application.util.Constants;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ColorFactory.AppColor;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;

public class ChangeOwnerDialog extends DialogWithTopBar {
	private Text accountNameText;
	private String accountName;
	private int fileCount;
	
	public ChangeOwnerDialog(Shell shell, int fileCount) {
		super(shell);
		this.fileCount = fileCount;
	}

	@Override
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setImage(ImageFactory.getInstance().getImage(AppImage.ACCESS_CONTROL_16X16));
		shell.setSize(600, 300);
		shell.setText("Change file owner");
	}
	
	@Override
	protected Control createDialogArea(Composite container) {
		Composite composite = (Composite) super.createDialogArea(container, "Change file owner", ImageFactory.getInstance().getImage(AppImage.ACCESS_CONTROL_48X48));
		container.setBackgroundMode(SWT.INHERIT_DEFAULT);
				
		GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.marginTop = 10;
		gridLayout.marginLeft = 7;
		gridLayout.marginRight = 7;
		composite.setLayout(gridLayout);
		
		if (fileCount > 1) {
			Composite trialComposite = new Composite(composite, SWT.NONE);
			GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
			gridData.horizontalSpan = 2;
			gridData.heightHint = 40;
			trialComposite.setLayoutData(gridData);
			gridLayout = new GridLayout(1, false);	
			trialComposite.setLayout(gridLayout);
			trialComposite.setBackground(ColorFactory.getInstance().getColor(AppColor.TRIAL_BANNER));
			
			Label trialLabel = new Label(trialComposite, SWT.NONE);
			trialLabel.setText("Warning: you are about to change the owner of " + fileCount + " files/directories");
			gridData = new GridData(GridData.FILL_HORIZONTAL);
			gridData.horizontalIndent = 20;
			gridData.verticalAlignment = SWT.CENTER;
			gridData.grabExcessVerticalSpace = true;
			trialLabel.setLayoutData(gridData);
			trialLabel.setData(Constants.CSS_CLASS_NAME_KEY, "h2");
		}
		
		Label label = new Label(composite, SWT.NONE);
		label.setText("Please enter the account name of the new owner:");
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.heightHint = 30;
		
		if (fileCount > 1) {
			gridData.verticalIndent = 10;
		}
		
		gridData.horizontalSpan = 2;
		label.setLayoutData(gridData);
		
		accountNameText = new Text(composite, SWT.SINGLE | SWT.BORDER);
		accountNameText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		Button browseAccount = new Button(composite, SWT.PUSH);
		browseAccount.setImage(ImageFactory.getInstance().getImage(AppImage.SEARCH_16X16));
		browseAccount.setToolTipText("Search for an account name");
		browseAccount.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				SearchMembersDialog dialog = new SearchMembersDialog(getShell(), null);
				dialog.setTitle("Select a new owner for the file(s)");
				dialog.setDescription("Search users and groups (use * as wildcard):");
				if (dialog.open() == IDialogConstants.OK_ID) {
					accountNameText.setText(dialog.getSelectedMember().getName(true));
				}
			}
		});
		
		return composite;
	}

	protected void buttonPressed(int buttonId) {
		this.accountName = accountNameText.getText();		
		super.buttonPressed(buttonId);
	}

	public String getAccountName() {
		return accountName;
	}

	public void setAccountName(String accountName) {
		this.accountName = accountName;
	}
}
