package com.perdemia.permissionanalyzer.application.jface.dialogs;

import org.apache.commons.mail.EmailException;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.handlers.menu.OpenLogFileHandler;
import com.perdemia.permissionanalyzer.application.logging.GUILogHandler;
import com.perdemia.permissionanalyzer.application.util.ApplicationUtil;
import com.perdemia.permissionanalyzer.application.util.Attachment;
import com.perdemia.permissionanalyzer.application.util.Constants;
import com.perdemia.permissionanalyzer.application.util.EmailHelper;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;

public class ContactSupportDialog extends DialogWithTopBar {
	private static Logger logger = LoggerFactory.getLogger(ContactSupportDialog.class.getName());
	private Text fromEmailAddressText;
	private Text subjectText;
	private Text contentText;
	private Button sendLogFileButton;
	
	public ContactSupportDialog(Shell parentShell) {
		super(parentShell);
	}

	@Override
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText("Contact support");
		newShell.setSize(1200, 800);
		newShell.setImage(ImageFactory.getInstance().getImage(AppImage.APP_ICON_16X16));
	}
	
	@Override
	protected boolean isResizable()  {
	  return true;
	}
	
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite composite = (Composite) super.createDialogArea(parent, "Contact support", ImageFactory.getInstance().getImage(AppImage.APP_ICON_48X48));
		parent.setBackgroundMode(SWT.INHERIT_DEFAULT);
		parent.setData(Constants.CSS_CLASS_NAME_KEY, "MPart");
		
		GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.marginTop = 10;
		gridLayout.marginLeft = 10;
		gridLayout.marginRight = 10;
		composite.setLayout(gridLayout);		
		
		Label emailFromLabel = new Label(composite, SWT.NONE);
		emailFromLabel.setText("Your e-mail address:   ");
		fromEmailAddressText = new Text(composite, SWT.BORDER);
		fromEmailAddressText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		Label subjectLabel = new Label(composite, SWT.NONE);
		subjectLabel.setText("Subject:");
		subjectText = new Text(composite, SWT.BORDER);
		subjectText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		Label messageLabel = new Label(composite, SWT.NONE);
		messageLabel.setText("Message:");
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		messageLabel.setLayoutData(gridData);
		
		contentText = new Text(composite, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL | SWT.H_SCROLL);
		contentText.setText("Dear support,\r\n\r\n");
		gridData = new GridData(GridData.FILL_BOTH);
		gridData.horizontalSpan = 2;
		contentText.setLayoutData(gridData);
		
		sendLogFileButton = new Button(composite, SWT.CHECK);
		sendLogFileButton.setText("Zip the application log file as attachment in the e-mail");
		sendLogFileButton.setSelection(true);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		sendLogFileButton.setLayoutData(gridData);
		
		return composite;		
	}
		
	@Override
	protected void buttonPressed(int buttonId) {
		if (buttonId == IDialogConstants.OK_ID) {
			Attachment attachment = null;
			
			if (sendLogFileButton.getSelection()) {
				attachment = new Attachment();
				attachment.setName("Log file");
				attachment.setDescription("Log file");
				attachment.setPath(OpenLogFileHandler.getLogFile().getAbsolutePath());
			}
			
			// send the e-mail
			try {
				ApplicationUtil.setBusyCursor();
				EmailHelper.sendMail(subjectText.getText(), contentText.getText(), fromEmailAddressText.getText(), "supportdesk@permissionanalyzer.com", attachment);
				MessageDialog.openInformation(getShell(), "E-mail sent", "Your e-mail has been sent.");
				
			} catch (EmailException e) {
				logger.error("Error while sending e-mail to support.", e);
				
				String logFile = "";
				try {
					logFile = OpenLogFileHandler.getLogFile().getAbsolutePath();
					logFile = ", the log file can be found at " + logFile; 
				} catch (Exception ex) {}
				
				MessageDialog.openError(getShell(), "Error", "Error while sending the e-mail to support: " + GUILogHandler.getNestedMessage(e) + "\r\n\r\nPlease send an e-mail manually to supportdesk@permissionanalyzer.com" + logFile);
				return;
				
			} finally {
				ApplicationUtil.setDefaultCursor();
			}
		}
		
		close();
	}
}
