package com.perdemia.permissionanalyzer.application.jface.dialogs;

import com.perdemia.permissionanalyzer.application.swt.ImageComposite;
import com.perdemia.permissionanalyzer.application.util.Constants;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Monitor;
import org.eclipse.swt.widgets.Shell;

public abstract class DialogWithTopBar extends Dialog {
	private Label titleLabel;
	private ImageComposite imageComposite;
	
	protected DialogWithTopBar(Shell parentShell) {
		super(parentShell);
	}

	protected Control createDialogArea(Composite container, String title, Image image) {		
		centerDialog(container.getShell());
		
		// top bar
		Composite topBar = new Composite(container, SWT.NONE);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.heightHint = 70;
		topBar.setLayoutData(gridData);
		topBar.setData(Constants.CSS_CLASS_NAME_KEY, "dialog-top-bar");
		GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.marginTop = 5;
		gridLayout.marginLeft = 25;
		gridLayout.horizontalSpacing = 30;
		topBar.setLayout(gridLayout);
		
		imageComposite = new ImageComposite(topBar, image);
		titleLabel = new Label(topBar, SWT.NONE);
		titleLabel.setText(title);
		
		// area below the top bar
		Composite composite = new Composite(container, SWT.NONE);
		composite.setLayoutData(new GridData(GridData.FILL_BOTH));
		return composite;
	}
	
	protected void centerDialog(Shell shell) {
		// center the dialog
		Monitor monitor = Display.getCurrent().getPrimaryMonitor();
		Rectangle monitorRect = monitor.getBounds();
		Rectangle shellRect = shell.getBounds();
		int x = monitorRect.x + (monitorRect.width - shellRect.width) / 2;
		int y = monitorRect.y + (monitorRect.height - shellRect.height) / 2;
		shell.setLocation(x, y);
	}
	
	protected void setTitle(String title) {
		titleLabel.setText(title);
		titleLabel.pack(true);
	}
	
	protected void setImage(Image image) {
		imageComposite.setImage(image);
	}
}
