package com.perdemia.permissionanalyzer.application.jface.dialogs;

import java.util.List;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.model.DirectoryInfo;
import com.perdemia.permissionanalyzer.application.model.DirectoryInfo.SourceType;
import com.perdemia.permissionanalyzer.application.util.ColorFactory;
import com.perdemia.permissionanalyzer.application.util.ColorFactory.AppColor;
import com.perdemia.permissionanalyzer.application.util.Constants;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.application.util.LicenseHelper;
import com.perdemia.permissionanalyzer.application.util.LicenseHelper.Edition;
import com.perdemia.permissionanalyzer.scanner.acl.AclScanner;

public class DirectoryInfoDialog extends DialogWithTopBar {
	private static Logger logger = LoggerFactory.getLogger(DirectoryInfoDialog.class.getName());
	
	private Text directoryPathText;
	private Spinner numberSubdirectoriesSpinner;
	private Button includeFilesButton;
	private Button includeInheritedPermissionsButton;
	private Button followSymbolicLinksButton;
	private Button scanLocalGroupsButton;
	
	/** UI controls for SourceType SERVER */
	private Button includeHiddenSharedDrivesButton;
	private Button includeHiddenSharesButton;
	private Text excludeSharesText;
	
	private DirectoryInfo dirInfo;
	private List<DirectoryInfo> existingDirectoyInfoList;
	
	public DirectoryInfoDialog(Shell shell, DirectoryInfo dirInfo, List<DirectoryInfo> existingDirectoyInfoList) {
		super(shell);
		this.dirInfo = dirInfo;
		this.existingDirectoyInfoList = existingDirectoyInfoList;
	}
	
	@Override
	protected boolean isResizable() {
		return true;
	}

	@Override
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		
		int heightIncrement = LicenseHelper.getEdition().equals(Edition.TRIAL) ? 50 : 0;
				
		if (dirInfo.getSourceType().equals(SourceType.SERVER)) {
			shell.setImage(ImageFactory.getInstance().getImage(AppImage.SERVER_16X16));
			shell.setText("Server details");
			shell.setSize(800, 600 + heightIncrement);
			
		} else if (dirInfo.getSourceType().equals(SourceType.DIRECTORY)) {
			shell.setImage(ImageFactory.getInstance().getImage(AppImage.FOLDER_16X16));
			shell.setText("Directory or network share details");
			shell.setSize(800, 520 + heightIncrement);
			
		} else if (dirInfo.getSourceType().equals(SourceType.POWER_SHELL)) {
			shell.setImage(ImageFactory.getInstance().getImage(AppImage.POWER_SHELL_16X16));
			shell.setText("PowerShell ACL output file");
			shell.setSize(800, 500 + heightIncrement);
			
		} else if (dirInfo.getSourceType().equals(SourceType.EMC_ISILON)) {
			shell.setImage(ImageFactory.getInstance().getImage(AppImage.FILE_TXT_READ_16X16));
			shell.setText("EMC Isilon ACL output file");
			shell.setSize(800, 500 + heightIncrement);
		}
	}
	
	@Override
	protected Control createDialogArea(final Composite container) {
		Composite composite = null;
		
		if (dirInfo.getSourceType().equals(SourceType.SERVER)) {
			composite = (Composite) super.createDialogArea(container, "Server details", ImageFactory.getInstance().getImage(AppImage.SERVER_48X48));
			
		} else if (dirInfo.getSourceType().equals(SourceType.DIRECTORY)) {
			composite = (Composite) super.createDialogArea(container, "Directory or network share details", ImageFactory.getInstance().getImage(AppImage.FOLDER_48X48));
			
		} else if (dirInfo.getSourceType().equals(SourceType.POWER_SHELL)) {
			composite = (Composite) super.createDialogArea(container, "PowerShell ACL output file", ImageFactory.getInstance().getImage(AppImage.POWER_SHELL_48X48));
			
		} else if (dirInfo.getSourceType().equals(SourceType.EMC_ISILON)) {
			composite = (Composite) super.createDialogArea(container, "EMC Isilon ACL output file", ImageFactory.getInstance().getImage(AppImage.TEXT_FILE_48X48));
		}
		container.setBackgroundMode(SWT.INHERIT_DEFAULT);
				
		GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.marginTop = 10;
		gridLayout.marginLeft = 7;
		gridLayout.marginRight = 7;
		composite.setLayout(gridLayout);
		
		if (LicenseHelper.getEdition().equals(Edition.TRIAL)) {
			Composite trialComposite = new Composite(composite, SWT.NONE);
			GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
			gridData.heightHint = 40;
			gridData.horizontalSpan = 2;
			trialComposite.setLayoutData(gridData);
			gridLayout = new GridLayout(1, false);
			trialComposite.setLayout(gridLayout);
			trialComposite.setBackground(ColorFactory.getInstance().getColor(AppColor.TRIAL_BANNER));
			
			Label trialLabel = new Label(trialComposite, SWT.NONE);
			trialLabel.setText("The Trial version of Permission Analyzer has a maximum depth of " + AclScanner.getNumberOfSubDirsForTrial() + " subdirectories.");
			gridData = new GridData(GridData.FILL_HORIZONTAL);
			gridData.horizontalIndent = 20;
			gridData.verticalAlignment = SWT.CENTER;
			gridData.grabExcessVerticalSpace = true;
			trialLabel.setLayoutData(gridData);
			trialLabel.setData(Constants.CSS_CLASS_NAME_KEY, "h2");
		}
		
		Label label = new Label(composite, SWT.WRAP);
		if (dirInfo.getSourceType().equals(SourceType.SERVER)) {
			label.setText("All the network shares of the server will be scanned:");
		} else if (dirInfo.getSourceType().equals(SourceType.DIRECTORY)) {
			label.setText("Add the path to a local directory or a particular network share to scan:");
		} else if (dirInfo.getSourceType().equals(SourceType.POWER_SHELL)) {
			label.setText("The raw ACL text file (or zip file) generated by a PowerShell script will be scanned and processed in the database:");
		} else if (dirInfo.getSourceType().equals(SourceType.EMC_ISILON)) {
			label.setText("The raw ACL text file (or zip file) generated by the EMC Isilon server will be scanned and processed in the database:");
		}
		
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		gridData.heightHint = 30;
		
		if (LicenseHelper.getEdition().equals(Edition.TRIAL)) {
			gridData.verticalIndent = 15;
		}
		
		label.setLayoutData(gridData);
		
		Label directoryPathLabel = new Label(composite, SWT.NONE);
		
		if (dirInfo.getSourceType().equals(SourceType.SERVER)) {
			directoryPathLabel.setText("Server name:");
		} else if (dirInfo.getSourceType().equals(SourceType.DIRECTORY)) {
			directoryPathLabel.setText("Directory path or network share:");	
		} else if (dirInfo.getSourceType().equals(SourceType.POWER_SHELL)) {
			directoryPathLabel.setText("File path:");
		} else if (dirInfo.getSourceType().equals(SourceType.EMC_ISILON)) {
			directoryPathLabel.setText("File path:");
		}
					
		Composite browseDirectoryComposite = new Composite(composite, SWT.NONE);
		browseDirectoryComposite.setLayout(new GridLayout(4, false));
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalIndent = -5;
		browseDirectoryComposite.setLayoutData(gridData);
		directoryPathText = new Text(browseDirectoryComposite, SWT.BORDER);
		directoryPathText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		directoryPathText.setText(dirInfo.getPath());
		
		if (!dirInfo.getSourceType().equals(SourceType.SERVER)) {
			Button browseButton = new Button(browseDirectoryComposite, SWT.PUSH);
			browseButton.setLayoutData(new GridData());
			
			if (dirInfo.getSourceType().equals(SourceType.DIRECTORY)) {
				browseButton.setImage(ImageFactory.getInstance().getImage(AppImage.BROWSE_16X16));
				browseButton.setToolTipText("Browse directory");
			} else {
				browseButton.setImage(ImageFactory.getInstance().getImage(AppImage.FILE_TXT_READ_16X16));
				browseButton.setToolTipText("Browse file");
			}
			
			browseButton.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					String path;
					if (dirInfo.getSourceType().equals(SourceType.DIRECTORY)) {
						DirectoryDialog browseDialog = new DirectoryDialog(container.getShell());
						path = browseDialog.open();
					} else {
						FileDialog browseDialog = new FileDialog(getShell(), SWT.OPEN);
						path = browseDialog.open();
					}
					
					if (path != null) {
						directoryPathText.setText(path);
					}
				}
			});
		}
		
		Label numberSubdirectoriesLabel = new Label(composite, SWT.NONE);
		numberSubdirectoriesLabel.setText("Subdirectories depth:");
		Composite numberSubdirectoriesComposite = new Composite(composite, SWT.NONE);
		numberSubdirectoriesComposite.setLayoutData(new GridData());
		gridLayout = new GridLayout(2, false);
		gridLayout.horizontalSpacing = 10;
		gridLayout.marginLeft = -5;
		gridLayout.marginTop = -5;
		gridLayout.marginBottom = -5;
		numberSubdirectoriesComposite.setLayout(gridLayout);
		numberSubdirectoriesSpinner = new Spinner(numberSubdirectoriesComposite, SWT.BORDER);
		numberSubdirectoriesSpinner.setLayoutData(new GridData());
		numberSubdirectoriesSpinner.setMinimum(0);
		numberSubdirectoriesSpinner.setTextLimit(2);
		
		// trial limit?
		int subdirs = dirInfo.getNumberOfSubdirectories();
		if (LicenseHelper.getEdition().equals(Edition.TRIAL) && (subdirs == 0 || subdirs > 5)) {
			subdirs = 5;
		}
		
		numberSubdirectoriesSpinner.setSelection(subdirs);
		numberSubdirectoriesSpinner.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(ModifyEvent e) {
				if (LicenseHelper.getEdition().equals(Edition.TRIAL)) {
					if (numberSubdirectoriesSpinner.getSelection() == 0 || numberSubdirectoriesSpinner.getSelection() > AclScanner.getNumberOfSubDirsForTrial()) {
						SyncMessageDialog.showMessage(MessageDialog.ERROR, "Directory limit", "The trial version of Permission Analyzer has a maximum depth of " + AclScanner.getNumberOfSubDirsForTrial() + " subdirectories.");
						numberSubdirectoriesSpinner.setSelection(AclScanner.getNumberOfSubDirsForTrial());
					}
				}
			}
		});		
		
		Label numberSubdirectoriesButtonLabel = new Label(numberSubdirectoriesComposite, SWT.NONE);
		numberSubdirectoriesButtonLabel.setLayoutData(new GridData());
		numberSubdirectoriesButtonLabel.setText("(0 means unlimited)");
				
		Label includeFilesLabel = new Label(composite, SWT.NONE);
		includeFilesLabel.setText("Include files:");
		includeFilesButton = new Button(composite, SWT.CHECK);
		includeFilesButton.setLayoutData(new GridData());
		includeFilesButton.setSelection(dirInfo.isIncludeFiles());
		
		Label includeInheritedPermissionsLabel = new Label(composite, SWT.NONE);
		includeInheritedPermissionsLabel.setText("Include inherited permissions:    ");
		includeInheritedPermissionsButton = new Button(composite, SWT.CHECK);
		includeInheritedPermissionsButton.setLayoutData(new GridData());
		includeInheritedPermissionsButton.setSelection(dirInfo.isIncludeInheritedPermissions());
		
		if (dirInfo.getSourceType().equals(SourceType.DIRECTORY) || dirInfo.getSourceType().equals(SourceType.SERVER)) {
			Label followSymbolicLinksLabel = new Label(composite, SWT.NONE);
			followSymbolicLinksLabel.setText("Follow symbolic links:");
			followSymbolicLinksButton = new Button(composite, SWT.CHECK);
			followSymbolicLinksButton.setLayoutData(new GridData());
			followSymbolicLinksButton.setSelection(dirInfo.isFollowSymbolicLinks());
			
			Label scanLocalGroupsLabel = new Label(composite, SWT.NONE);
			scanLocalGroupsLabel.setText("Scan local groups on this server:    ");
			scanLocalGroupsButton = new Button(composite, SWT.CHECK);
			scanLocalGroupsButton.setLayoutData(new GridData());
			scanLocalGroupsButton.setSelection(dirInfo.isScanLocalGroups());
		}
		
		if (dirInfo.getSourceType().equals(SourceType.SERVER)) {
			Label includeHiddenSharesLabel = new Label(composite, SWT.NONE);
			includeHiddenSharesLabel.setText("Include hidden shares (ending with $):");
			includeHiddenSharesButton = new Button(composite, SWT.CHECK);
			includeHiddenSharesButton.setLayoutData(new GridData());
			includeHiddenSharesButton.setSelection(dirInfo.isIncludeHiddenShares());
			
			Label includeHiddenSharedDrivesLabel = new Label(composite, SWT.NONE);
			includeHiddenSharedDrivesLabel.setText("Include hidden shared drives (like C$):");
			includeHiddenSharedDrivesButton = new Button(composite, SWT.CHECK);
			includeHiddenSharedDrivesButton.setLayoutData(new GridData());
			includeHiddenSharedDrivesButton.setSelection(dirInfo.isIncludeHiddenSharedDrives());
			
			Label excludeSharesLabel = new Label(composite, SWT.NONE);
			excludeSharesLabel.setText("Exclude the following shares (comma separated):    ");
			excludeSharesText = new Text(composite, SWT.BORDER);
			excludeSharesText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			excludeSharesText.setText(dirInfo.getExcludeShares());
		}
		
		return composite;
	}

	protected void buttonPressed(int buttonId) {
		if (buttonId == IDialogConstants.OK_ID) {
			// validate fields
			if (directoryPathText.getText().length() == 0) {
				MessageDialog.openError(null, "Details not complete", "Please enter a directory path.");
				return;
			}
			
			// check if the name already exists
			if (dirInfo.getDirectoryId() == -1) {
				for (DirectoryInfo directory : existingDirectoyInfoList) {
					if (directory.getPath().equalsIgnoreCase(directoryPathText.getText())) {
						// connection name already exists
						MessageDialog.openError(null, "Directory already exists", "The directory path " + directoryPathText.getText() + " already exists in the list.");
						return;
					}
				}
			}
		
			dirInfo.setPath(directoryPathText.getText());
			dirInfo.setNumberOfSubdirectories(numberSubdirectoriesSpinner.getSelection());
			dirInfo.setIncludeFiles(includeFilesButton.getSelection());
			dirInfo.setIncludeInheritedPermissions(includeInheritedPermissionsButton.getSelection());
			
			if (dirInfo.getSourceType().equals(SourceType.DIRECTORY) || dirInfo.getSourceType().equals(SourceType.SERVER)) {
				dirInfo.setFollowSymbolicLinks(followSymbolicLinksButton.getSelection());
				dirInfo.setScanLocalGroups(scanLocalGroupsButton.getSelection());
			}
			
			if (dirInfo.getSourceType().equals(SourceType.SERVER)) {
				dirInfo.setExcludeShares(excludeSharesText.getText());
				dirInfo.setIncludeHiddenSharedDrives(includeHiddenSharedDrivesButton.getSelection());
				dirInfo.setIncludeHiddenShares(includeHiddenSharesButton.getSelection());
			}
		}
		
		super.buttonPressed(buttonId);
	}

	public DirectoryInfo getDirInfo() {
		return dirInfo;
	}
}
