package com.perdemia.permissionanalyzer.application.jface.dialogs;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import com.perdemia.permissionanalyzer.application.handlers.menu.OpenPreferencesHandler;
import com.perdemia.permissionanalyzer.application.model.LdapConnectionInfo;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;

public class LdapPasswordDialog extends DialogWithTopBar {
	private final int OPEN_PREFS_BUTTON = 100;
	private Text passwordText;
	private Text usernameText;
	private Button storePasswordButton;
	
	private LdapConnectionInfo ldapInfo;
	private String user;
	private String password;
	private boolean storePassword;
	
	public LdapPasswordDialog(Shell shell, LdapConnectionInfo ldapInfo) {
		super(shell);
		this.ldapInfo = ldapInfo;
	}

	@Override
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setImage(ImageFactory.getInstance().getImage(AppImage.CONNECTION_16X16));
		shell.setSize(600, 360);
		shell.setText("LDAP Login");
	}
	
	@Override
	protected Control createDialogArea(Composite container) {
		Composite composite = (Composite) super.createDialogArea(container, "LDAP Login", ImageFactory.getInstance().getImage(AppImage.CONNECTION_48X48));
		container.setBackgroundMode(SWT.INHERIT_DEFAULT);
				
		GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.marginTop = 10;
		gridLayout.marginLeft = 7;
		gridLayout.marginRight = 7;
		composite.setLayout(gridLayout);
		
		Label label = new Label(composite, SWT.NONE);
		label.setText("Please enter your credentials for the bind user to LDAP");
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.heightHint = 30;
		gridData.horizontalSpan = 2;
		label.setLayoutData(gridData);
		
		Label domainControllerHostname = new Label(composite, SWT.NONE);
		domainControllerHostname.setText("Domain controller: ");
		gridData = new GridData();
		gridData.heightHint = 30;
		domainControllerHostname.setLayoutData(gridData);
		
		Label domainControllerHostnameValue = new Label(composite, SWT.NONE);
		domainControllerHostnameValue.setText(ldapInfo.getHostName() + " : " + ldapInfo.getHostPort());
		gridData = new GridData();
		gridData.heightHint = 30;
		domainControllerHostnameValue.setLayoutData(gridData);
		
		Label usernameLabel = new Label(composite, SWT.NONE);
		usernameLabel.setText("Username: ");
		usernameText = new Text(composite, SWT.SINGLE | SWT.BORDER);
		usernameText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		usernameText.setText(ldapInfo.getUsername());

		Label passwordLabel = new Label(composite, SWT.NONE);
		passwordLabel.setText("Password: ");
		passwordText = new Text(composite, SWT.SINGLE | SWT.BORDER | SWT.PASSWORD);
		passwordText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
					
		storePasswordButton = new Button(composite, SWT.CHECK);
		storePasswordButton.setText("Store the password in the preferences using strong AES encryption.");
		storePasswordButton.setSelection(true);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.heightHint = 50;
		gridData.horizontalSpan = 2;
		storePasswordButton.setLayoutData(gridData);
		
		return composite;
	}

	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		GridLayout gridLayout = (GridLayout) parent.getLayout();
		gridLayout.makeColumnsEqualWidth = false;
		((GridData)parent.getLayoutData()).horizontalAlignment = GridData.FILL;
				
		Button openPreferencesButton = createButton(parent, OPEN_PREFS_BUTTON, "Open connection details", false);
		openPreferencesButton.setImage(ImageFactory.getInstance().getImage(AppImage.PREFERENCES_16X16));
		((GridData) openPreferencesButton.getLayoutData()).widthHint = 200;
				
		// filler
		Composite fillComposite = new Composite(parent, SWT.NONE);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.heightHint = 10;
		fillComposite.setLayoutData(gridData);
		((GridLayout) parent.getLayout()).numColumns++; //also triggered by the createButton calls
		
		super.createButtonsForButtonBar(parent);
	}
	
	protected void buttonPressed(int buttonId) {
		if (buttonId == IDialogConstants.OK_ID) {
			if (passwordText.getText().isEmpty() || usernameText.getText().isEmpty()) {
				MessageDialog.openError(getShell(), "Error", "Please fill out a username and password.");
				return;
			}
			this.password = passwordText.getText();
			this.user = usernameText.getText();
			this.storePassword = storePasswordButton.getSelection();
			
		} else if (buttonId == OPEN_PREFS_BUTTON) {
			// trigger the open preference command
			OpenPreferencesHandler handler = new OpenPreferencesHandler();
			handler.execute(getShell());
		}
		
		super.buttonPressed(buttonId);
	}

	public String getPassword() {
		return password;
	}

	public boolean isStorePassword() {
		return storePassword;
	}
	
	public String getUser() {
		return user;
	}
}
