package com.perdemia.permissionanalyzer.application.jface.dialogs;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.util.ColorFactory;
import com.perdemia.permissionanalyzer.application.util.Constants;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.LicenseHelper;
import com.perdemia.permissionanalyzer.application.util.ColorFactory.AppColor;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.application.util.LicenseHelper.Edition;
import com.perdemia.permissionanalyzer.application.util.PermissionUtil;
import com.perdemia.permissionanalyzer.data.AclItem;
import com.perdemia.permissionanalyzer.data.PermissionLookup;
import com.perdemia.permissionanalyzer.data.PermissionLookup.SimplePermission;

public class ModifyAceDialog extends DialogWithTopBar {
	private static final Logger logger = LoggerFactory.getLogger(ModifyAceDialog.class.getName());
	private AclItem aclItem;
	
	/** GUI controls */
	private Combo flagsCombo;
	private Button allowButton;
	private Button denyButton;
	private Button[] permissionButtons = new Button[20];
	private Button genericAllButton;
	private Button genericExecuteButton;
	private Button genericWriteButton;
	private Button genericReadButton;
	private Button noPropagateButton;
	
	/** Delete button id */
	public final static int DELETE_ID = 5;
	public final static int ADD_ID = 6;
	public final static int UPDATE_ID = 7;
	
	public ModifyAceDialog(Shell shell, AclItem aclItem) {
		super(shell);
		this.aclItem = aclItem;
	}
		
	@Override
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText("Modify Access Control Entry");
		if (aclItem.isFlagInheritedAce()) {
			newShell.setSize(620, 880);
		} else {
			newShell.setSize(620, 840);
		}
		newShell.setImage(ImageFactory.getInstance().getImage(AppImage.PERM_READ_ACL_16X16));
	}
	
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite composite = (Composite) super.createDialogArea(parent, "Modify Access Control Entry", ImageFactory.getInstance().getImage(AppImage.ACL_48X48));
		parent.setBackgroundMode(SWT.INHERIT_DEFAULT);
		
		GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.marginTop = 15;
		gridLayout.marginLeft = 15;
		gridLayout.marginRight = 15;
		gridLayout.horizontalSpacing = 20;
		gridLayout.verticalSpacing = 10;
		composite.setLayout(gridLayout);
		
		if (aclItem.isFlagInheritedAce()) {
			Composite trialComposite = new Composite(composite, SWT.NONE);
			GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
			gridData.horizontalSpan = 2;
			gridData.heightHint = 40;
			trialComposite.setLayoutData(gridData);
			gridLayout = new GridLayout(1, false);
			trialComposite.setLayout(gridLayout);
			trialComposite.setBackground(ColorFactory.getInstance().getColor(AppColor.TRIAL_BANNER));
			
			Label trialLabel = new Label(trialComposite, SWT.NONE);
			trialLabel.setText("This ACE is inherited from a parent folder, modify the ACE in the parent folder");
			gridData = new GridData(GridData.FILL_HORIZONTAL);
			gridData.horizontalIndent = 5;
			gridData.verticalAlignment = SWT.CENTER;
			gridData.grabExcessVerticalSpace = true;
			trialLabel.setLayoutData(gridData);
			trialLabel.setData(Constants.CSS_CLASS_NAME_KEY, "h2");
		}
		
		Label fileLabel = new Label(composite, SWT.NONE);
		fileLabel.setText("File / folder: ");
		Label fileValueLabel = new Label(composite, SWT.NONE);
				
		Label accountNameLabel = new Label(composite, SWT.NONE);
		accountNameLabel.setText("Account name: ");
		Label accountNameValueLabel = new Label(composite, SWT.NONE);
		
		Label flagsLabel = new Label(composite, SWT.NONE);
		flagsLabel.setText("ACE flags: ");
		flagsCombo = new Combo(composite, SWT.BORDER | SWT.READ_ONLY);
		if (aclItem.getFile().isDirectory()) {
			flagsCombo.add("This folder only");
		} else {
			flagsCombo.add("This file only");
		}
		flagsCombo.add("This folder, subfolders and files");
		flagsCombo.add("This folder and subfolders");
		flagsCombo.add("This folder and files");
		flagsCombo.add("Subfolders and files only (not this folder)");
		flagsCombo.add("Subfolders only (not this folder)");
		flagsCombo.add("Files only (not this folder)");
		
		flagsCombo.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (flagsCombo.getSelectionIndex() == 0) {
					// no_propagate flag not possible when ACE only applied to this container
					noPropagateButton.setEnabled(false);
					
					// remember the old setting before resetting it
					aclItem.setFlagNoPropagateInherit(noPropagateButton.getSelection());
					noPropagateButton.setSelection(false);
				} else {
					noPropagateButton.setEnabled(aclItem.getFile().isDirectory());
					noPropagateButton.setSelection(aclItem.isFlagNoPropagateInherit());
				}
			}
		});
		
		Label typeLabel = new Label(composite, SWT.NONE);
		typeLabel.setText("ACE type: ");
		
		Composite aceTypesComposite = new Composite(composite, SWT.NONE);
		aceTypesComposite.setLayout(new GridLayout(2, false));
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		//gridData.horizontalSpan = 2;
		aceTypesComposite.setLayoutData(gridData);
		
		allowButton = new Button(aceTypesComposite, SWT.RADIO);
		allowButton.setImage(ImageFactory.getInstance().getImage(AppImage.ALLOW_20X20));
		allowButton.setText("Allow");
		denyButton = new Button(aceTypesComposite, SWT.RADIO);
		denyButton.setImage(ImageFactory.getInstance().getImage(AppImage.DENY_20X20));
		denyButton.setText("Deny");

		Composite permissionsComposite = new Composite(composite, SWT.NONE);
		gridLayout = new GridLayout(2, false);
		gridLayout.horizontalSpacing = 20;
		gridLayout.verticalSpacing = 5;
		permissionsComposite.setLayout(gridLayout);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		permissionsComposite.setLayoutData(gridData);
		
		// header for simple permissions
		Label simplePermissionsLabel = new Label(permissionsComposite, SWT.NONE);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		gridData.verticalIndent = 5;
		gridData.heightHint = 20;
		simplePermissionsLabel.setLayoutData(gridData);
		simplePermissionsLabel.setText("Simple permissions:");
		simplePermissionsLabel.setData(Constants.CSS_CLASS_NAME_KEY, "h2");
		
		// simple permissions
		for (int counter = 0; counter < 7; counter++) {
			final Button permissionButton = new Button(permissionsComposite, SWT.CHECK);
			permissionButton.setText(PermissionUtil.simplePermissionToString(SimplePermission.values()[counter]));
			permissionButton.setImage(PermissionUtil.getSimplePermissionImage(SimplePermission.values()[counter]));
			permissionButton.setData("simplePermission", SimplePermission.values()[counter]);
			permissionButtons[counter] = permissionButton;
			
			// react on changes
			permissionButton.addSelectionListener(new SelectionAdapter(){
				@Override
				public void widgetSelected(SelectionEvent e) {
					// check or uncheck all corresponding special permissions					
					SimplePermission simplePermission = (SimplePermission) permissionButton.getData("simplePermission");
					
					// uncheck all special permissions when the 'special permission' is uncheckbox
					if (simplePermission.equals(SimplePermission.SPECIAL) && !permissionButton.getSelection()) {
						for (int counter = 7; counter < 20; counter++) {
							permissionButtons[counter].setSelection(false);
						}
						
					} else if (!simplePermission.equals(SimplePermission.SPECIAL)){
						// when the special permission checkbox is checked, it means that a custom set of special permissions is selected.
						// checking this simple permission will immediately uncheck it, because there is no exact match on the simple permission
						// therefore, we uncheck all special permissions when the 'special permission' checkbox is checked
						if (permissionButtons[6].getSelection()) {
							for (int counter = 7; counter < 20; counter++) {
								permissionButtons[counter].setSelection(false);
							}
						}
						
						// check or uncheck all corresponding special permissions
						Integer[] specialPermissions = PermissionLookup.permissionMappings.get(simplePermission);
						for (Integer specialPermission : specialPermissions) {
							permissionButtons[specialPermission + 7].setSelection(permissionButton.getSelection()); // 7 is offset for the simple permissions
						}
					}
					
					// re-calculate the matching between simple and special permissions by triggering a select event on a special permission checkbox
					permissionButtons[7].notifyListeners(SWT.Selection, new Event());
				}
			});
		}
		
		// header for special permissions
		Label specialPermissionsLabel = new Label(permissionsComposite, SWT.NONE);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		gridData.verticalIndent = 20;
		gridData.heightHint = 30;
		specialPermissionsLabel.setLayoutData(gridData);
		specialPermissionsLabel.setText("Special permissions:");
		specialPermissionsLabel.setData(Constants.CSS_CLASS_NAME_KEY, "h2");
		
		// special permissions
		for (int counter = 0; counter < 13; counter++) {
			final Button permissionButton = new Button(permissionsComposite, SWT.CHECK);
			permissionButton.setText(PermissionUtil.specialPermissionToString(counter, false, null));
			permissionButton.setImage(PermissionUtil.getSpecialPermissionImage(counter, false, true));
			permissionButton.setSelection(PermissionLookup.containsSpecialPermission(aclItem.getPermissionId(), counter));
			permissionButton.setData("specialPermission", counter);
			permissionButtons[counter + 7] = permissionButton;
			
			// react on changes
			permissionButton.addSelectionListener(new SelectionAdapter(){
				@Override
				public void widgetSelected(SelectionEvent e) {
					// first uncheck all simple permissions
					for (int counter = 0; counter < 7; counter++) {
						permissionButtons[counter].setSelection(false);
					}
					
					// get all checked special permissions
					List<Integer> checkedSpecialPermissions = new ArrayList<Integer>();
					for (int counter = 7; counter < 20; counter++) { // 7 is offset for the simple permissions
						if (permissionButtons[counter].getSelection()) {
							checkedSpecialPermissions.add(counter - 7);
						}
					}
					
					// do we have an exact match with one or more simple permissions?
					List<Integer> checkedSpecialPermissionsCopy = new ArrayList<Integer>(checkedSpecialPermissions);
					boolean exactMatchWithSimplePermissions = false;
					Collection<Integer[]> specialPermissionSets = PermissionLookup.permissionMappings.values();
					for (Integer[] specialPermissionSet : specialPermissionSets) {
						List<Integer> specialPermissions = Arrays.asList(specialPermissionSet);
						if (checkedSpecialPermissionsCopy.containsAll(specialPermissions)) {
							// remove the special permissions that belong to this simple permission
							checkedSpecialPermissionsCopy.removeAll(specialPermissions);
						}
					}
					exactMatchWithSimplePermissions = checkedSpecialPermissionsCopy.size() == 0;
					
					if (exactMatchWithSimplePermissions) {
						// check all simple permissions that are effective based on the selected special permissions
						for (int index = 0; index < specialPermissionSets.size(); index++) {
							
							// get the special permissions that belong to the next simple permission
							Integer[] specialPermissionSet = (Integer[]) specialPermissionSets.toArray()[index];
							List<Integer> specialPermissions = Arrays.asList(specialPermissionSet);
							if (checkedSpecialPermissions.containsAll(specialPermissions)) {
								permissionButtons[index].setSelection(true);
							}
						}
					} else {
						// check the 'special permissions' checkbox
						permissionButtons[6].setSelection(true);
					}
				}
			});
		}
		
		// header for generic permissions
		Label genericPermissionsLabel = new Label(permissionsComposite, SWT.NONE);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		gridData.verticalIndent = 5;
		gridData.heightHint = 20;
		genericPermissionsLabel.setLayoutData(gridData);
		genericPermissionsLabel.setText("Generic permissions:");
		genericPermissionsLabel.setData(Constants.CSS_CLASS_NAME_KEY, "h2");
		
		// generic permissions
		genericAllButton = new Button(permissionsComposite, SWT.CHECK);
		genericAllButton.setText("Generic ALL");
		genericAllButton.setImage(ImageFactory.getInstance().getImage(AppImage.PERM_READ_ACL_16X16));
		
		genericWriteButton = new Button(permissionsComposite, SWT.CHECK);
		genericWriteButton.setText("Generic WRITE");
		genericWriteButton.setImage(ImageFactory.getInstance().getImage(AppImage.PERM_READ_ACL_16X16));
		
		genericExecuteButton = new Button(permissionsComposite, SWT.CHECK);
		genericExecuteButton.setText("Generic EXECUTE");
		genericExecuteButton.setImage(ImageFactory.getInstance().getImage(AppImage.PERM_READ_ACL_16X16));
		
		genericReadButton = new Button(permissionsComposite, SWT.CHECK);
		genericReadButton.setText("Generic READ");
		genericReadButton.setImage(ImageFactory.getInstance().getImage(AppImage.PERM_READ_ACL_16X16));
		
		noPropagateButton = new Button(composite, SWT.CHECK);
		noPropagateButton.setText("Only apply this ACE to this folder and direct subfolders and files.");
		noPropagateButton.setImage(ImageFactory.getInstance().getImage(AppImage.BROWSE_16X16));
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		gridData.horizontalIndent = 5;
		noPropagateButton.setLayoutData(gridData);
		noPropagateButton.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e) {
				aclItem.setFlagNoPropagateInherit(noPropagateButton.getSelection());
			};
		});
		
		// initialize the controls
		accountNameValueLabel.setText(aclItem.getMember(false).getName(true));
		fileValueLabel.setText(aclItem.getFile().determineFullPath());
		
		if (!aclItem.isFlagDirectoryInherit() && !aclItem.isFlagFileInherit() && !aclItem.isFlagInheritOnly()) {
			flagsCombo.select(0);
		} else if (aclItem.isFlagDirectoryInherit() && aclItem.isFlagFileInherit() && !aclItem.isFlagInheritOnly()) {
			flagsCombo.select(1);
		} else if (aclItem.isFlagDirectoryInherit() && !aclItem.isFlagFileInherit() && !aclItem.isFlagInheritOnly()) {
			flagsCombo.select(2);
		} else if (!aclItem.isFlagDirectoryInherit() && aclItem.isFlagFileInherit() && !aclItem.isFlagInheritOnly()) {
			flagsCombo.select(3);
		} else if (aclItem.isFlagDirectoryInherit() && aclItem.isFlagFileInherit() && aclItem.isFlagInheritOnly()) {
			flagsCombo.select(4);		
		} else if (aclItem.isFlagDirectoryInherit() && !aclItem.isFlagFileInherit() && aclItem.isFlagInheritOnly()) {
			flagsCombo.select(5);
		} else if (!aclItem.isFlagDirectoryInherit() && aclItem.isFlagFileInherit() && aclItem.isFlagInheritOnly()) {
			flagsCombo.select(6);
		} else {
			logger.error("Could not determine ACL flag text for the acl " + aclItem.toString());
		}
		
		flagsCombo.setEnabled(aclItem.getFile().isDirectory());
				
		allowButton.setSelection(!aclItem.isDeny());
		denyButton.setSelection(aclItem.isDeny());
		genericAllButton.setSelection(aclItem.isGenericAll());
		genericWriteButton.setSelection(aclItem.isGenericWrite());
		genericExecuteButton.setSelection(aclItem.isGenericExecute());
		genericReadButton.setSelection(aclItem.isGenericRead());
		noPropagateButton.setSelection(aclItem.isFlagNoPropagateInherit());
		noPropagateButton.setEnabled(aclItem.getFile().isDirectory());
		
		// re-calculate the matching between simple and special permissions by triggering a select event on a special permission checkbox
		permissionButtons[8].notifyListeners(SWT.Selection, new Event());
		flagsCombo.notifyListeners(SWT.Selection, new Event());
		
		// disable all controls when the ACE is inherited
		if (aclItem.isFlagInheritedAce()) {
			flagsCombo.setEnabled(false);
			allowButton.setEnabled(false);
			denyButton.setEnabled(false);
			genericAllButton.setEnabled(false);
			genericWriteButton.setEnabled(false);
			genericExecuteButton.setEnabled(false);
			genericReadButton.setEnabled(false);
			noPropagateButton.setEnabled(false);
			
			for (Button button : permissionButtons) {
				button.setEnabled(false);
			}
		}
		
		return composite;		
	}
	
	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		//GridLayout gridLayout = (GridLayout) parent.getLayout();
		//gridLayout.makeColumnsEqualWidth = false;
		//((GridData)parent.getLayoutData()).horizontalAlignment = GridData.FILL;
				
		//createButton(parent, DELETE_ID, "Delete", false);
		//createButton(parent, ADD_ID, "Add", false);
		
		Button updateButton = createButton(parent, UPDATE_ID, "Update", true);
		updateButton.setEnabled(!aclItem.isFlagInheritedAce());
		
		// filler
		//Composite fillComposite = new Composite(parent, SWT.NONE);
		//GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		//gridData.heightHint = 10;
		//fillComposite.setLayoutData(gridData);
		//((GridLayout) parent.getLayout()).numColumns++; //also triggered by the createButton calls
		
		// license, but not yet activated?
		//activateButton = createButton(parent, ACTIVATE_BUTTON, "Activate license", false);
		//((GridData) activateButton.getLayoutData()).widthHint = 150;
		
		//Button button = createButton(parent, IMPORT_LICENSE_BUTTON, "Import License", false);
		//((GridData) button.getLayoutData()).widthHint = 150;
		
		createButton(parent, IDialogConstants.CANCEL_ID, "Cancel", false);		
	}
	
	@Override
	protected void buttonPressed(int buttonId) {
		if (buttonId == ADD_ID || buttonId == UPDATE_ID) {
			// copy the GUI fields to the AclItem
			aclItem.setDeny(denyButton.getSelection());
			
			// get all checked special permissions
			List<Integer> checkedSpecialPermissions = new ArrayList<Integer>();
			for (int counter = 7; counter < 20; counter++) { // 7 is offset for the simple permissions:
				if (permissionButtons[counter].getSelection()) {
					checkedSpecialPermissions.add(counter - 7);
				}
			}
			aclItem.setPermissionId(PermissionLookup.getIdBySpecialPermissions(checkedSpecialPermissions));
			
			// flags
			int flagIndex = flagsCombo.getSelectionIndex();
			aclItem.setFlagDirectoryInherit(flagIndex == 1 || flagIndex == 2 || flagIndex == 4 || flagIndex == 5);
			aclItem.setFlagFileInherit(flagIndex == 1 || flagIndex == 3 || flagIndex == 4 || flagIndex == 6);
			aclItem.setFlagInheritOnly(flagIndex == 4 || flagIndex == 5 || flagIndex == 6);
			aclItem.setFlagNoPropagateInherit(noPropagateButton.getSelection() && flagIndex != 0);
			
			// generic permissions
			aclItem.setGenericAll(genericAllButton.getSelection());
			aclItem.setGenericWrite(genericWriteButton.getSelection());
			aclItem.setGenericExecute(genericExecuteButton.getSelection());
			aclItem.setGenericRead(genericReadButton.getSelection());
		}
		
		setReturnCode(buttonId);
		close();
	}

	public AclItem getAclItem() {
		return aclItem;
	}

	public void setAclItem(AclItem aclItem) {
		this.aclItem = aclItem;
	}
}
