package com.perdemia.permissionanalyzer.application.jface.dialogs;

import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;

import com.perdemia.permissionanalyzer.application.preferences.AESEncrypter;
import com.perdemia.permissionanalyzer.application.preferences.PreferenceHelper;
import com.perdemia.permissionanalyzer.application.util.ApplicationUtil;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

public class PasswordDialog extends DialogWithTopBar {
	private Text passwordText;
	private int wrongPasswordCount;
	
	public PasswordDialog(Shell shell) {
		super(shell);
	}

	@Override
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setImage(ImageFactory.getInstance().getImage(AppImage.USER_FILTER_16X16));
		shell.setSize(600, 300);
		shell.setText("Permission Analyzer Login");
	}
	
	@Override
	protected Control createDialogArea(Composite container) {
		Composite composite = (Composite) super.createDialogArea(container, "Permission Analyzer Login", ImageFactory.getInstance().getImage(AppImage.USER_FILTER_48X48));
		container.setBackgroundMode(SWT.INHERIT_DEFAULT);
				
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.marginTop = 10;
		gridLayout.marginLeft = 7;
		gridLayout.marginRight = 7;
		composite.setLayout(gridLayout);
		
		Label label = new Label(composite, SWT.NONE);
		label.setText("Please enter the password to login:");
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.heightHint = 30;
		label.setLayoutData(gridData);
		
		passwordText = new Text(composite, SWT.SINGLE | SWT.BORDER | SWT.PASSWORD);
		passwordText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
						
		return composite;
	}

	protected void buttonPressed(int buttonId) {
		if (buttonId == IDialogConstants.OK_ID) {
			try {
				ApplicationUtil.setBusyCursor();
				if (AESEncrypter.hashPasswordToString(passwordText.getText()).equals(PreferenceHelper.getInstance().getMasterPasswordHash())){
					//password hashes match
					PreferenceHelper.getInstance().setMasterPassword(passwordText.getText());
				} else {
					wrongPasswordCount++;
					MessageDialog.openError(null, "Wrong password", "The password is not valid, please enter the correct password.");
					
					if (wrongPasswordCount >= 3) {
						// return cancel to exit the application
						buttonId = IDialogConstants.CANCEL_ID;
					} else {
						// keep the dialog open
						return;
					}
				}
			} catch (NoSuchAlgorithmException | InvalidKeySpecException e) {
				MessageDialog.openError(null, "Error", "Error while checking the password: " + e.getMessage());
				return;
			} finally {
				ApplicationUtil.setDefaultCursor();
			}
		}
		
		super.buttonPressed(buttonId);
	}
}
