package com.perdemia.permissionanalyzer.application.jface.dialogs;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;

import com.perdemia.permissionanalyzer.application.E4LifeCycle;

/**
 * A helper class to display a {@link MessageDialog} from a non-UI thread. Especially useful when a variable must be displayed that is defined
 * on method level and therefore not accessible by the Runnable object. Making the variable final is not always an option.
 * 
 * @author Reinier
 *
 */
public class SyncMessageDialog {
	private static boolean result;
	private static int intResult;
	
	/**
	 * Open the MessageDialog.
	 *  
	 * @return the result of the open method
	 */
	public static boolean showMessage(int messageType, String title, String message) {
		return showMessage(messageType, title, message, true);
	}
	
	/**
	 * Open the MessageDialog.
	 *  
	 * @return the result of the open method
	 */
	public static boolean showMessage(final int messageType, final String title, final String message, boolean defaultReturnValue) {		
		if (!E4LifeCycle.isSilentMode) {
			Display.getDefault().syncExec(new Runnable(){
				@Override
				public void run() {
					result = MessageDialog.open(messageType, null, title, message, SWT.NONE);
				}							
			});
		} else {
			result = defaultReturnValue;
		}
		
		return result;
	}
	
	/**
	 * We have a separate method for the type {@link MessageDialog#QUESTION_WITH_CANCEL} because we can't use the method {@link MessageDialog#open()}. This open method returns 
	 * a boolean (even for type QUESTION_WITH_CANCEL), while we need a int value to determine the difference between YES, NO and CANCEL.
	 * 
	 * @param title
	 * @param message
	 * @return {@link IDialogConstants#YES_ID}, {@link IDialogConstants#NO_ID} or {@link IDialogConstants#CANCEL_ID}
	 */
	public static int showQuestionWithCancelMessage(final String title, final String message) {
		return showQuestionWithCancelMessage(title, message, IDialogConstants.YES_LABEL, IDialogConstants.NO_LABEL);
	}
	
	/**
	 * We have a separate method for the type {@link MessageDialog#QUESTION_WITH_CANCEL} because we can't use the method {@link MessageDialog#open()}. This open method returns 
	 * a boolean (even for type QUESTION_WITH_CANCEL), while we need a int value to determine the difference between YES, NO and CANCEL.
	 * 
	 * @param title
	 * @param message
	 * @return {@link IDialogConstants#YES_ID}, {@link IDialogConstants#NO_ID} or {@link IDialogConstants#CANCEL_ID}
	 */
	public static int showQuestionWithCancelMessage(final String title, final String message, final String yesLabel, final String noLabel) {
		if (!E4LifeCycle.isSilentMode) {
			Display.getDefault().syncExec(new Runnable() {
				@Override
				public void run() {
					MessageDialog dialog = new MessageDialog(null, title, null, message, MessageDialog.QUESTION_WITH_CANCEL, new String[] { yesLabel, noLabel, IDialogConstants.CANCEL_LABEL}, 0);
					intResult = dialog.open();
					
					// map the button index to the right result
					if (intResult == 0) {
						intResult = IDialogConstants.YES_ID;
						
					} else if (intResult == 1) {
						intResult = IDialogConstants.NO_ID;
						
					} else if (intResult == 2) {
						intResult = IDialogConstants.CANCEL_ID;
					}				
				}							
			});
		} else {
			intResult = IDialogConstants.YES_ID;
		}
		
		return intResult;
	}
}
