package com.perdemia.permissionanalyzer.application.jface.labelproviders;

import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;

import com.perdemia.permissionanalyzer.application.util.ColorFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ColorFactory.AppColor;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.data.AclItem;
import com.perdemia.permissionanalyzer.data.FileInfo;

public class OwnerColumnLabelProvider extends ColumnLabelProvider {
	@Override
	public String getText(Object element) {
		if (element instanceof FileInfo) {
			FileInfo file = (FileInfo) element;
			if (file.getOwner() != null) {
				return file.getOwner().getName(true);
			} else {
				return null;
			}
		} else {
			return "Unknown type";
		}
	}
	
	@Override
	public Image getImage(Object element) {
		if (element instanceof FileInfo) {
			FileInfo file = (FileInfo) element;
			if (file.getOwner() != null) {
				if (file.getOwner().isGroup()) {
					return ImageFactory.getInstance().getImage(AppImage.GROUP_16X16);
				} else {
					return ImageFactory.getInstance().getImage(AppImage.USER_16X16);
				}
			}
		}
		
		return null;
	}
	
	@Override
	public Color getForeground(Object element) {
		if (element instanceof FileInfo) {
			FileInfo file = (FileInfo) element;
			
			boolean allImplicit = true;
			if (file.getEffectivePermissions() != null) {
				for (AclItem aclItem : file.getEffectivePermissions().getPermissions().values()) {
					allImplicit = allImplicit && aclItem.isFlagInheritedAce();
				}
			}
			
			if (allImplicit) {
				return ColorFactory.getInstance().getColor(AppColor.GREY);
			}
		}
		
		return null;
	}
}