package com.perdemia.permissionanalyzer.application.jface.labelproviders;

import org.eclipse.jface.viewers.OwnerDrawLabelProvider;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Event;

import com.perdemia.permissionanalyzer.application.util.ColorFactory;
import com.perdemia.permissionanalyzer.application.util.ColorFactory.AppColor;
import com.perdemia.permissionanalyzer.application.util.PermissionUtil;
import com.perdemia.permissionanalyzer.data.AclItem;
import com.perdemia.permissionanalyzer.data.EffectivePermissions;
import com.perdemia.permissionanalyzer.data.FileInfo;
import com.perdemia.permissionanalyzer.data.Member;
import com.perdemia.permissionanalyzer.data.PermissionLookup;

public class SpecialPermissionLabelProvider extends OwnerDrawLabelProvider {
	private Integer permission;
	private boolean useAceBackgroundColor;
	private boolean displayMemberPermissions;
	
	public SpecialPermissionLabelProvider(Integer permission, boolean useAceBackgroundColor, boolean displayMemberPermissions) {
		this.permission = permission;
		this.useAceBackgroundColor = useAceBackgroundColor;
		this.displayMemberPermissions = displayMemberPermissions;
	}

	@Override
	protected void measure(Event event, Object element) {}

	@Override
	protected void paint(Event event, Object element) {
		AclItem aclItem = null;
		
		if (element instanceof FileInfo) {
			EffectivePermissions effectivePermissions = ((FileInfo) element).getEffectivePermissions();
			if (effectivePermissions != null) {
				aclItem = effectivePermissions.getPermissions().get(permission);
			}
		} else if (element instanceof Member) {
			EffectivePermissions effectivePermissions = ((Member) element).getEffectivePermissions();
			if (effectivePermissions != null && displayMemberPermissions) {
				aclItem = effectivePermissions.getPermissions().get(permission);
			} else {
				return;
			}
		} else {
			aclItem = (AclItem) element;
		}
		
		if (aclItem != null) {
			Color start = null;
			Color end = null;
			boolean hasPermission = PermissionLookup.containsSpecialPermission(aclItem.getPermissionId(), permission);
			
			// 'disabled' grey row in the Trace View
			if (aclItem.getMember(false) == null && element instanceof AclItem) {
				start = ColorFactory.getInstance().getColor(AppColor.DISABLED_ROW);
				end = ColorFactory.getInstance().getColor(AppColor.DISABLED_ROW);
			
			// background color for explicit or deny permissions
			} else if (hasPermission &&	!aclItem.isFlagInheritedAce() || aclItem.isDeny()) {	
				
				// explicit allow				
				if (!aclItem.isDeny()) {
					start = ColorFactory.getInstance().getColor(AppColor.EXPLICIT_PERMISSION_ALLOW_START);
					end = ColorFactory.getInstance().getColor(AppColor.EXPLICIT_PERMISSION_ALLOW_END);
					
				} else if (aclItem.isDeny() && !aclItem.isFlagInheritedAce()) {
					// explicit deny
					start = ColorFactory.getInstance().getColor(AppColor.EXPLICIT_PERMISSION_DENY_START);
					end = ColorFactory.getInstance().getColor(AppColor.EXPLICIT_PERMISSION_DENY_END);
					
				} else if (aclItem.isDeny() && aclItem.isFlagInheritedAce()) { 
					//just deny, not explicit
					start = ColorFactory.getInstance().getColor(AppColor.PERMISSION_DENY_START);
					end = ColorFactory.getInstance().getColor(AppColor.PERMISSION_DENY_END);
				}
			}
			
			if (start != null) {
				event.gc.setForeground(start);
				event.gc.setBackground(end);				
				event.gc.fillGradientRectangle(event.getBounds().x, event.getBounds().y, 30, 22, true);
				
			} else if (useAceBackgroundColor) {
				event.gc.setBackground(ColorFactory.getInstance().getColor(AppColor.ACE_BACKGROUND_COLOR));
				event.gc.fillRectangle(event.getBounds().x, event.getBounds().y, 30, 22);				
			}
			
			// image of the permission
			if (hasPermission) {
				Image image = PermissionUtil.getSpecialPermissionImage(permission, aclItem.isDeny(), false);
				event.gc.drawImage(image, event.getBounds().x + 2, event.getBounds().y);
			}
		}
	}
	
	@Override
	public String getToolTipText(Object element) {
		AclItem aclItem;
		
		if (element instanceof FileInfo) {
			aclItem = ((FileInfo) element).getEffectivePermissions().getPermissions().get(permission);
		} else if (element instanceof Member) {
			EffectivePermissions effectivePermissions = ((Member) element).getEffectivePermissions();
			if (effectivePermissions == null) {
				return null;
			} else {
				aclItem = effectivePermissions.getPermissions().get(permission);
			}
		} else {
			aclItem = (AclItem) element;
		}
		
		if (aclItem != null && PermissionLookup.containsSpecialPermission(aclItem.getPermissionId(), permission)) {
			return PermissionUtil.specialPermissionToString(permission, aclItem.isDeny(), !aclItem.isFlagInheritedAce());
		} else {
			return null;
		}
	}
	
	@Override
	protected void erase(Event event, Object element) {
		//don't call super, because that will render a dark-blue background for the selected row
	}
}
