package com.perdemia.permissionanalyzer.application.jface.wizard.firstuse;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import com.perdemia.permissionanalyzer.application.jface.wizard.WizardDialogPage;
import com.perdemia.permissionanalyzer.application.jface.wizard.WizardDialogWithTopBar;
import com.perdemia.permissionanalyzer.application.preferences.PreferenceHelper;
import com.perdemia.permissionanalyzer.application.swt.ImageComposite;
import com.perdemia.permissionanalyzer.application.util.Constants;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;

public class FirstUseWizard extends WizardDialogWithTopBar {
		
	public FirstUseWizard(Shell shell) {
		super(shell, ImageFactory.getInstance().getImage(AppImage.APP_ICON_16X16), "Welcome", false);
	}

	@Override
	public List<WizardDialogPage> createPages(Composite parent) {
		String[] descriptions = new String[10];
		descriptions[0] = "Step 1: Open the Scan View to configure directories and LDAP OU's:";
		descriptions[1] = "Step 2: Start the scan:";
		descriptions[2] = "Step 3: Open the Report View and build your filters:";
		descriptions[3] = "Step 4: Filter on Permissions:";
		descriptions[4] = "Step 5: Filter on folders:";
		descriptions[5] = "Step 6: View the results:";
		descriptions[6] = "Step 7: Use the Trace View and view the origin of permissions:";
		descriptions[7] = "Step 8: Save your filters as report:";
		descriptions[8] = "Step 9: Define policies and check for unwanted permissions:";
		descriptions[9] = "Step 10: Modify the LDAP connections in the preferences:";
		
		AppImage[] images = new AppImage[10];
		images[0] = AppImage.FIRST_USE_1;
		images[1] = AppImage.FIRST_USE_2;
		images[2] = AppImage.FIRST_USE_3;
		images[3] = AppImage.FIRST_USE_4;
		images[4] = AppImage.FIRST_USE_5;
		images[5] = AppImage.FIRST_USE_6;
		images[6] = AppImage.FIRST_USE_7;
		images[7] = AppImage.FIRST_USE_8;
		images[8] = AppImage.FIRST_USE_9;
		images[9] = AppImage.FIRST_USE_10;
		
		List<WizardDialogPage> pages = new ArrayList<WizardDialogPage>();
		
		for (int counter = 0; counter < descriptions.length; counter++) {
			WizardDialogPage page = new WizardDialogPage(parent, SWT.NONE);
			page.setLayout(new GridLayout(1, false));
			
			Label descriptionLabel = new Label(page, SWT.NONE);
			GridData gridData = new GridData();
			gridData.heightHint = 35;
			gridData.horizontalIndent = 30;
			gridData.verticalIndent = 20;
			descriptionLabel.setLayoutData(gridData);
			descriptionLabel.setText(descriptions[counter]);
			descriptionLabel.setData(Constants.CSS_CLASS_NAME_KEY, "h1");
			
			new ImageComposite(page, ImageFactory.getInstance().getImage(images[counter]), "", true);
			
			pages.add(page);
		}
		
		return pages;
	}

	@Override
	public String getPageTitle(int index) {
		return MessageFormat.format("Welcome!  Please see the following screenshots for a quick first-use impression (step {0} of the {1})", index + 1, "8");
	}

	@Override
	public Image getPageImage(int index) {
		return ImageFactory.getInstance().getImage(AppImage.APP_ICON_48X48);
	}
	
	@Override
	protected Button getAdditionalButton(Composite parent) {
		final Button showNextTimeButton = new Button(parent, SWT.CHECK);
		showNextTimeButton.setText("Show this wizard next time");
		showNextTimeButton.setSelection(true);
		showNextTimeButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				PreferenceHelper.getInstance().setShowFirstUseWizard(showNextTimeButton.getSelection());
				PreferenceHelper.getInstance().storePreferences(false);
			}
		});
		
		return showNextTimeButton;
	}
	
	
	
	/*
	public FirstUseWizard(Shell shell) {
		super(shell);
		//setShellStyle(SWT.DIALOG_TRIM | SWT.RESIZE | SWT.MAX | SWT.APPLICATION_MODAL);
		
		
	}

	@Override
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setImage();
		shell.setSize(1000, 800);
		shell.setText();
	}
	
	@Override
	protected Control createDialogArea(Composite container) {
		Composite composite = (Composite) super.createDialogArea(container, MessageFormat.format(TITLE,  1, images.length), ImageFactory.getInstance().getImage(AppImage.APP_ICON_48X48));
		container.setBackgroundMode(SWT.INHERIT_DEFAULT);
				
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.marginTop = 15;
		gridLayout.marginLeft = 15;
		gridLayout.marginRight = 15;
		composite.setLayout(gridLayout);
		
		descriptionLabel = new Label(composite, SWT.NONE);
		GridData gridData = new GridData();
		gridData.heightHint = 35;
		descriptionLabel.setLayoutData(gridData);
		descriptionLabel.setText(descriptions[0]);
		descriptionLabel.setData(Constants.CSS_CLASS_NAME_KEY, "h1");
		
		image = new ImageComposite(composite, ImageFactory.getInstance().getImage(images[0]), "", true);
		
		return composite;
	}

	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		GridLayout gridLayout = (GridLayout) parent.getLayout();
		gridLayout.makeColumnsEqualWidth = false;
		((GridData)parent.getLayoutData()).horizontalAlignment = GridData.FILL;
		
		Label separator = new Label(parent, SWT.SEPARATOR | SWT.HORIZONTAL);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 6;
		separator.setLayoutData(gridData);
		((GridLayout) parent.getLayout()).numColumns++; //also triggered by the createButton calls
		
		final Button showNextTimeButton = new Button(parent, SWT.CHECK);
		showNextTimeButton.setText("Show this wizard next time");
		showNextTimeButton.setSelection(true);
		showNextTimeButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				PreferenceHelper.getInstance().setShowFirstUseWizard(showNextTimeButton.getSelection());
				PreferenceHelper.getInstance().storePreferences(false);
			}
		});
		
		((GridLayout) parent.getLayout()).numColumns++; //also triggered by the createButton calls
		
		// filler
		Composite fillComposite = new Composite(parent, SWT.NONE);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.heightHint = 10;
		fillComposite.setLayoutData(gridData);
		((GridLayout) parent.getLayout()).numColumns++; //also triggered by the createButton calls
		
		backButton = createButton(parent, IDialogConstants.BACK_ID, "< Previous", true);
		backButton.setEnabled(false);
		nextButton = createButton(parent, IDialogConstants.NEXT_ID, "Next >", true);
		createButton(parent, IDialogConstants.CLOSE_ID, "Close", true);
	}
	
	@Override
	protected void buttonPressed(int buttonId) {
		switch (buttonId) {
			case IDialogConstants.BACK_ID:
				index--;
				break;
				
			case IDialogConstants.NEXT_ID:
				index++;				
				break;
				
			case IDialogConstants.CLOSE_ID:
				close();
				return;
		}
		
		setTitle(MessageFormat.format(TITLE,  index + 1, images.length));
		image.setImage(ImageFactory.getInstance().getImage(images[index]));
		descriptionLabel.setText(descriptions[index]);
		descriptionLabel.pack(true);
		
		nextButton.setEnabled(index != images.length - 1);
		backButton.setEnabled(index != 0);
	}
	*/
}
