package com.perdemia.permissionanalyzer.application.jface.wizard.policy;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;

import com.perdemia.permissionanalyzer.application.jface.dialogs.SearchLdapDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SearchMembersDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.WildcardInputDialog;
import com.perdemia.permissionanalyzer.application.jface.labelproviders.TableLabelProvider;
import com.perdemia.permissionanalyzer.application.jface.wizard.WizardDialogPage;
import com.perdemia.permissionanalyzer.application.jface.wizard.policy.PagePolicyType.PolicyType;
import com.perdemia.permissionanalyzer.application.util.FilterHelper;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.data.Filter;
import com.perdemia.permissionanalyzer.data.Filter.FilterType;

/**
 * Page that takes care of selecting the right member filters for the corresponding policy type in page one.
 * 
 * @author Reinier
 * 
 */
class PageMemberFilters extends WizardDialogPage {
	private ArrayList<Filter> memberFilters = new ArrayList<Filter>();
	private TableViewer memberFilterListViewer;
	private Label explainLabel;
	private Button includeGroupMembershipButton;
	private Button includeUsersButton;
	private Button includeGroupsButton;
	private Button searchMembersButton;
	private Button searchLdapOUButton;
	
	public PageMemberFilters(Composite parent, int style) {
		super(parent, style);
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.marginLeft = 20;
		gridLayout.marginRight = 20;
		gridLayout.marginTop = 20;
		gridLayout.marginBottom = 20;
		setLayout(gridLayout);
		
		// explain text
		explainLabel = new Label(this, SWT.WRAP);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		explainLabel.setLayoutData(gridData);
		
		// button bar
		Composite buttonBar = new Composite(this, SWT.NONE);
		GridLayout layout = new GridLayout(4, false);
		layout.marginLeft = -5;
		layout.marginTop = 10;
		buttonBar.setLayout(layout);
		buttonBar.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		//search members
		searchMembersButton = new Button(buttonBar, SWT.PUSH);
		searchMembersButton.setImage(ImageFactory.getInstance().getImage(AppImage.SEARCH_16X16));
		searchMembersButton.setText("Select users or groups");
		searchMembersButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		searchMembersButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				boolean useInclude = getPolicyType().equals(PolicyType.NO_PARTICULAR_GROUP_MEMBERS) ||  getPolicyType().equals(PolicyType.NO_PARTICULAR_MEMBERS);
				boolean onlyUseGroups = getPolicyType().equals(PolicyType.NO_PARTICULAR_GROUP_MEMBERS) ||  getPolicyType().equals(PolicyType.ONLY_PARTICULAR_GROUP_MEMBERS);
				FilterType filterType = onlyUseGroups ? FilterType.MEMBER_GROUP_ID : FilterType.MEMBER_ID;
				
				SearchMembersDialog searchDialog = new SearchMembersDialog(Display.getCurrent().getActiveShell(), memberFilterListViewer, filterType, onlyUseGroups, useInclude, !useInclude);
				searchDialog.open();
			}
		});
		
		//Add LDAP OU
		searchLdapOUButton = new Button(buttonBar, SWT.PUSH);
		searchLdapOUButton.setImage(ImageFactory.getInstance().getImage(AppImage.LDAP_OU_16X16));
		searchLdapOUButton.setText("Select LDAP OU");
		searchLdapOUButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		searchLdapOUButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				boolean useInclude = getPolicyType().equals(PolicyType.NO_PARTICULAR_GROUP_MEMBERS) ||  getPolicyType().equals(PolicyType.NO_PARTICULAR_MEMBERS);
				SearchLdapDialog searchDialog = new SearchLdapDialog(Display.getCurrent().getActiveShell(), memberFilterListViewer, true, null, useInclude, !useInclude);
				searchDialog.open();
			}
		});
				
		// Add wildcard value button
		Button addWildCardToMemberListButton = new Button(buttonBar, SWT.PUSH);
		addWildCardToMemberListButton.setImage(ImageFactory.getInstance().getImage(AppImage.ADD_WILDCARD_16X16));
		addWildCardToMemberListButton.setText("Add wildcard filter");
		addWildCardToMemberListButton.setToolTipText("Add wildcard search text");
		addWildCardToMemberListButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		addWildCardToMemberListButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				boolean useInclude = getPolicyType().equals(PolicyType.NO_PARTICULAR_GROUP_MEMBERS) || getPolicyType().equals(PolicyType.NO_PARTICULAR_MEMBERS);
				boolean isAccount = getPolicyType().equals(PolicyType.ONLY_PARTICULAR_MEMBERS) || getPolicyType().equals(PolicyType.NO_PARTICULAR_MEMBERS);
				String title;
				String description;
				FilterType filterType;
				
				if (isAccount) {
					title = "Add wildcard filter on account name";
					description = "Please enter a wildcard value using the * character. Wildcard values will be used to filter on account name (without the domain name) and are not case-sensitive.\r\n\r\nExample: John*\r\n\r\nNote that a wildcard filter has impact on the performance of the overviews. If possible use the wildcard at the end of the search text.";
					filterType = FilterType.MEMBER_NAME;
				} else {
					title = "Add wildcard filter on group name";
					description = "Please enter a wildcard value using the * character. Wildcard values will be used to filter on group name (without the domain name).\r\n\r\nExample: HR*\r\n\r\nNote that a wildcard filter has impact on the performance of the overviews. If possible use the wildcard at the end of the search text.";
					filterType = FilterType.MEMBER_GROUP_NAME;
				}
				
				InputDialog inputDialog = new WildcardInputDialog(Display.getCurrent().getActiveShell(), title, description,	useInclude, !useInclude);
				int result = inputDialog.open();
				if (result == WildcardInputDialog.ADD_TO_INCLUDE_ID || result == WildcardInputDialog.ADD_TO_EXCLUDE_ID){
					if (!inputDialog.getValue().contains("%") && !inputDialog.getValue().contains("*")) {
						MessageDialog.openWarning(Display.getCurrent().getActiveShell(), "Invalid value", "The search text contains no wildcards, please use the member search button to filter on specific account names.");
					} else {
						if (FilterHelper.checkNumberOfMemberFiltersForTrial(memberFilters, true)) {
							memberFilters.add(new Filter(filterType, useInclude, inputDialog.getValue(), inputDialog.getValue()));
							memberFilterListViewer.refresh();
						}
					}
				}
			}
		});
		
		// Remove from member list
		Button removeFromMemberListButton = new Button(buttonBar, SWT.PUSH);
		removeFromMemberListButton.setImage(ImageFactory.getInstance().getImage(AppImage.DELETE_16X16));
		removeFromMemberListButton.setText("Remove selected filters");
		removeFromMemberListButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		removeFromMemberListButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				IStructuredSelection selection = (IStructuredSelection) memberFilterListViewer.getSelection();
				for (Iterator iterator = selection.iterator(); iterator.hasNext();) {
					memberFilters.remove(iterator.next());
				}
				memberFilterListViewer.refresh();
			}
		});
		
		//create a one column TableViewer so an image can be used
		memberFilterListViewer = new TableViewer(this, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL | SWT.FULL_SELECTION);
		gridData = new GridData(GridData.FILL_BOTH);
		memberFilterListViewer.getTable().setLayoutData(gridData);
		
		TableViewerColumn memberColumn = new TableViewerColumn(memberFilterListViewer, SWT.NONE);
		memberColumn.getColumn().setWidth(600);
				
		memberFilterListViewer.setContentProvider(ArrayContentProvider.getInstance());
		memberFilterListViewer.setLabelProvider(new TableLabelProvider(){
			@Override
			public Image getColumnImage(Object element, int columnIndex) {
				if (columnIndex == 0) {
					Filter filter = (Filter) element;
					if (filter.getType().equals(Filter.FilterType.MEMBER_ID)) {
						if (filter.isMemberGroup()) {
							if (filter.isInclude()) {
								return ImageFactory.getInstance().getImage(AppImage.INCLUDE_GROUPS_16X16);
							} else {
								return ImageFactory.getInstance().getImage(AppImage.EXCLUDE_GROUPS_16X16);
							}
						} else {
							if (filter.isInclude()) {
								return ImageFactory.getInstance().getImage(AppImage.INCLUDE_USERS_16X16);
							} else {
								return ImageFactory.getInstance().getImage(AppImage.EXCLUDE_USERS_16X16);
							}
						}
					} else if (filter.getType().equals(Filter.FilterType.MEMBER_NAME)) {
						if (filter.isInclude()) {
							return ImageFactory.getInstance().getImage(AppImage.INCLUDE_WILDCARD_16X16);
						} else {
							return ImageFactory.getInstance().getImage(AppImage.EXCLUDE_WILDCARD_16X16);
						}
					} else if (filter.getType().equals(Filter.FilterType.MEMBER_GROUP_ID)) {
						if (filter.isInclude()) {
							return ImageFactory.getInstance().getImage(AppImage.INCLUDE_GROUPS_16X16);
						} else {
							return ImageFactory.getInstance().getImage(AppImage.EXCLUDE_GROUPS_16X16);
						}
					} else if (filter.getType().equals(Filter.FilterType.MEMBER_GROUP_NAME)) {
						if (filter.isInclude()) {
							return ImageFactory.getInstance().getImage(AppImage.INCLUDE_WILDCARD_16X16);
						} else {
							return ImageFactory.getInstance().getImage(AppImage.EXCLUDE_WILDCARD_16X16);
						}
					} else if (filter.getType().equals(Filter.FilterType.MEMBER_LDAP_OU)) {
						if (filter.isInclude()) {
							return ImageFactory.getInstance().getImage(AppImage.INCLUDE_LDAP_OU_16X16);
						} else {
							return ImageFactory.getInstance().getImage(AppImage.EXCLUDE_LDAP_OU_16X16);
						}
					} else if (filter.getType().equals(Filter.FilterType.MEMBER_REPORT)) {
						if (filter.isInclude()) {
							return ImageFactory.getInstance().getImage(AppImage.INCLUDE_REPORT_16X16);
						} else {
							return ImageFactory.getInstance().getImage(AppImage.EXCLUDE_REPORT_16X16);
						}
					}
				}
				return null;
			}

			@Override
			public String getColumnText(Object element, int columnIndex) {
				Filter filter = (Filter) element;
				switch (columnIndex) {
					case 0: return filter.getName();
					default: return "";
				}
			}
		});
				
		memberFilterListViewer.setInput(memberFilters);

		Composite optionsComposite = new Composite(this, SWT.NONE);
		optionsComposite.setLayout(new GridLayout(3, false));
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		optionsComposite.setLayoutData(gridData);
		
		includeGroupMembershipButton = new Button(optionsComposite, SWT.CHECK);
		includeGroupMembershipButton.setText("Include (nested) group memberships");
		includeGroupMembershipButton.setImage(ImageFactory.getInstance().getImage(AppImage.GROUP_MEMBERSHIP_20X20));
		includeGroupMembershipButton.setToolTipText("Include (nested) group membership");
		includeGroupMembershipButton.setSelection(true);
		
		includeUsersButton = new Button(optionsComposite, SWT.CHECK);
		includeUsersButton.setText("Include users");
		includeUsersButton.setImage(ImageFactory.getInstance().getImage(AppImage.USER_20X20));
		includeUsersButton.setToolTipText("Include users");
		includeUsersButton.setSelection(true);
		
		includeGroupsButton = new Button(optionsComposite, SWT.CHECK);
		includeGroupsButton.setText("Include groups");
		includeGroupsButton.setImage(ImageFactory.getInstance().getImage(AppImage.GROUP_20X20));
		includeGroupsButton.setToolTipText("Include groups");
		includeGroupsButton.setSelection(true);
	}
	
	@Override
	public void pageDisplayed(boolean previousClicked) {
		switch (getPolicyType()) {
			case ONLY_PARTICULAR_GROUP_MEMBERS:
				explainLabel.setText("You have chosen to allow only the members of particular groups. This means you want to exclude those groups from the report, because a policy displays "
						+ "the unwanted permissions and should not yield any results. It displays the exceptions and triggers an alert when more directories and files have been found "
						+ "than the configured threshold value.\r\n\r\nPlease select the groups that will be added as \"exclude\" filter of the type \"All members from group\":");
				break;
			case NO_PARTICULAR_GROUP_MEMBERS:
				explainLabel.setText("You have chosen to check for unwanted permissions for the members of particular groups. This means you want to include those groups in the report, because a policy displays "
						+ "the unwanted permissions and should not yield any results. It displays the exceptions and triggers an alert when more directories and files have been found "
						+ "than the configured threshold value.\r\n\r\nPlease select the groups that will be added as \"include\" filter of the type \"All members from group\":");
				break;
			case ONLY_PARTICULAR_MEMBERS:
				explainLabel.setText("You have chosen to allow only particular users or groups. This means you want to exclude those members in the report, because a policy displays "
						+ "the unwanted permissions and should not yield any results. It displays the exceptions and triggers an alert when more directories and files have been found "
						+ "than the configured threshold value.\r\n\r\nPlease select the users and groups that will be added as \"exclude\" filter of the type \"Users/groups\":");
				break;
			case NO_PARTICULAR_MEMBERS:
				explainLabel.setText("You have chosen to check for unwanted permissions for particular users or groups. This means you want to include those members in the report, because a policy displays "
						+ "the unwanted permissions and should not yield any results. It displays the exceptions and triggers an alert when more directories and files have been found "
						+ "than the configured threshold value.\r\n\r\nPlease select the users and groups that will be added as \"include\" filter of the type \"Users/groups\":");
				break;
				
			case NO_EXPLICIT_USER_PERMISSIONS:
				includeGroupsButton.setSelection(false);
				includeGroupMembershipButton.setSelection(false);
				if (previousClicked) {
					wizardDialog.previous(); //skip the user page
				} else {
					wizardDialog.next(); //skip the user page
				}
				break;
			
		}
		explainLabel.pack();
		explainLabel.getParent().layout();
		
		boolean onlyUseGroups = getPolicyType().equals(PolicyType.NO_PARTICULAR_GROUP_MEMBERS) ||  getPolicyType().equals(PolicyType.ONLY_PARTICULAR_GROUP_MEMBERS);
		
		if (onlyUseGroups) {
			searchMembersButton.setText("Search groups");
			searchLdapOUButton.setVisible(true);
			((GridData) searchLdapOUButton.getLayoutData()).exclude = false;
		} else {
			searchMembersButton.setText("Search users or groups");
			searchLdapOUButton.setVisible(false);
			((GridData) searchLdapOUButton.getLayoutData()).exclude = true;
		}
		searchMembersButton.getParent().layout(true);
	}
	
	@Override
	public void resetPage() {
		memberFilters.clear();
		memberFilterListViewer.refresh();
	}
	
	private PolicyType getPolicyType() {
		return ((PagePolicyType)wizardDialog.getPages().get(0)).getSelectedPolicyType();
	}

	public ArrayList<Filter> getMemberFilters() {
		return memberFilters;
	}

	public Button getIncludeGroupMembershipButton() {
		return includeGroupMembershipButton;
	}

	public Button getIncludeUsersButton() {
		return includeUsersButton;
	}

	public Button getIncludeGroupsButton() {
		return includeGroupsButton;
	}
}
