package com.perdemia.permissionanalyzer.application.jface.wizard.policy;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import com.perdemia.permissionanalyzer.application.jface.wizard.WizardDialogPage;
import com.perdemia.permissionanalyzer.application.jface.wizard.policy.PagePolicyType.PolicyType;
import com.perdemia.permissionanalyzer.application.util.FilterHelper;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.PermissionUtil;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.data.PermissionLookup.SimplePermission;

/**
 * Page that takes care of selecting the right permission filters for the corresponding policy type in page one.
 * 
 * @author Reinier
 * 
 */
class PagePermissionFilters extends WizardDialogPage {
	private Label explainLabel;
	private List<Button> permissionCheckboxes = new ArrayList<Button>();
	private Button allowCheckbox;
	private Button denyCheckbox;
	private Button hideInheritedPermissionsButton;
	
	public PagePermissionFilters(Composite parent, int style) {
		super(parent, style);
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.marginLeft = 20;
		gridLayout.marginRight = 20;
		gridLayout.marginTop = 20;
		gridLayout.marginBottom = 20;
		setLayout(gridLayout);
		
		// explain text
		explainLabel = new Label(this, SWT.WRAP);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		explainLabel.setLayoutData(gridData);
		explainLabel.setText("Filter on specific permissions if you wish:");
		
		// spacer
		Composite spacer = new Composite(this, SWT.NONE);
		gridData = new GridData();
		gridData.heightHint = 20;
		spacer.setLayoutData(gridData);
		
		// button bar
		Composite permissionsComposite = new Composite(this, SWT.NONE);
		GridLayout layout = new GridLayout(3, false);
		layout.marginLeft = -5;
		layout.marginTop = 10;
		permissionsComposite.setLayout(layout);
		permissionsComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
				
		for (int permission = 0; permission < 13; permission++) {
			Button permissionCheckbox = new Button(permissionsComposite, SWT.CHECK);
			permissionCheckbox.setText(PermissionUtil.specialPermissionToString(permission, false, null));
			permissionCheckbox.setImage(PermissionUtil.getSpecialPermissionImage(permission, false, true));
			permissionCheckbox.setSelection(true);
			permissionCheckboxes.add(permissionCheckbox);
		}
		
		// spacer
		spacer = new Composite(this, SWT.NONE);
		gridData = new GridData();
		gridData.heightHint = 20;
		spacer.setLayoutData(gridData);
		
		allowCheckbox = new Button(this, SWT.CHECK);
		allowCheckbox.setText("Include allow permissions");
		allowCheckbox.setImage(ImageFactory.getInstance().getImage(AppImage.ALLOW_16X16));
		allowCheckbox.setSelection(true);
		
		denyCheckbox = new Button(this, SWT.CHECK);
		denyCheckbox.setText("Include deny permissions");
		denyCheckbox.setImage(ImageFactory.getInstance().getImage(AppImage.DENY_16X16));
		denyCheckbox.setSelection(true);
		
		hideInheritedPermissionsButton = new Button(this, SWT.CHECK);
		hideInheritedPermissionsButton.setImage(ImageFactory.getInstance().getImage(AppImage.BROWSE_16X16));
		hideInheritedPermissionsButton.setText("Hide permissions that are inherited");
		hideInheritedPermissionsButton.setToolTipText("Hide permissions that are inherited from a parent folder");
	}
	
	@Override
	public void pageDisplayed(boolean previousClicked) {
		PolicyType policyType = ((PagePolicyType) wizardDialog.getPages().get(0)).getSelectedPolicyType();
		switch (policyType) {
			case ONLY_PARTICULAR_GROUP_MEMBERS:
				explainLabel.setText("You have chosen to exclude all members of particular groups from the policy (meaning they will not be marked as unwanted). Now you can decide which of the remaining permissions you would like to mark as unwanted:");
				break;
			case NO_PARTICULAR_GROUP_MEMBERS:
				explainLabel.setText("You have chosen to check for unwanted permissions for the members of particular groups. Now you can choose which of their permissions you want to check:");
				break;
			case ONLY_PARTICULAR_MEMBERS:
				explainLabel.setText("You have chosen to exclude particular uesrss or groups from the policy (meaning they will not be marked as unwanted). Now you can decide which of the remaining permissions you would like to mark as unwanted:");
				break;
			case NO_PARTICULAR_MEMBERS:
				explainLabel.setText("You have chosen to check for unwanted permissions of particular users or groups. Now you can choose which of their permissions you want to check:");
				break;
			case NO_EXPLICIT_USER_PERMISSIONS:
				explainLabel.setText("You have chosen to check for explicit user permissions, therefore the option \"Hide inherited permissions\" has been enabled. Now you can choose which of the explicit permissions you want to check:");
				hideInheritedPermissionsButton.setSelection(true);
				break;
			
		}
		explainLabel.pack();
		explainLabel.getParent().layout();
	}
	
	@Override
	public void resetPage() {
		hideInheritedPermissionsButton.setSelection(false);
	}

	public List<Button> getPermissionCheckboxes() {
		return permissionCheckboxes;
	}

	public Button getAllowCheckbox() {
		return allowCheckbox;
	}

	public Button getDenyCheckbox() {
		return denyCheckbox;
	}

	public Button getHideInheritedPermissionsButton() {
		return hideInheritedPermissionsButton;
	}
}
