package com.perdemia.permissionanalyzer.application.jface.wizard.policy;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.jface.wizard.WizardDialogPage;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;

/**
 * The page to select a predefined policy type.
 * 
 * @author Reinier
 *
 */
class PagePolicyType extends WizardDialogPage {
	private Text nameText;
	private Text descriptionText;
	private PolicyType selectedPolicyType = PolicyType.ONLY_PARTICULAR_GROUP_MEMBERS;
	private static Map<PolicyType, String> policyTypeLabels;
	private static Map<PolicyType, String> policyTypeDescriptions;
	
	public enum PolicyType {
		/** Only the members of particular groups are allowed to have permissions in the given folder(s) */
		ONLY_PARTICULAR_GROUP_MEMBERS, 
		
		/** Members from particular groups are not allowed to have permissions in the given folder(s) */
		NO_PARTICULAR_GROUP_MEMBERS, 
		
		/** Only particular users or groups are allowed to have explicit permissions in the given folder(s) */
		ONLY_PARTICULAR_MEMBERS,
		
		/** Particular users or groups are not allowed to have explicit permissions in the given folder(s) */
		NO_PARTICULAR_MEMBERS,
		
		/** Explicit user permissions are not allowed in the given folder(s) */
		NO_EXPLICIT_USER_PERMISSIONS
	}
	
	static {
		policyTypeLabels = new HashMap<PolicyType, String>();
		policyTypeLabels.put(PolicyType.ONLY_PARTICULAR_GROUP_MEMBERS, "Only the members of particular groups are allowed to have permissions in the given folder(s)");
		policyTypeLabels.put(PolicyType.NO_PARTICULAR_GROUP_MEMBERS, "Members from particular groups are not allowed to have permissions in the given folder(s)");
		policyTypeLabels.put(PolicyType.ONLY_PARTICULAR_MEMBERS, "Only particular users or groups are allowed to have explicit permissions in the given folder(s)");
		policyTypeLabels.put(PolicyType.NO_PARTICULAR_MEMBERS, "Particular users or groups are not allowed to have explicit permissions in the given folder(s)");
		policyTypeLabels.put(PolicyType.NO_EXPLICIT_USER_PERMISSIONS, "Explicit user permissions are not allowed in the given folder(s)");
		
		policyTypeDescriptions = new HashMap<PolicyType, String>();
		policyTypeDescriptions.put(PolicyType.ONLY_PARTICULAR_GROUP_MEMBERS, "In this policy you select one or more groups to exclude from the overview, remember that a policy displays the unwanted permissions. ALL the members of those groups are allowed and will be excluded, which means the policy will only display the permissions of users/groups that are now allowed. For example, you may select the group \"Finance management\" and include the folder \"\\\\dataserver01\\Reports\\Finance\".");
		policyTypeDescriptions.put(PolicyType.NO_PARTICULAR_GROUP_MEMBERS, "In this policy you select one or more groups to include in the overview, remember that a policy displays the unwanted permissions. ALL the members of those groups will be included, which means the policy will display all the unwanted permissions of the users/groups. For each member the (nested) group membership are also checked. For example, you may select the group \"Freelancers\" and include the folder \"\\\\dataserver01\\Reports\\Finance\".");
		policyTypeDescriptions.put(PolicyType.ONLY_PARTICULAR_MEMBERS, "In this policy you select users and groups to exclude from the overview, remember that a policy displays the unwanted permissions. Only those users and groups are allowed and will be excluded, which means the policy will display all the unwanted permissions of other users and groups. For example, you may select the group \"Auditors\" and the permission \"Read Data\" and include the folder \"\\\\dataserver01\\Reports\\Finance\".");
		policyTypeDescriptions.put(PolicyType.NO_PARTICULAR_MEMBERS, "In this policy you select users and groups to include in the overview, remember that a policy displays the unwanted permissions. Only those users and groups will be included, which means the policy will display all the unwanted permissions. For each member the (nested) group membership are also checked. For example, you may select the group \"Freelancers\" and the permission \"Write Data\" and include the folder \"\\\\dataserver01\\Projects\\IT\".");
		policyTypeDescriptions.put(PolicyType.NO_EXPLICIT_USER_PERMISSIONS, "This policy will scope the results to users, includes the filter \"Hide inherited permissions\" and unchecks the option \"Include group memberships\". This will display all explicitly assigned user permissions, which may be considered as a bad practice.");
	}
	
	public PagePolicyType(Composite parent, int style) {
		super(parent, style);
		GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.marginLeft = 20;
		gridLayout.marginRight = 20;
		gridLayout.marginTop = 20;
		gridLayout.marginBottom = 20;
		setLayout(gridLayout);
		
		// explain label
		Label explainLabel = new Label(this, SWT.WRAP);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		explainLabel.setLayoutData(gridData);
		explainLabel.setText("This wizard guides you through a number of predefined policies. Remember you can save any set of filters as a new policy, but this wizard will limit the number of filters based on the selected policy to help you understand the possibilities.");
		
		// spacer
		Composite spacer = new Composite(this, SWT.NONE);
		gridData = new GridData();
		gridData.heightHint = 10;
		spacer.setLayoutData(gridData);
				
		// name
		Label nameLabel = new Label(this, SWT.NONE);
		nameLabel.setText("Policy name:");
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		nameLabel.setLayoutData(gridData);
		
		nameText = new Text (this, SWT.BORDER | SWT.SINGLE);
		nameText.setText("");
		nameText.addKeyListener(new KeyAdapter() {
			@Override
			public void keyReleased(KeyEvent e) {
				wizardDialog.enableNext(!nameText.getText().isEmpty());
			}
		});
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		nameText.setLayoutData(gridData);
		
		// spacer
		spacer = new Composite(this, SWT.NONE);
		gridData = new GridData();
		gridData.heightHint = 20;
		spacer.setLayoutData(gridData);
		
		// description
		Label descriptionLabel = new Label(this, SWT.NONE);
		descriptionLabel.setText("Policy description:");
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		descriptionLabel.setLayoutData(gridData);
		
		descriptionText = new Text(this, SWT.BORDER | SWT.MULTI | SWT.WRAP | SWT.V_SCROLL);
		descriptionText.setText("");
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		gridData.heightHint = 3 * descriptionText.getLineHeight();
		descriptionText.setLayoutData(gridData);
		
		// spacer
		spacer = new Composite(this, SWT.NONE);
		gridData = new GridData();
		gridData.heightHint = 20;
		spacer.setLayoutData(gridData);
				
		// policy types
		Label policyTypeLabel = new Label(this, SWT.NONE);
		policyTypeLabel.setText("Policy type:");
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		policyTypeLabel.setLayoutData(gridData);
		
		for (PolicyType type : PolicyType.values()) {
			// radio button
			Button policyTypeButton = new Button(this, SWT.RADIO);
			policyTypeButton.setData(type);
			policyTypeButton.setText(policyTypeLabels.get(type));
			policyTypeButton.setImage(ImageFactory.getInstance().getImage(AppImage.POLICY_16X16));
			policyTypeButton.addSelectionListener(new SelectionAdapter(){
				@Override
				public void widgetSelected(SelectionEvent e) {
					selectedPolicyType = (PolicyType) e.widget.getData();
					
					// reset the next wizard pages, because the policy type has been changed
					for (WizardDialogPage page : wizardDialog.getPages()) {
						page.resetPage();
					}
				}
			});
			
			// hard-coded, select the first one
			if (type.equals(PolicyType.ONLY_PARTICULAR_GROUP_MEMBERS)) {
				policyTypeButton.setSelection(true);
			}
			
			// info button
			Button infoButton = new Button(this, SWT.PUSH);
			infoButton.setImage(ImageFactory.getInstance().getImage(AppImage.INFO_16X16));
			infoButton.setData(type);
			infoButton.addSelectionListener(new SelectionAdapter(){
				@Override
				public void widgetSelected(SelectionEvent e) {
					SyncMessageDialog.showMessage(MessageDialog.INFORMATION, "Policy details", policyTypeDescriptions.get((PolicyType) e.widget.getData()));
				}
			});
		}
	}
	
	@Override
	public void pageDisplayed(boolean previousClicked) {
		wizardDialog.enableNext(!nameText.getText().isEmpty());
	}

	public String getName() {
		return nameText.getText();
	}
	
	public String getDescription() {
		return descriptionText.getText();
	}

	public PolicyType getSelectedPolicyType() {
		return selectedPolicyType;
	}
}
