package com.perdemia.permissionanalyzer.application.logging;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.Formatter;
import java.util.logging.LogRecord;

/**
 * Custom JDK log formatter to include milliseconds and compact log lines.
 *  
 * @author Reinier
 *
 */
public class LogFormatter extends Formatter {
	public static final DateFormat dateFormat = new SimpleDateFormat("dd-MM-yyyy HH:mm:ss.SSS");

	@Override
	public String format(LogRecord record) {
		StringBuilder builder = new StringBuilder(1000);
		builder.append(dateFormat.format(new Date(record.getMillis()))).append(" - ");

		builder.append("[").append(record.getLevel()).append("] ");

		if (record.getSourceClassName() != null) {
			builder.append(record.getSourceClassName()).append(".").append(record.getSourceMethodName()).toString();
		} else {
			builder.append(record.getLoggerName());
		}

		builder.append(": ").append(formatMessage(record));
		builder.append("\n");

		if (record.getThrown() != null) {
			StringWriter stringwriter = new StringWriter();
			PrintWriter printwriter = new PrintWriter(stringwriter);
			printwriter.println();
			record.getThrown().printStackTrace(printwriter);
			printwriter.close();
			builder.append(stringwriter.toString());
		}

		return builder.toString();
	}

}
