package com.perdemia.permissionanalyzer.application.parts;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.inject.Inject;
import javax.inject.Named;

import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MToolBarElement;
import org.eclipse.e4.ui.model.application.ui.menu.impl.ToolControlImpl;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.e4.ui.workbench.modeling.ESelectionService;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StyledCellLabelProvider;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.TreeViewerColumn;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;

import com.perdemia.permissionanalyzer.application.handlers.toolbar.ApplyFilterToolControl;
import com.perdemia.permissionanalyzer.application.jface.contentproviders.AclPartContentProvider;
import com.perdemia.permissionanalyzer.application.jface.contentproviders.AclPartContentProviderConfig;
import com.perdemia.permissionanalyzer.application.jface.dialogs.MemberInfoDialog;
import com.perdemia.permissionanalyzer.application.jface.labelproviders.SpecialPermissionLabelProvider;
import com.perdemia.permissionanalyzer.application.swt.ExplainLabel;
import com.perdemia.permissionanalyzer.application.swt.ImageComposite;
import com.perdemia.permissionanalyzer.application.swt.ExplainLabel.ExplainType;
import com.perdemia.permissionanalyzer.application.util.ColorFactory;
import com.perdemia.permissionanalyzer.application.util.ColorFactory.AppColor;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.application.util.PermissionUtil;
import com.perdemia.permissionanalyzer.data.AclItem;
import com.perdemia.permissionanalyzer.data.FileInfo;
import com.perdemia.permissionanalyzer.data.Member;
import com.perdemia.permissionanalyzer.data.PermissionLookup.SpecialPermission;

public class AclPart {
	public static final String ID = "com.perdemia.permissionanalyzer.application.parts.acl";
	
	private TreeViewer viewer;
	private AclPartContentProviderConfig fileDetailModel = new AclPartContentProviderConfig();
	
	@Inject ESelectionService selectionService;
	@Inject EPartService partService;
	
	/**
	 * Receives selection events from the main FileInfo tree and updates the input for the {@link AclPartContentProvider}
	 */
	@Inject
	public void fileSelected(MApplication application, EModelService modelService, @Optional @Named(IServiceConstants.ACTIVE_SELECTION) IStructuredSelection selectedItem) {
		// This line is necessary because when the users switches between the AclPart and TracePart this method is called
		// with an empty selectedPath
		selectedItem = (IStructuredSelection) selectionService.getSelection(FilePart.ID);
		boolean fileChanged = false;;
		
		if (selectedItem != null && !selectedItem.isEmpty() && selectedItem.getFirstElement() instanceof FileInfo) {
		
			// determine if the Apply Filter check button has been checked in the ToolBar
			boolean applyFilter = false;
			MPart aclPart = partService.findPart("com.perdemia.permissionanalyzer.application.parts.acl");
			if (aclPart != null) {
				for (MToolBarElement element : aclPart.getToolbar().getChildren()) {
					if (element.getElementId().equals("com.perdemia.permissionanalyzer.application.parts.acl.toolbar.applyFilter")) {
						//if (((ToolControlImpl)element).getObject() != null) {
							ApplyFilterToolControl toolItem = (ApplyFilterToolControl) ((ToolControlImpl)element).getObject();
							if (toolItem != null) {
							applyFilter = toolItem.isChecked();
							}
						//} else {
						//	applyFilter = true;
						//}
					}
				}
			}
			
			// fill the model with the selected file and its parent folders
			// the parent items are used to build to "Inherited from" folder path
			FileInfo selectedFile = (FileInfo) selectedItem.getFirstElement();
						
			// check on object reference of selectedFile (we do want to refresh when the filters have changed and the file id is the same!)
			fileChanged = selectedFile != fileDetailModel.getSelectedFile() || fileDetailModel.isApplyFilter() != applyFilter;
						
			fileDetailModel.setSelectedFile(selectedFile);
			fileDetailModel.setApplyFilter(applyFilter);
			
		} else {
			// no file selected or found, clear the view
			fileDetailModel.setSelectedFile(null);
			fileChanged = true;
		}
		
		if (viewer != null && !viewer.getTree().isDisposed() && fileChanged) {
			viewer.setInput(fileDetailModel);
			viewer.refresh();
			viewer.getTree().getParent().layout(true);
		}
	}
		
	@PostConstruct
	public void createComposite(Composite parent) {
		// View layout
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.verticalSpacing = 5;
		gridLayout.horizontalSpacing = 0;
		parent.setLayout(gridLayout);
				
		new ExplainLabel(parent, "This tab displays the permissions as they exist on the file system for the selected file.", ExplainType.ACL_TAB);
		
		// Tree with details	
		viewer = new TreeViewer(parent, SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER | SWT.FULL_SELECTION | SWT.VIRTUAL);
		viewer.getTree().setLinesVisible(true);
		viewer.getTree().setHeaderVisible(true);
		viewer.setUseHashlookup(true);
		
		//layout data for the tree
		GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
		viewer.getTree().setLayoutData(gridData);
		
		// Member column
		TreeViewerColumn column = new TreeViewerColumn(viewer, SWT.NONE);
		column.getColumn().setText("Access Control Entry");
		column.getColumn().setWidth(400);
		column.getColumn().setResizable(true);
		column.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				if (element instanceof AclItem) {
					return ((AclItem) element).getMember(true).getDisplayName(true, false);
				} else {
					return ((Member) element).getDisplayName(true, false);
				}				
			}
			
			@Override
			public Image getImage(Object element) {
				if (element instanceof AclItem) {
					return ImageFactory.getInstance().getImage(AppImage.ACL_20X20);
				} else {
					if (((Member) element).isGroup()){
						return ImageFactory.getInstance().getImage(AppImage.GROUP_20X20);
			        } else {
			        	return ImageFactory.getInstance().getImage(AppImage.USER_20X20);
			        }
				}				
			}
			
			@Override
			public Color getForeground(Object element) {
				if (element instanceof AclItem && ((AclItem) element).isFlagInheritedAce()) {
					return ColorFactory.getInstance().getColor(AppColor.GREY);					
				} else {
					return super.getForeground(element);
				}
			}
		});
				
		// Permission column
		column = new TreeViewerColumn(viewer, SWT.NONE);
		column.getColumn().setText("Permission");
		column.getColumn().setWidth(200);
		column.getColumn().setResizable(true);
		column.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				if (element instanceof AclItem) {
					AclItem aclItem = (AclItem) element;
					return PermissionUtil.permissionsToString(aclItem, false);
				} else {
					return null;
				}
			}
			
			@Override
			public Color getForeground(Object element) {
				if (element instanceof AclItem && ((AclItem) element).isFlagInheritedAce()) {
					return ColorFactory.getInstance().getColor(AppColor.GREY);					
				} else {
					return super.getForeground(element);
				}
			}
		});
				
		// Special permission columns
		for (final Integer permission : PermissionUtil.getCustomPermissionOrderList()) {
			//skip SYNCHRONIZE
			if (permission.intValue() != SpecialPermission.SYNCHRONIZE.ordinal()) {
				column = new TreeViewerColumn(viewer, SWT.NONE);
				column.getColumn().setWidth(30);
				column.getColumn().setResizable(false);
				column.getColumn().setToolTipText(PermissionUtil.specialPermissionToString(permission, false, null));
				column.getColumn().setImage(PermissionUtil.getSpecialPermissionImage(permission, false, true));
				column.setLabelProvider(new SpecialPermissionLabelProvider(permission, false, false));
			}
		}
		
		// Inheritance flags column
		column = new TreeViewerColumn(viewer, SWT.NONE);
		column.getColumn().setText("Inheritance flags");
		column.getColumn().setResizable(true);
		column.getColumn().setWidth(250);
		column.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				if (element instanceof AclItem) {
					AclItem aclItem = (AclItem) element;
					return PermissionUtil.aclEntryFlagsToString(aclItem, fileDetailModel.getSelectedFile().isDirectory());
				} else {
					return null;
				}
			}
			
			@Override
			public Color getForeground(Object element) {
				if (element instanceof AclItem && ((AclItem) element).isFlagInheritedAce()) {
					return ColorFactory.getInstance().getColor(AppColor.GREY);					
				} else {
					return super.getForeground(element);
				}
			}
		});
		
		// From folder column
		column = new TreeViewerColumn(viewer, SWT.NONE);
		column.getColumn().setText("From folder");
		column.getColumn().setResizable(true);
		column.getColumn().setWidth(350);
		column.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				if (element instanceof AclItem) {
					AclItem aclItem = (AclItem) element;
									
					if (!aclItem.isFlagInheritedAce()) {
						return "(explicitly assigned)";
					} else if (aclItem.getInheritedFromFolder() != null && aclItem.getInheritedFromFolder().length() > 0) {
						return aclItem.getInheritedFromFolder();
					} else {
						return "Unknown parent folder";
					}
				} else {
					return null;
				}
			}
			
			@Override
			public Color getForeground(Object element) {
				if (element instanceof AclItem && ((AclItem) element).isFlagInheritedAce()) {
					return ColorFactory.getInstance().getColor(AppColor.GREY);					
				} else {
					return super.getForeground(element);
				}
			}
		});
		
		viewer.setContentProvider(new AclPartContentProvider(viewer));
		
		//enable tooltips for this tree viewer
		ColumnViewerToolTipSupport.enableFor(viewer);
		
		// call the listener with the current selection
		//IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
		//mainFileTreelistener.selectionChanged(null, ((FilePart)page.findView(FilePart.ID)).getTreeViewer().getSelection());
		
		//TODO use postSelectionListener for performance?
		//getSite().getWorkbenchWindow().getSelectionService().addPostSelectionListener(mainFileTreelistener);
		
		// select the first item in the tree so that the AclPart get refreshed through the SelectionService
		MPart part = partService.findPart(FilePart.ID);
		FilePart fileTreePart = (FilePart) part.getObject();
		selectionService.setSelection(fileTreePart.getTreeViewer().getSelection());
		
		// context menu for the Tree
		MenuManager menuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			@Override
			public void menuAboutToShow(IMenuManager manager) {
				final IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
				if (selection != null && !selection.isEmpty()) {
					// Add to Trace view action
					Action action = new Action() {
						public void run() {
							Member member;
							if (selection.getFirstElement() instanceof AclItem) {
								member = ((AclItem) selection.getFirstElement()).getMember(true);
							} else {
								member = (Member) selection.getFirstElement();
							}
							MPart part = partService.findPart(TracePart.ID);
							partService.activate(part, true);
							TracePart tracePart = (TracePart) part.getObject();
							tracePart.applySelectedMember(member);
						}
					};
					action.setText("Pin member in the Trace view");
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.TRACE_16X16));
					manager.add(action);
					
					// Show user info action
					action = new Action() {
						public void run() {
							Member member;
							if (selection.getFirstElement() instanceof AclItem) {
								member = ((AclItem) selection.getFirstElement()).getMember(true);
							} else {
								member = (Member) selection.getFirstElement();
							}
							MemberInfoDialog dialog = new MemberInfoDialog(null, member);
							dialog.open();
						}
					};
					action.setText("Show member info");
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.USER_INFO_16X16));
					manager.add(action);
					
					// Add as Include filter action
					action = new Action() {
						public void run() {
							MPart part = partService.findPart(FilterMemberPart.ID);
							FilterMemberPart filterMemberPart = (FilterMemberPart) part.getObject();
							
							// loop through all the selected items
							@SuppressWarnings("rawtypes")
							Iterator iter = selection.iterator();
							while (iter.hasNext()) {
								Object nextSelection = iter.next();
								Member member;
								if (nextSelection instanceof AclItem) {
									member = ((AclItem) nextSelection).getMember(true);
								} else {
									member = (Member) nextSelection;
								}
								filterMemberPart.addNewMemberFilter(member, true);
							}
						}
					};
					action.setText("Add selected members as include filter");
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.INCLUDE_GROUPS_16X16));
					action.setToolTipText("Adds the selected members as includes in the main filter to scope the data to the selected members.");
					manager.add(action);
					
					// Add as Exclude filter action
					action = new Action() {
						public void run() {
							MPart part = partService.findPart(FilterMemberPart.ID);
							FilterMemberPart filterMemberPart = (FilterMemberPart) part.getObject();
							
							// loop through all the selected items
							@SuppressWarnings("rawtypes")
							Iterator iter = selection.iterator();
							while (iter.hasNext()) {
								Object nextSelection = iter.next();
								Member member;
								if (nextSelection instanceof AclItem) {
									member = ((AclItem) nextSelection).getMember(true);
								} else {
									member = (Member) nextSelection;
								}
								filterMemberPart.addNewMemberFilter(member, false);
							}
						}
					};
					action.setText("Add selected members as exclude filter");
					action.setToolTipText("Adds the selected members as excludes in the main filter.");
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.EXCLUDE_GROUPS_16X16));
					manager.add(action);
					
					// Open file in explorer action
					if (selection.getFirstElement() instanceof AclItem) {
						action = new Action() {
							public void run() {
								FileInfo file = ((AclItem) selection.getFirstElement()).getFile(true);
								Program.launch(file.determineFullPath());
							}
						};
						action.setText("Open file in Explorer");
						action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.FOLDER_EXPLORER_16X16));
						manager.add(action);
					}
				}
			}
		});
		
		Menu menu = menuMgr.createContextMenu(viewer.getTree());
		viewer.getTree().setMenu(menu);
	}	
	
	@Focus
	public void setFocus() {
		viewer.getControl().setFocus();		
	}
	
	public AclItem getSelectedAclItem() {
		if (viewer != null && !viewer.getSelection().isEmpty()) {
			IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
			if (selection.getFirstElement() instanceof AclItem) {
				return (AclItem) selection.getFirstElement();
			}
		}	
		return null;
	}
	
	public List<AclItem> getAclItems() {
		return ((AclPartContentProvider)viewer.getContentProvider()).getAclItems();
	}
	
	public boolean isApplyFilter() {
		return fileDetailModel.isApplyFilter();
	}
}
