package com.perdemia.permissionanalyzer.application.parts;

import java.awt.Color;
import java.lang.reflect.InvocationTargetException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.DecimalFormat;
import java.util.Iterator;
import java.util.Map.Entry;

import javax.annotation.PostConstruct;
import javax.inject.Inject;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.services.EMenuService;
import org.eclipse.e4.ui.services.IStylingEngine;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Text;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.CategoryLabelPositions;
import org.jfree.chart.labels.PieSectionLabelGenerator;
import org.jfree.chart.labels.StandardPieSectionLabelGenerator;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PiePlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.dial.DialBackground;
import org.jfree.chart.plot.dial.DialCap;
import org.jfree.chart.plot.dial.DialPlot;
import org.jfree.chart.plot.dial.DialTextAnnotation;
import org.jfree.chart.plot.dial.DialValueIndicator;
import org.jfree.chart.plot.dial.StandardDialFrame;
import org.jfree.chart.plot.dial.StandardDialRange;
import org.jfree.chart.plot.dial.StandardDialScale;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.data.general.DefaultPieDataset;
import org.jfree.data.general.DefaultValueDataset;
import org.jfree.data.general.PieDataset;
import org.jfree.data.general.ValueDataset;
import org.jfree.experimental.chart.swt.ChartComposite;
import org.jfree.ui.RectangleInsets;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.dialogs.SearchFolderDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.application.util.PermissionUtil;
import com.perdemia.permissionanalyzer.data.AclItem;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.DBObject;
import com.perdemia.permissionanalyzer.data.FileInfo;
import com.perdemia.permissionanalyzer.data.Member;
import com.perdemia.permissionanalyzer.data.PermissionLookup;
import com.perdemia.permissionanalyzer.data.PermissionLookup.SimplePermission;
import com.perdemia.permissionanalyzer.data.util.CustomProcessor;
import com.perdemia.permissionanalyzer.data.util.DBUtil;
import com.perdemia.permissionanalyzer.data.util.DBUtil.DbType;
import com.perdemia.permissionanalyzer.data.util.FileCache;

public class AuditPartDemo {
	private static final Logger logger = LoggerFactory.getLogger(AuditPartDemo.class.getName());
	public static final String ID = "com.perdemia.permissionanalyzer.application.parts.audit";
	
	private static final Color CHART_BACKGROUND = new Color(249, 249, 249);
	
	private Combo categoryCombo;
	private Composite[] chartComposites = new Composite[6];
	private CLabel[] chartTitles = new CLabel[6];
	private Button[] runButtons = new Button[6];
	private FileInfo selectedFolder;
	private ProgressMonitorDialog progressDialog = new ProgressMonitorDialog(Display.getCurrent().getActiveShell());
	private int selectedCategory;
	
	@Inject private IStylingEngine engine;
		
	@PostConstruct
	public void createComposite(Composite parent, EMenuService menuService) {
				
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.verticalSpacing = 0;
		gridLayout.horizontalSpacing = 0;
		parent.setLayout(gridLayout);
		
		Composite topComposite = new Composite(parent, SWT.NONE);
		gridLayout = new GridLayout(9, false);
		gridLayout.marginBottom = 10;
		topComposite.setLayout(gridLayout);
		
		//selected folders
		CLabel directoryLabel = new CLabel(topComposite, SWT.NONE);
		directoryLabel.setText("Directory:");
		engine.setClassname(directoryLabel, "h2");
		directoryLabel.setLayoutData(new GridData());
		
		final Text directoryText = new Text(topComposite, SWT.READ_ONLY | SWT.BORDER);
		directoryText.setText("All");
		GridData gridData = new GridData();
		gridData.widthHint = 400;
		directoryText.setLayoutData(gridData);
		
		Button searchFolderButton = new Button(topComposite, SWT.PUSH);
		searchFolderButton.setImage(ImageFactory.getInstance().getImage(AppImage.SEARCH_16X16));
		searchFolderButton.setToolTipText("Select folder");
		searchFolderButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		searchFolderButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				SearchFolderDialog searchDialog = new SearchFolderDialog(Display.getCurrent().getActiveShell());
				int result = searchDialog.open();
				if (result == SearchFolderDialog.OK) {
					selectedFolder = searchDialog.getSelectedFolder();
					directoryText.setText(searchDialog.getSelectedPath());
				}
			}
		});
		
		Button clearFolderButton = new Button(topComposite, SWT.PUSH);
		clearFolderButton.setImage(ImageFactory.getInstance().getImage(AppImage.DELETE_16X16));
		clearFolderButton.setToolTipText("Clear selected folder");
		clearFolderButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		clearFolderButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectedFolder = null;
				directoryText.setText("All");
			}
		});
		
		Composite spacer = new Composite(topComposite, SWT.NONE);
		gridData = new GridData();
		gridData.widthHint = 30;
		gridData.heightHint = 10;
		spacer.setLayoutData(gridData);
		
		CLabel categoryLabel = new CLabel(topComposite, SWT.NONE);
		categoryLabel.setText("Audit category:");
		engine.setClassname(categoryLabel, "h2");
		gridData = new GridData(SWT.RIGHT, SWT.CENTER, true, false);
		categoryLabel.setLayoutData(gridData);
		
		categoryCombo = new Combo(topComposite, SWT.BORDER | SWT.READ_ONLY);
		categoryCombo.setLayoutData(new GridData());
		categoryCombo.add("Users and groups");
		categoryCombo.add("Permissions");
		categoryCombo.add("Files and folders");
		categoryCombo.select(0);
		categoryCombo.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectedCategory = categoryCombo.getSelectionIndex();
			}
		});		
		
		Button runAllButton = new Button(topComposite, SWT.PUSH);
		runAllButton.setImage(ImageFactory.getInstance().getImage(AppImage.RUN_16X16));
		runAllButton.setText("Build all charts");
		runAllButton.setToolTipText("Build dashboard");
		runAllButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		runAllButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				clearCharts();
				try {
					progressDialog.run(true, true, new IRunnableWithProgress(){
						@Override
						public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
							int totalWork = 0;
							totalWork += getWorkUnitsForChart(0);
							totalWork += getWorkUnitsForChart(1);
							totalWork += getWorkUnitsForChart(2);
							totalWork += getWorkUnitsForChart(3);
							totalWork += getWorkUnitsForChart(4);
							totalWork += getWorkUnitsForChart(5);
							
							monitor.beginTask("Building charts", totalWork);
							
							for (int counter = 0; counter < 6; counter++) {
								buildChart(counter, monitor);
																
								if (monitor.isCanceled()) {
									break;
								}
							}
						}
					});
				} catch (InvocationTargetException | InterruptedException ex) {
					logger.error("Error while building charts", ex);
					SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while building charts: " + ex.getMessage());
				}
			}
		});
		
		Button clearButton = new Button(topComposite, SWT.PUSH);
		clearButton.setImage(ImageFactory.getInstance().getImage(AppImage.DELETE_16X16));
		clearButton.setText("Reset all charts");
		clearButton.setToolTipText("Reset dashboard");
		clearButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		clearButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				clearCharts();
			}
		});
		
		Composite chartsComposite = new Composite(parent, SWT.BORDER);
		gridLayout = new GridLayout(3, false);
		gridLayout.horizontalSpacing = 25;
		gridLayout.verticalSpacing = 25;
		gridLayout.marginLeft = 10;
		gridLayout.marginRight = 10;
		
		chartsComposite.setLayout(gridLayout);
		chartsComposite.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		// chart containers
		for (int counter = 0; counter < 6; counter++) {
			// Container for the refresh button, title and chart
			final Composite chartComposite = new Composite(chartsComposite, SWT.BORDER);
			chartComposite.setLayout(new GridLayout(1, false));
			chartComposite.setLayoutData(new GridData(GridData.FILL_BOTH));
			chartComposites[counter] = chartComposite;
			
			// Composite that holds the refresh button and title placeholder
			Composite placeholderComposite = new Composite(chartComposite, SWT.NONE);
			placeholderComposite.setLayout(new GridLayout(1, false));
			placeholderComposite.setLayoutData(new GridData(GridData.FILL_BOTH));
			
			// Chart title
			final CLabel title = new CLabel(placeholderComposite, SWT.NONE);
			title.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			title.setAlignment(SWT.CENTER);
			engine.setClassname(title, "h1");
			chartTitles[counter] = title;
			
			// Button to manually refresh the chart
			final Button refreshChartButton = new Button(placeholderComposite, SWT.PUSH);
			refreshChartButton.setImage(ImageFactory.getInstance().getImage(AppImage.RUN_16X16));
			refreshChartButton.setData(counter);
			refreshChartButton.setText("Build chart");
			refreshChartButton.setLayoutData(new GridData(GridData.CENTER, GridData.CENTER, true, true));
			runButtons[counter] = refreshChartButton;
			
			refreshChartButton.addSelectionListener(new SelectionAdapter(){
				final int buttonIndex = (Integer) refreshChartButton.getData();
				
				@Override
				public void widgetSelected(SelectionEvent e) {
					try {
						progressDialog.run(true, true, new IRunnableWithProgress(){
							@Override
							public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {								
								monitor.beginTask("Building chart", getWorkUnitsForChart(buttonIndex));
								buildChart(buttonIndex, monitor);
							}
						});
					} catch (InvocationTargetException | InterruptedException ex) {
						logger.error("Error while building chart", ex);
						SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while building charts: " + ex.getMessage());
					}
				}
			});
		}
		
		
		clearCharts();
		 //File barChart3D = new File( "barChart3D.jpeg" );                            
	     //ChartUtilities.saveChartAsJPEG( barChart3D, barChart, width, height);
	}
	
	private void buildChart(int containerIndex, IProgressMonitor monitor) {
		Composite chartComposite = chartComposites[containerIndex];
		//progressDialog.getProgressMonitor().subTask("Building chart \"" + chartTitles[containerIndex].getText() + "\"");
		monitor.setTaskName("Building chart \"" + chartTitles[containerIndex].getText() + "\"");
		
		// determine the right chart to build
		switch (selectedCategory) {
			case 0: // users and groups
				switch (containerIndex) {
					case 0:
						createUsersGroupsPieChart(chartComposite, monitor);
						break;
					case 1:
						createTopExplicitUserPermissionsBarChart(chartComposite, monitor);
						break;
					case 2:
						createTopExplicitGroupPermissionsBarChart(chartComposite, monitor);
						break;
					case 3:
						createTopGroupsWithMostMembers(chartComposite, monitor);
						break;
					case 4:
						createTopUserMemberships(chartComposite, monitor);
						break;					
					case 5:
						createGroupComplexityDialChart(chartComposite, monitor);
						break;
				}
				break;
			
			case 1: // permissions
				switch (containerIndex) {
					case 0:
						createPermissionTypeBarChart(chartComposite, monitor);
						break;
					case 1:
						createPermissionTypePieChart(chartComposite, monitor);
						break;
					case 2:
						createExplicitUserPermissionsPerFile(chartComposite, monitor);
						break;
					case 3:
						createExplicitGroupPermissionsPerFile(chartComposite, monitor);
						break;
					case 4:
						createUsersWithExplicitPermissions(chartComposite, monitor);
						break;
					case 5:
						createGroupsWithExplicitPermissions(chartComposite, monitor);
						break;
				}
				break;
				
			case 2: // files and directories
				switch (containerIndex) {
					case 0:
						createFilesDirectoriesPieChart(chartComposite, monitor);
						break;
					case 1:
						createUsersGroupsOnFileSystemPieChart(chartComposite, monitor);
						break;
					case 2:
						createFolderWithMostChildrenChart(chartComposite, monitor);
						break;
					case 3:
						createAvgAcesPerFile(chartComposite, monitor);
						break;
					case 4:
						createFolderWithMostExplicitPermissionsChart(chartComposite, monitor);
						break;
					case 5:
						createFolderWithMostExplicitUserPermissionsChart(chartComposite, monitor);
						break;
				}
				break;
		}
	}
	
	/**
	 * Returns the number of work units to be used in the IProgressMonitor. This method determines what
	 * chart is being build and knows how many database calls are required to build the chart.
	 * 
	 * @param containerIndex
	 * @return number of work units
	 */
	private int getWorkUnitsForChart(int containerIndex) {
		// determine the right chart to build
		switch (selectedCategory) {
			case 0: // users and groups
				switch (containerIndex) {
					case 0:
						return 6; //createUsersGroupsPieChart
					case 1:
						return 1; //createTopExplicitUserPermissionsBarChart
					case 2:
						return 1; //createTopExplicitGroupPermissionsBarChart
					case 3:
						return 1; //createTopGroupsWithMostMembers
					case 4:
						return 1; //createTopUserMemberships
					case 5:
						return 2; //createGroupComplexityDialChart
				}
				break;
			
			case 1: // permissions
				switch (containerIndex) {
					case 0:
						return 6; //createPermissionTypeBarChart
					case 1:
						return 7; //createPermissionTypePieChart
					case 2:
						return 2; //createExplicitUserPermissionsPercentage
					case 3:
						return 2; //createExplicitGroupPermissionsPercentage
					case 4:
						return 2; //createUsersWithExplicitPermissions
					case 5:
						return 2; //createGroupsWithExplicitPermissions
				}
				break;
				
			case 2: // files and directories
				switch (containerIndex) {
					case 0:
						return 2; //createFilesDirectoriesPieChart
					case 1:
						return 6; //createUsersGroupsOnFileSystemPieChart
					case 2:
						return 1; //createFolderWithMostChildrenChart
					case 3:
						return 2; //createAvgAcesPerFile
					case 4:
						return 2; //createFolderWithMostExplicitPermissionsChart
					case 5:
						return 2; //createFolderWithMostExplicitUserPermissionsChart
				}
				break;
		}
		
		return 0;
	}
	
	/**
	 * Replaces the title labels with the correct ones based on the selected category. It also removed
	 * the charts from the chart containers and makes the refresh button composite visible.
	 */
	private void clearCharts() {
		
		// set the right title in the placeholder composites
		switch (selectedCategory) {
			case 0: // users and groups
				chartTitles[0].setText("Users and groups from LDAP");
				chartTitles[1].setText("Top 25 of user with most explicit ACE's");
				chartTitles[2].setText("Top 25 of groups with most explicit ACE's");
				chartTitles[3].setText("Top 25 of groups with most members");
				chartTitles[4].setText("Top 25 of users with most group memberships");
				chartTitles[5].setText("Group complexity: direct vs indirect memberships)");				
				break;
				
			case 1: // permissions
				chartTitles[0].setText("ACE's");
				chartTitles[1].setText("ACE's");
				chartTitles[2].setText("Explicit user ACE's per file");
				chartTitles[3].setText("Explicit group ACE's per file"); 
				chartTitles[4].setText("Number of users with explicit permissions"); 
				chartTitles[5].setText("Number of explicit ACE's per group");				
				break;
				
			case 2: // files and directories
				chartTitles[0].setText("Files and directories");
				chartTitles[1].setText("Users and groups on the file system");
				chartTitles[2].setText("Top 25 of folders with most direct sub files/folders");
				chartTitles[3].setText("Avg. ACE's per file");
				chartTitles[4].setText("Top 25 of folders with most explicit ACE's");
				chartTitles[5].setText("Top 25 of folders with most explicit user ACE's");
		}
		
		// dispose the current charts and make the run button visible
		for (int counter = 0; counter < 6; counter++) {
			if (chartComposites[counter].getChildren().length > 1) {
				chartComposites[counter].getChildren()[1].dispose();
			}
			
			chartComposites[counter].getChildren()[0].setVisible(true);
			((GridData)chartComposites[counter].getChildren()[0].getLayoutData()).exclude = false;
			chartComposites[counter].layout();
		}
	}
	
	/**
	 * Creates a bar chart that shows the balance between permissions. It counts the number of each simple permission and
	 * splits the count into allow, deny, explicit and inherited.
	 * 
	 * @param chartsComposite
	 * @param monitor
	 */
    private void createPermissionTypeBarChart(Composite chartsComposite, IProgressMonitor monitor) {
    	final DefaultCategoryDataset dataset = new DefaultCategoryDataset();
    	Iterator<Entry<SimplePermission, Integer>> permissions = PermissionLookup.simplePermissionValues.entrySet().iterator();
    	
    	String baseQuery;
		if (selectedFolder == null) {
			baseQuery = "select count(*) from permissionanalyzer.aclitem where permission_id = ?";
		} else {
			baseQuery = "select count(*) from permissionanalyzer.aclitem inner join permissionanalyzer.fileinfo on aclitem.fileinfo_id = fileinfo.id "
					+ "where permission_id = ? and " + getSelectedFolderFilter();
		}
		
    	while (permissions.hasNext()) {
    		Entry<SimplePermission, Integer> entry = permissions.next();
    		String label = PermissionUtil.simplePermissionToString(entry.getKey());
    		
    		String suffix = "";
    		if (entry.getKey().equals(SimplePermission.READ_EXECUTE)) {
    			suffix = " and flag_file_inherit = true";
    		}
    		if (entry.getKey().equals(SimplePermission.LIST_FOLDER_CONTENT)) {
    			suffix = " and flag_file_inherit = false";
    		}
    		
    		monitor.subTask("Counting number of explicit " + label + " permissions...");
    		Integer result = getCount(baseQuery + " and FLAG_INHERITED_ACE = false" + suffix, entry.getValue());
            dataset.addValue(result, "Explicit", label);
            if (monitor.isCanceled()) return;
            
            monitor.subTask("Counting number of inherited " + label + " permissions...");
            result = getCount(baseQuery + " and FLAG_INHERITED_ACE = true" + suffix, entry.getValue());
            dataset.addValue(result, "Inherited", label);
            if (monitor.isCanceled()) return;
            
            monitor.subTask("Counting number of allow " + label + " permissions...");
            result = getCount(baseQuery + " and IS_DENY = false" + suffix, entry.getValue());
            dataset.addValue(result, "Allow", label);
            if (monitor.isCanceled()) return;
            
            monitor.subTask("Counting number of deny " + label + " permissions...");
            result = getCount(baseQuery + " and IS_DENY = true" + suffix, entry.getValue());
            dataset.addValue(result, "Deny", label);
            if (monitor.isCanceled()) return;
            monitor.worked(1);
    	}
	    
    	addBarChart("ACE's", "Type", "ACE Count", dataset, false, chartsComposite);
    }
    
    /**
	 * Creates a pie chart that shows the balance between all the simple permissions (including a category 'Special').
	 * 
	 * @param chartsComposite
	 * @param monitor
	 */
    private void createPermissionTypePieChart(Composite chartsComposite, IProgressMonitor monitor) {
    	final DefaultPieDataset dataset = new DefaultPieDataset();
    	Iterator<Entry<SimplePermission, Integer>> permissions = PermissionLookup.simplePermissionValues.entrySet().iterator();
    	int totalSimpelPermissions = 0;
    	
    	String baseQuery;
		if (selectedFolder == null) {
			baseQuery = "select count(*) from permissionanalyzer.aclitem where permission_id = ?";
		} else {
			baseQuery = "select count(*) from permissionanalyzer.aclitem inner join permissionanalyzer.fileinfo on aclitem.fileinfo_id = fileinfo.id "
					+ "where permission_id = ? and " + getSelectedFolderFilter();
		}
				
    	while (permissions.hasNext()) {
    		Entry<SimplePermission, Integer> entry = permissions.next();
    		String label = PermissionUtil.simplePermissionToString(entry.getKey());
    		
    		String suffix = "";
    		if (entry.getKey().equals(SimplePermission.READ_EXECUTE)) {
    			suffix = " and flag_file_inherit = true";
    		}
    		if (entry.getKey().equals(SimplePermission.LIST_FOLDER_CONTENT)) {
    			suffix = " and flag_file_inherit = false";
    		}
    		
    		monitor.subTask("Counting number of " + label + " permissions...");
    		Integer result = getCount(baseQuery + suffix, entry.getValue());
    		totalSimpelPermissions += result;
    		dataset.setValue(label, result);
            if (monitor.isCanceled()) return;
            
            monitor.worked(1);
    	}
    	
    	monitor.subTask("Counting number of special permissions...");
    	int aceCount = getAclCount();
    			
    	dataset.setValue("Special", aceCount - totalSimpelPermissions);
		monitor.worked(1);
    	
    	addPieChart("ACE's", dataset, chartsComposite);
    }
    
    /**
     * Creates a chart that displays the percentage of explicit user permissions (explicit user permissions / number of files)
     * 
     * @param chartsComposite
     * @param monitor
     */
    private void createExplicitUserPermissionsPerFile(Composite chartsComposite, IProgressMonitor monitor) {
    	monitor.subTask("Counting explicit user permissions ...");
    	
    	String explicitUserPermissionsQuery;
		if (selectedFolder == null) {
			explicitUserPermissionsQuery = "select count(*) from permissionanalyzer.aclitem inner join permissionanalyzer.member on aclitem.member_id = member.id where FLAG_INHERITED_ACE = false and member.is_group = false";
		} else {
			explicitUserPermissionsQuery = "select count(*) from permissionanalyzer.aclitem inner join permissionanalyzer.member on aclitem.member_id = member.id "
					+ "inner join permissionanalyzer.fileinfo on aclitem.fileinfo_id = fileinfo.id where FLAG_INHERITED_ACE = false and member.is_group = false and " + getSelectedFolderFilter();
		}
		
    	int numberOfExplicitUserPermissions = getCount(explicitUserPermissionsQuery);
    	monitor.worked(1);
    	
    	monitor.subTask("Counting number of directories and files ...");
    	int fileCount = getFileCount();
		monitor.worked(1);
    	
    	double percentage  = 0;
    	
    	if (fileCount > 0) {
    		percentage = (double)numberOfExplicitUserPermissions / (double)fileCount;
    	}
    	    	
    	addDialChart("Explicit user ACE's (" + numberOfExplicitUserPermissions + ") / files (" + fileCount + ")" , "ACE's per file", percentage, chartsComposite);
    }
        
    /**
     * Creates a chart that displays the percentage of explicit group permissions (explicit group permissions / number of files)
     * 
     * @param chartsComposite
     * @param monitor
     */
    private void createExplicitGroupPermissionsPerFile(Composite chartsComposite, IProgressMonitor monitor) {
    	monitor.subTask("Counting explicit group permissions ...");
    	
    	String explicitGroupPermissionsQuery;
		if (selectedFolder == null) {
			explicitGroupPermissionsQuery = "select count(*) from permissionanalyzer.aclitem inner join permissionanalyzer.member on aclitem.member_id = member.id where FLAG_INHERITED_ACE = false and member.is_group = true";
		} else {
			explicitGroupPermissionsQuery = "select count(*) from permissionanalyzer.aclitem inner join permissionanalyzer.member on aclitem.member_id = member.id "
					+ "inner join permissionanalyzer.fileinfo on aclitem.fileinfo_id = fileinfo.id where FLAG_INHERITED_ACE = false and member.is_group = true and " + getSelectedFolderFilter();
		}
		
    	int numberOfExplicitGroupPermissions = getCount(explicitGroupPermissionsQuery);
    	monitor.worked(1);
    	
    	monitor.subTask("Counting number of directories and files ...");
		int fileCount = getFileCount();		
    	monitor.worked(1);
    	
    	double percentage  = 0;
    	
    	if (fileCount > 0) {
    		percentage = (double)numberOfExplicitGroupPermissions / (double)fileCount;
    	}
    	    	
    	addDialChart("Explicit group ACE's (" + numberOfExplicitGroupPermissions + ") / files (" + fileCount + ")" , "ACE's per file", percentage, chartsComposite);
    }
    
    /**
     * Creates a chart that displays the percentage of users with explicit permissions
     * 
     * @param chartsComposite
     * @param monitor
     */
    private void createUsersWithExplicitPermissions(Composite chartsComposite, IProgressMonitor monitor) {
    	monitor.subTask("Counting users with explicit permissions ...");
    	
    	String explicitPermissionsUsersQuery;
		if (selectedFolder == null) {
			explicitPermissionsUsersQuery = "select count(distinct member_id) from permissionanalyzer.aclitem inner join permissionanalyzer.member on aclitem.member_id = member.id where FLAG_INHERITED_ACE = false and member.is_group = false";
		} else {
			explicitPermissionsUsersQuery = "select count(distinct member_id) from permissionanalyzer.aclitem inner join permissionanalyzer.member on aclitem.member_id = member.id inner join "
					+ "permissionanalyzer.fileinfo on aclitem.fileinfo_id = fileinfo.id where FLAG_INHERITED_ACE = false and member.is_group = false and " + getSelectedFolderFilter();
		}
		
    	int numberOfUsersWithExplicitPermissions = getCount(explicitPermissionsUsersQuery);
    	monitor.worked(1);
    	
    	monitor.subTask("Counting number of users ...");
    	int userCount = getCount("select count(*) from permissionanalyzer.member where is_group = false");
    	monitor.worked(1);
    	
    	double percentage  = 0;
    	
    	if (userCount > 0) {
    		percentage = (double)numberOfUsersWithExplicitPermissions / (double)userCount * 100;
    	}
    	    	
    	addDialChart("Users with explicit permissions (" + numberOfUsersWithExplicitPermissions + ") / total users (" + userCount + ")" , "Percentage", percentage, chartsComposite);
    }
    
    /**
     * Creates a chart that displays the percentage of groups with explicit permissions
     * 
     * @param chartsComposite
     * @param monitor
     */
    private void createGroupsWithExplicitPermissions(Composite chartsComposite, IProgressMonitor monitor) {
    	monitor.subTask("Counting groups with explicit permissions ...");
    	
    	String explicitPermissionsGroupsQuery;
		if (selectedFolder == null) {
			explicitPermissionsGroupsQuery = "select count(distinct member_id) from permissionanalyzer.aclitem inner join permissionanalyzer.member on aclitem.member_id = member.id where FLAG_INHERITED_ACE = false and member.is_group = true";
		} else {
			explicitPermissionsGroupsQuery = "select count(distinct member_id) from permissionanalyzer.aclitem inner join permissionanalyzer.member on aclitem.member_id = member.id inner join "
					+ "permissionanalyzer.fileinfo on aclitem.fileinfo_id = fileinfo.id where FLAG_INHERITED_ACE = false and member.is_group = true and " + getSelectedFolderFilter();
		}
		
    	int numberOfGroupsWithExplicitPermissions = getCount(explicitPermissionsGroupsQuery);
    	monitor.worked(1);
    	
    	monitor.subTask("Counting number of groups ...");
    	int groupCount = getCount("select count(*) from permissionanalyzer.member where is_group = true");
    	monitor.worked(1);
    	
    	double percentage  = 0;
    	
    	if (groupCount > 0) {
    		percentage = (double)numberOfGroupsWithExplicitPermissions / (double)groupCount * 100;
    	}
    	    	
    	addDialChart("Groups with explicit permissions (" + numberOfGroupsWithExplicitPermissions + ") / total groups (" + groupCount + ")" , "Percentage", percentage, chartsComposite);
    }
    
    /**
     * Creates a chart with the different group types and the number of users
     * 	
     * @return A chart.
     * @throws DBException 
     * @throws SQLException 
     */
    private void createUsersGroupsPieChart(Composite chartsComposite, IProgressMonitor monitor) {
    	DefaultPieDataset dataset = new DefaultPieDataset();
    	String query = "select count(*) from permissionanalyzer.member where group_type = ?";
    	
    	monitor.subTask("Counting number of local groups...");
    	dataset.setValue("Local groups", 78);
    	if (monitor.isCanceled()) return;
    	monitor.worked(1);
    	
    	monitor.subTask("Counting number of domain local groups...");
    	dataset.setValue("Domain local groups", 134);
    	if (monitor.isCanceled()) return;
    	monitor.worked(1);
    	
    	monitor.subTask("Counting number of global groups...");
    	dataset.setValue("Global groups", 76);
    	if (monitor.isCanceled()) return;
    	monitor.worked(1);
    	
    	monitor.subTask("Counting number of universal groups...");
    	dataset.setValue("Universal groups", 53);
    	if (monitor.isCanceled()) return;
    	monitor.worked(1);
    	
    	monitor.subTask("Counting number of domain built-in groups...");
    	dataset.setValue("Domain built-in groups", 48);
    	if (monitor.isCanceled()) return;
    	monitor.worked(1);
    	
    	monitor.subTask("Counting number of users...");
    	dataset.setValue("Users", 859);
    	if (monitor.isCanceled()) return;
    	monitor.worked(1);
    	
        addPieChart("Users and groups from LDAP", dataset, chartsComposite);
    }
    
    /**
     * Creates a chart with the different group types and the number of users found on the file system
     * 	
     * @return A chart.
     * @throws DBException 
     * @throws SQLException 
     */
    private void createUsersGroupsOnFileSystemPieChart(Composite chartsComposite, IProgressMonitor monitor) {
    	DefaultPieDataset dataset = new DefaultPieDataset();
    	
    	String query;
    	if (selectedFolder == null) {
    		query = "select distinct count(*) from permissionanalyzer.aclitem inner join permissionanalyzer.member on aclitem.member_id = member.id where member.group_type = ?";
    	} else {
    		query = "select distinct count(*) from permissionanalyzer.aclitem inner join permissionanalyzer.member on aclitem.member_id = member.id "
    				+ "inner join permissionanalyzer.fileinfo on aclitem.fileinfo_id = fileinfo.id where member.group_type = ? and " + getSelectedFolderFilter();
    	}
    	
    	monitor.subTask("Counting number of permissions for local groups...");
    	dataset.setValue("Local groups (ACE's)", getCount(query, Member.GroupType.COMPUTER_LOCAL.ordinal()));
    	if (monitor.isCanceled()) return;
    	monitor.worked(1);
    	
    	monitor.subTask("Counting number of permissions for domain local groups...");
    	dataset.setValue("Domain local groups (ACE's)", getCount(query, Member.GroupType.DOMAIN_LOCAL.ordinal()));
    	if (monitor.isCanceled()) return;
    	monitor.worked(1);
    	
    	monitor.subTask("Counting number of permissions for global groups...");
    	dataset.setValue("Global groups (ACE's)", getCount(query, Member.GroupType.GLOBAL.ordinal()));
    	if (monitor.isCanceled()) return;
    	monitor.worked(1);
    	
    	monitor.subTask("Counting number of permissions for universal groups...");
    	dataset.setValue("Universal groups (ACE's)", getCount(query, Member.GroupType.UNIVERSAL.ordinal()));
    	if (monitor.isCanceled()) return;
    	monitor.worked(1);
    	
    	monitor.subTask("Counting number of permissions for domain built-in groups...");
    	dataset.setValue("Domain built-in groups (ACE's)", getCount(query, Member.GroupType.DOMAIN_BUILTIN.ordinal()));
    	if (monitor.isCanceled()) return;
    	monitor.worked(1);
    	
    	monitor.subTask("Counting number of permissions for users...");
    	dataset.setValue("Users (ACE's)", getCount("select distinct count(*) from permissionanalyzer.aclitem inner join permissionanalyzer.member on aclitem.member_id = member.id where member.group_type = 0 or member.group_type is null"));
    	if (monitor.isCanceled()) return;
    	monitor.worked(1);
    	
        addPieChart("Users and groups on the file system", dataset, chartsComposite);
    }
    
    /**
     * Creates a bar chart with the top 25 of users with the most explicit permissions.
     */
    private void createTopExplicitUserPermissionsBarChart(Composite chartsComposite, IProgressMonitor monitor) {
    	final DefaultCategoryDataset dataset = new DefaultCategoryDataset();
    	
    	String query;
    	if (selectedFolder == null) {
    		query = "select member.name, count(*) from permissionanalyzer.aclitem inner join permissionanalyzer.member on aclitem.member_id = member.id where flag_inherited_ace = false and member.is_group = false and member.is_builtin = false";
    	} else {
    		query = "select member.name, count(*) from permissionanalyzer.aclitem inner join permissionanalyzer.member on aclitem.member_id = member.id inner join "
    				+ "permissionanalyzer.fileinfo on aclitem.fileinfo_id = fileinfo.id where flag_inherited_ace = false and member.is_group = false and member.is_builtin = false and " + getSelectedFolderFilter();
    	}
    	
    	query = DBObject.addLimitAndOrderBy(query, "count(*) desc", "aclitem.member_id, member.name", 25);
    	
    	monitor.subTask("Counting explicit user permissions...");
    	
    	dataset.addValue(76, "Explicit ACE's", "Gene Ball");
    	dataset.addValue(63, "Explicit ACE's", "Brynn Walker");
    	dataset.addValue(58, "Explicit ACE's", "Caden Price");
    	dataset.addValue(55, "Explicit ACE's", "Jamie Evans");
    	dataset.addValue(53, "Explicit ACE's", "Kerry Taylor");
    	dataset.addValue(52, "Explicit ACE's", "Harper Maynard");
    	dataset.addValue(51, "Explicit ACE's", "Alexis Buck");
    	dataset.addValue(48, "Explicit ACE's", "Shay Warren");
    	dataset.addValue(47, "Explicit ACE's", "Jesse Byrd");
    	dataset.addValue(44, "Explicit ACE's", "Logan Marsh");
    	dataset.addValue(41, "Explicit ACE's", "Tanner Cook");
    	dataset.addValue(40, "Explicit ACE's", "River Hawkins");
    	dataset.addValue(35, "Explicit ACE's", "Blake Evans");
    	dataset.addValue(32, "Explicit ACE's", "Justice Mcdonald");
    	dataset.addValue(28, "Explicit ACE's", "Brook Simpson");
    	dataset.addValue(27, "Explicit ACE's", "Lane Osborn");
    	dataset.addValue(25, "Explicit ACE's", "Steff Mckay");
    	dataset.addValue(24, "Explicit ACE's", "Denny Kaufman");
    	dataset.addValue(22, "Explicit ACE's", "Rene Murray");
    	dataset.addValue(20, "Explicit ACE's", "Ali Hinton");
    	dataset.addValue(17, "Explicit ACE's", "Vic Adams");
    	dataset.addValue(16, "Explicit ACE's", "Jesse Cook");
    	dataset.addValue(14, "Explicit ACE's", "Frankie Booth");
    	dataset.addValue(12, "Explicit ACE's", "Brice King");
    	dataset.addValue(12, "Explicit ACE's", "Cory Clarke");
    	
    	monitor.worked(1);
    	
    	addBarChart("Top 25 of users with most explicit ACE's", "Users", "ACE Count", dataset, true, chartsComposite);
    }
    
    /**
     * Creates a bar chart with the top 25 of users with the most group memberships.
     */
    private void createTopUserMemberships(Composite chartsComposite, IProgressMonitor monitor) {
    	final DefaultCategoryDataset dataset = new DefaultCategoryDataset();
    	
    	String query = "select member.name, count(*) from permissionanalyzer.membership inner join permissionanalyzer.member on membership.member_id = member.id where member.is_group = false";
    	query = DBObject.addLimitAndOrderBy(query, "count(*) desc", "membership.member_id, member.name", 25);
    	
    	monitor.subTask("Counting group memberships...");
    	dataset.addValue(48, "Group memberships", "Raylee Richardson");
    	dataset.addValue(45, "Group memberships", "Logan James");
    	dataset.addValue(38, "Group memberships", "Marley Hunt");
    	dataset.addValue(37, "Group memberships", "Kiran Fraser");
    	dataset.addValue(35, "Group memberships", "Gale Matthews");
    	dataset.addValue(33, "Group memberships", "Steff Schultz");
    	dataset.addValue(31, "Group memberships", "Blake Walsh");
    	dataset.addValue(28, "Group memberships", "Maddox Emerson");
    	dataset.addValue(27, "Group memberships", "Bev Watkins");
    	dataset.addValue(25, "Group memberships", "Morgan Cervantes");
    	dataset.addValue(23, "Group memberships", "Rudy Owen");
    	dataset.addValue(18, "Group memberships", "Blair Davies");
    	dataset.addValue(16, "Group memberships", "Alex Stone");
    	dataset.addValue(15, "Group memberships", "Jessie Robertson");
    	dataset.addValue(13, "Group memberships", "Quinn Perry");
    	dataset.addValue(13, "Group memberships", "Angel Cotton");
    	dataset.addValue(12, "Group memberships", "Kiran Walsh");
    	dataset.addValue(12, "Group memberships", "Shay Brewer");
    	dataset.addValue(11, "Group memberships", "Reed Mitchell");
    	dataset.addValue(10, "Group memberships", "Cameron Carr");
    	dataset.addValue(10, "Group memberships", "Jess Young");
    	dataset.addValue(10, "Group memberships", "Lee Hill");
    	dataset.addValue(9, "Group memberships", "Rene Wallace");
    	dataset.addValue(9, "Group memberships", "Addison Booth");
    	dataset.addValue(9, "Group memberships", "Skye Walsh");
    	
    	monitor.worked(1);
    	
    	addBarChart("Top 25 of users with most nested group memberships", "Users", "Nested membership Count", dataset, true, chartsComposite);
    }
    
    /**
     * Creates a bar chart with the top 25 of groups with the most members.
     */
    private void createTopGroupsWithMostMembers(Composite chartsComposite, IProgressMonitor monitor) {
    	final DefaultCategoryDataset dataset = new DefaultCategoryDataset();
    	
    	String query = "select member.name, count(*) from permissionanalyzer.membership inner join permissionanalyzer.member on membership.group_id = member.id";
    	query = DBObject.addLimitAndOrderBy(query, "count(*) desc", "membership.group_id, member.name", 25);
    	
    	monitor.subTask("Counting group members...");
    	
    	dataset.addValue(802, "Group members", "Employees");
    	dataset.addValue(763, "Group members", "Intranet Users");
    	dataset.addValue(234, "Group members", "Sales");
    	dataset.addValue(128, "Group members", "Finance Administration");
    	dataset.addValue(96, "Group members", "Project managers");
    	dataset.addValue(85, "Group members", "Marketing");
    	dataset.addValue(73, "Group members", "Freelancers");
    	dataset.addValue(68, "Group members", "HR");
    	dataset.addValue(61, "Group members", "Project Office");
    	dataset.addValue(56, "Group members", "Secretary");
    	dataset.addValue(51, "Group members", "Clients");
    	dataset.addValue(50, "Group members", "Project planning");
    	dataset.addValue(47, "Group members", "Product designers");
    	dataset.addValue(43, "Group members", "Callcenter");
    	dataset.addValue(42, "Group members", "Coaches");
    	dataset.addValue(40, "Group members", "Management");
    	dataset.addValue(36, "Group members", "Data analysts");
    	dataset.addValue(35, "Group members", "Technical support");
    	dataset.addValue(32, "Group members", "App testers");    	
    	dataset.addValue(31, "Group members", "Sales Support");
    	dataset.addValue(30, "Group members", "Quality auditors");
    	dataset.addValue(25, "Group members", "Deployment team");
    	dataset.addValue(22, "Group members", "Intranet Admins");
    	dataset.addValue(15, "Group members", "Database admins");
    	dataset.addValue(8, "Group members", "Intranet Developers");
    	   	
    	addBarChart("Top 25 of groups with most members", "Groups", "Member Count", dataset, true, chartsComposite);
    }
    
    /**
     * Creates a bar chart with the top 25 of groups with the most explicit permissions.
     */
    private void createTopExplicitGroupPermissionsBarChart(Composite chartsComposite, IProgressMonitor monitor) {
    	final DefaultCategoryDataset dataset = new DefaultCategoryDataset();
    	
    	String query;
    	if (selectedFolder == null) {
    		query = "select member.name, count(*) from permissionanalyzer.aclitem inner join permissionanalyzer.member on aclitem.member_id = member.id where flag_inherited_ace = false and member.is_group = true and member.is_builtin = false";
    	} else {
    		query = "select member.name, count(*) from permissionanalyzer.aclitem inner join permissionanalyzer.member on aclitem.member_id = member.id inner join "
    				+ "permissionanalyzer.fileinfo on aclitem.fileinfo_id = fileinfo.id where flag_inherited_ace = false and member.is_group = true and member.is_builtin = false and " + getSelectedFolderFilter();
    	}
    	
    	query = DBObject.addLimitAndOrderBy(query, "count(*) desc", "aclitem.member_id, member.name", 25);
    	
    	dataset.addValue(286, "Explicit ACE's", "Employees");
    	dataset.addValue(244, "Explicit ACE's", "Intranet users");
    	dataset.addValue(235, "Explicit ACE's", "Management");
    	dataset.addValue(230, "Explicit ACE's", "Quality auditors");
    	dataset.addValue(218, "Explicit ACE's", "Technical support");
    	dataset.addValue(178, "Explicit ACE's", "Freelancers");
    	dataset.addValue(155, "Explicit ACE's", "Finance administration");
    	dataset.addValue(135, "Explicit ACE's", "Sales");
    	dataset.addValue(126, "Explicit ACE's", "Clients");
    	dataset.addValue(122, "Explicit ACE's", "Callcenter");
    	dataset.addValue(115, "Explicit ACE's", "Coaches");
    	dataset.addValue(105, "Explicit ACE's", "Secretary");
    	dataset.addValue(98, "Explicit ACE's", "Project managers");
    	dataset.addValue(95, "Explicit ACE's", "Marketing");
    	dataset.addValue(95, "Explicit ACE's", "Project office");
    	dataset.addValue(87, "Explicit ACE's", "Deployment team");
    	dataset.addValue(82, "Explicit ACE's", "Intranet admins");
    	dataset.addValue(82, "Explicit ACE's", "App testers");    	
    	dataset.addValue(78, "Explicit ACE's", "HR");
    	dataset.addValue(75, "Explicit ACE's", "Database admins");
    	dataset.addValue(66, "Explicit ACE's", "Intranet developers");
    	dataset.addValue(63, "Explicit ACE's", "Project planning");
    	dataset.addValue(58, "Explicit ACE's", "Product designers");
    	dataset.addValue(57, "Explicit ACE's", "Data analysts");
    	dataset.addValue(48, "Explicit ACE's", "Visitors");
    	dataset.addValue(41, "Explicit ACE's", "Sales support");
    	
    	monitor.worked(1);
    	
    	addBarChart("Top 25 of groups with most explicit ACE's", "Groups", "ACE Count", dataset, true, chartsComposite);
    }
    
    /**
     * Creates a chart with the number of files and the number of directories
     *      * 	
     * @return A chart.
     * @throws DBException 
     * @throws SQLException 
     */
    private void createFilesDirectoriesPieChart(Composite chartsComposite, IProgressMonitor monitor) {
        
    	DefaultPieDataset dataset = new DefaultPieDataset();
    	
    	monitor.subTask("Counting directories...");
    	int numberOfDirectories;
    	if (selectedFolder == null) {
    		numberOfDirectories = getCount("select count(*) from permissionanalyzer.fileinfo where IS_DIRECTORY = true");
    	} else {
    		numberOfDirectories = getCount("select count(*) from permissionanalyzer.fileinfo where IS_DIRECTORY = true and " + getSelectedFolderFilter());
    	}
    	
    	dataset.setValue("Directories", numberOfDirectories);
    	monitor.worked(1);
    	
		monitor.subTask("Counting files...");
		dataset.setValue("Files", getFileCount() - numberOfDirectories);	
    	monitor.worked(1);
    	
    	addPieChart("Files and directories", dataset, chartsComposite);
    }
    
    /**
     * Creates a dial chart showing the number of indirect memberships vs direct memberships.
     * 
     * @param chartsComposite
     * @param monitor
     */
    private void createGroupComplexityDialChart(Composite chartsComposite, IProgressMonitor monitor) {
    	monitor.subTask("Counting indirect group memberships ...");
    	int numberOfIndirectMemberships = getCount("select count(*) from permissionanalyzer.membership where group_id <> member_id and VIA_GROUP_ID is not null");
    	monitor.worked(1);
    	
    	monitor.subTask("Counting total number of memberships ...");
    	int numberOfMemberships = getCount("select count(*) from permissionanalyzer.membership where group_id <> member_id");
    	monitor.worked(1);
    	
    	double percentage  = 0;
    	
    	numberOfIndirectMemberships = 6798;
    	numberOfMemberships = 13454;
    	
    	if (numberOfMemberships > 0) {
    		percentage = (double)numberOfIndirectMemberships / (double)numberOfMemberships * 100;
    	}
    	    	
    	
    	addDialChart("Group complexity: Indirect memberships (" + numberOfIndirectMemberships + ") / all memberships (" + numberOfMemberships + ")" , "Percentage", percentage, chartsComposite);
    }
    
    /**
     * Creates a top 25 of folders with the most <b>direct</b> sub files/folders
     * 
     * @param chartsComposite
     * @param monitor
     */
    private void createFolderWithMostChildrenChart(Composite chartsComposite, IProgressMonitor monitor) {
    	final DefaultCategoryDataset dataset = new DefaultCategoryDataset();
    	
    	String query;
    	if (selectedFolder == null) {
    		query = "select count(*), parent_id from permissionanalyzer.fileinfo";
    	} else {
    		query = "select count(*), parent_id from permissionanalyzer.fileinfo where " + getSelectedFolderFilter();
    	}
    	query = DBObject.addLimitAndOrderBy(query, "count(*) desc", "fileinfo.parent_id", 25);
    	
    	monitor.subTask("Getting folders with most direct sub files/folders...");
    	Connection connection = null;
		PreparedStatement ps = null;
		
    	try {
    		if (DBUtil.getDbType().equals(DbType.MYSQL)) {
				// MySQL cannot handle two open RecordSets on one connection. Since we loop through the record set to save memory and
				// we query new information in this loop, we need a new DB connection in case of MySQL
				connection = DBUtil.getDBConnection(false);
				ps = connection.prepareStatement(query, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
				ps.setFetchSize(Integer.MIN_VALUE);
				
			} else {
				// use the existing connection and create a new PreparedStatement
				connection = DBUtil.getDBConnection();
				ps = connection.prepareStatement(query);
			}
    		
	    	DBObject.getUsingCustomProcessor(ps, new CustomProcessor(){
				@Override
				public boolean processResultSet(ResultSet rs) throws SQLException, DBException {
					// we have the parent_id, get the corresponding file name
					FileInfo fileInfo = FileCache.getFile(rs.getInt(2));
															
					dataset.addValue(rs.getInt(1), "Sub files/folders", fileInfo.determineFullPath());					
					return true;
				}
	    		
	    	});
    	} catch (Exception ex) {
    		logger.error("Error while building folder chart.", ex);
    		SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while building folder chart: " + ex.getMessage());
    		
    	} finally {
    		// In case of MySQL a new connection has been created, close it
    		try {
    			if (DBUtil.getDbType().equals(DbType.MYSQL) && connection != null) {
					connection.close();
    			}
			} catch (Exception e) {
				logger.error("Error while closing database connection in createFolderWithMostChildrenChart", e);
			}
    	}
    	monitor.worked(1);
    	
    	addBarChart("Top 25 of folders with the most direct sub files/folders", "Folders", "Sub files/folders count", dataset, true, chartsComposite);
    }

    /**
     * Creates a top 25 of folders with the most <b>direct</b> sub files/folders
     * 
     * @param chartsComposite
     * @param monitor
     */
    private void createFolderWithMostExplicitUserPermissionsChart(Composite chartsComposite, IProgressMonitor monitor) {
    	final DefaultCategoryDataset dataset = new DefaultCategoryDataset();
    	
    	String query;
    	if (selectedFolder == null) {
    		query = "select count(*),fileinfo_id from permissionanalyzer.aclitem inner join permissionanalyzer.member on aclitem.member_id = member.id where is_group = false and flag_inherited_ace = false";
    	} else {
    		query = "select count(*),fileinfo_id from permissionanalyzer.aclitem inner join permissionanalyzer.member on aclitem.member_id = member.id inner join "
    				+ "permissionanalyzer.fileinfo on aclitem.fileinfo_id = fileinfo.id where is_group = false and flag_inherited_ace = false and " + getSelectedFolderFilter();
    	}
    	query = DBObject.addLimitAndOrderBy(query, "count(*) desc", "aclitem.fileinfo_id", 25);
    	
    	try {
			query = DBObject.checkQueryForDialect(query);
		} catch (DBException e) {
			logger.error("Error while checking database dialect.", e.getMessage());
			SyncMessageDialog.showMessage(MessageDialog.ERROR, "Database error", "Error while checking database dialect: " + e.getMessage());
		}
    	
    	monitor.subTask("Getting folders with most explicit user permissions...");
    	Connection connection = null;
		PreparedStatement ps = null;
		
    	try {
    		if (DBUtil.getDbType().equals(DbType.MYSQL)) {
				// MySQL cannot handle two open RecordSets on one connection. Since we loop through the record set to save memory and
				// we query new information in this loop, we need a new DB connection in case of MySQL
				connection = DBUtil.getDBConnection(false);
				ps = connection.prepareStatement(query, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
				ps.setFetchSize(Integer.MIN_VALUE);
				
			} else {
				// use the existing connection and create a new PreparedStatement
				connection = DBUtil.getDBConnection();
				ps = connection.prepareStatement(query);
			}
    		
	    	DBObject.getUsingCustomProcessor(ps, new CustomProcessor(){
				@Override
				public boolean processResultSet(ResultSet rs) throws SQLException, DBException {
					// we have the parent_id, get the corresponding file name
					FileInfo fileInfo = FileCache.getFile(rs.getInt(2));
								
					dataset.addValue(rs.getInt(1), "Explicit user ACE's", fileInfo.determineFullPath());					
					return true;
				}
	    		
	    	});
    	} catch (Exception ex) {
    		logger.error("Error while building folder chart.", ex);
    		SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while building folder chart: " + ex.getMessage());
    		
    	} finally {
    		// In case of MySQL a new connection has been created, close it
    		try {
    			if (DBUtil.getDbType().equals(DbType.MYSQL) && connection != null) {
					connection.close();
    			}
			} catch (Exception e) {
				logger.error("Error while closing database connection in createFolderWithMostExplicitUserPermissionsChart", e);
			}
    	}
    	monitor.worked(1);
    	
    	addBarChart("Top 25 of folders with the most explicit user ACE's", "Folders", "Explicit user ACE count", dataset, true, chartsComposite);
    }
    
    /**
     * Creates a top 25 of folders with the most <b>direct</b> sub files/folders
     * 
     * @param chartsComposite
     * @param monitor
     */
    private void createFolderWithMostExplicitPermissionsChart(Composite chartsComposite, IProgressMonitor monitor) {
    	final DefaultCategoryDataset dataset = new DefaultCategoryDataset();
    	
    	String query;
    	if (selectedFolder == null) {
    		query = "select count(*),fileinfo_id from permissionanalyzer.aclitem where flag_inherited_ace = false";
    	} else {
    		query = "select count(*),fileinfo_id from permissionanalyzer.aclitem inner join permissionanalyzer.fileinfo on aclitem.fileinfo_id = fileinfo.id "
    				+ "where flag_inherited_ace = false and " + getSelectedFolderFilter();
    	}
    	query = DBObject.addLimitAndOrderBy(query, "count(*) desc", "aclitem.fileinfo_id", 25);
    	try {
			query = DBObject.checkQueryForDialect(query);
		} catch (DBException e) {
			logger.error("Error while checking database dialect.", e.getMessage());
			SyncMessageDialog.showMessage(MessageDialog.ERROR, "Database error", "Error while checking database dialect: " + e.getMessage());
		}
    	
    	monitor.subTask("Getting folders with most explicit permissions...");
    	Connection connection = null;
		PreparedStatement ps = null;
		
    	try {
    		if (DBUtil.getDbType().equals(DbType.MYSQL)) {
				// MySQL cannot handle two open RecordSets on one connection. Since we loop through the record set to save memory and
				// we query new information in this loop, we need a new DB connection in case of MySQL
				connection = DBUtil.getDBConnection(false);
				ps = connection.prepareStatement(query, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
				ps.setFetchSize(Integer.MIN_VALUE);
				
			} else {
				// use the existing connection and create a new PreparedStatement
				connection = DBUtil.getDBConnection();
				ps = connection.prepareStatement(query);
			}
    		
	    	DBObject.getUsingCustomProcessor(ps, new CustomProcessor(){
				@Override
				public boolean processResultSet(ResultSet rs) throws SQLException, DBException {
					// we have the parent_id, get the corresponding file name
					FileInfo fileInfo = FileCache.getFile(rs.getInt(2));
								
					dataset.addValue(rs.getInt(1), "Explicit ACE's", fileInfo.determineFullPath());					
					return true;
				}
	    		
	    	});
    	} catch (Exception ex) {
    		logger.error("Error while building folder chart.", ex);
    		SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while building folder chart: " + ex.getMessage());
    		
    	} finally {
    		// In case of MySQL a new connection has been created, close it
    		try {
    			if (DBUtil.getDbType().equals(DbType.MYSQL) && connection != null) {
					connection.close();
    			}
			} catch (Exception e) {
				logger.error("Error while closing database connection in createFolderWithMostExplicitPermissionsChart", e);
			}
    	}
    	monitor.worked(1);
    	
    	addBarChart("Top 25 of folders with the most explicit ACE's", "Folders", "Explicit ACE count", dataset, true, chartsComposite);
    }
    
    /**
     * Creates a chart that displays a dial chart with the average number of ACE's per file
     * 
     * @param chartsComposite
     * @param monitor
     */
    private void createAvgAcesPerFile(Composite chartsComposite, IProgressMonitor monitor) {   	
    	monitor.subTask("Counting number of ACE's ...");
		int aceCount = getAclCount();
    	monitor.worked(1);
    	
    	monitor.subTask("Counting number of directories and files ...");
		int fileCount = getFileCount();	
    	monitor.worked(1);
    	
    	double acesPerFile  = 0;
    	
    	if (fileCount > 0) {
    		acesPerFile = (double)aceCount / (double)fileCount;
    	}
    	    	
    	addDialChart("ACE's (" + aceCount + ") / files (" + fileCount + ")" , "Average ACE's per file", acesPerFile, chartsComposite, 10, 20, 30);
    }
    
    /**
     * Adds some configuration values to the given PiePlot, like padding, line visibility and foreground alpha and
     * adds the chart to the chartsComposite.
     * 
     * @param plot
     */
    private void addPieChart(final String title, final PieDataset dataset, final Composite chartComposite) {
    	Display.getDefault().syncExec(new Runnable(){
    		public void run() {
		    	// clear the refresh button and title placeholder
		    	chartComposite.getChildren()[0].setVisible(false);
		    	((GridData)chartComposite.getChildren()[0].getLayoutData()).exclude = true;
		    	
		    	JFreeChart chart = ChartFactory.createPieChart(title, dataset, true, true, false);
		    	//chart.setBackgroundImageAlpha(0f);
		    	chart.setBackgroundPaint(CHART_BACKGROUND);
		    	PiePlot plot = (PiePlot) chart.getPlot();
		    	
		    	// configure plot
		    	plot.setStartAngle( 270 );             
		        plot.setForegroundAlpha( 0.70f );
		        //plot.setInteriorGap( 0.1 );
		        
		        plot.setSectionOutlinesVisible(false);
		        plot.setLabelPadding(new RectangleInsets(5,5,5,5));
		        plot.setNoDataMessage("No data available");
		        plot.setCircular(false);
		        plot.setLabelGap(0.02);
		        plot.setBackgroundAlpha(0);
		        plot.setOutlineVisible(false);
		                
		        PieSectionLabelGenerator gen = new StandardPieSectionLabelGenerator("{0}: {1} ({2})", new DecimalFormat("0"), new DecimalFormat("0%"));
		        plot.setLabelGenerator(gen);
		        
		//        TextTitle source = new TextTitle(
		//        		"Source: Java In A Nutshell (4th Edition) "
		//        		+ "by David Flanagan (OReilly)"
		//        		);
		//        		//source.setFont(new Font("SansSerif", Font.PLAIN, 10));
		//        		source.setPosition(RectangleEdge.BOTTOM);
		//        		source.setHorizontalAlignment(HorizontalAlignment.RIGHT);
		//        		chart.addSubtitle(source);
		        
		        // add the chart to the Composite
		        //ChartComposite frame = new ChartComposite(parent, SWT.NONE, chart, 1550, 1000, 1000, 800, 1500, 1000, true, true, true, true, true, true);
		  		ChartComposite frame = new ChartComposite(chartComposite, SWT.NONE, chart, true);
		  		GridData gridData = new GridData(GridData.FILL_BOTH);
		  		frame.setLayoutData(gridData);   
		  		
		  		chartComposite.layout();
    		}
    	});
    }
    
    /**
     * Creates and configures a BarChart and add the chart to the chartsComposite.
     * 
     * @param title
     * @param xLabel
     * @param yLabel
     * @param dataset
     */
    private void addBarChart(final String title, final String xLabel, final String yLabel, final CategoryDataset dataset, final boolean rotateLabels, final Composite chartComposite) {
    	Display.getDefault().syncExec(new Runnable(){
    		public void run() {
		    	// clear the refresh button and title placeholder
		    	chartComposite.getChildren()[0].setVisible(false);    	
		    	((GridData)chartComposite.getChildren()[0].getLayoutData()).exclude = true;
		    	
		    	JFreeChart barChart = ChartFactory.createBarChart3D(title, xLabel, yLabel, dataset, PlotOrientation.VERTICAL, true, true, false);
			    barChart.setBackgroundPaint(CHART_BACKGROUND);
			    barChart.setBackgroundImageAlpha(0f);
			    	    
			    CategoryPlot plot = (CategoryPlot) barChart.getPlot();
			    plot.setForegroundAlpha( 0.75f );             
				plot.setNoDataMessage("No data available");
				plot.setBackgroundPaint(Color.BLUE);
				plot.setBackgroundAlpha(0.1f);
				plot.setOutlineVisible(false);
				
				if (rotateLabels) {
					CategoryAxis axis = barChart.getCategoryPlot().getDomainAxis();
					axis.setCategoryLabelPositions(CategoryLabelPositions.UP_45);
				}
				
				// add the chart to the Composite
		        //ChartComposite frame = new ChartComposite(parent, SWT.NONE, chart, 1550, 1000, 1000, 800, 1500, 1000, true, true, true, true, true, true);
		  		ChartComposite frame = new ChartComposite(chartComposite, SWT.NONE, barChart, true);
		  		GridData gridData = new GridData(GridData.FILL_BOTH);
		  		frame.setLayoutData(gridData);   
		  		
		  		chartComposite.layout();
    		}
    	});
	}
    
    /**
     * Adds some configuration values to the given PiePlot, like padding, line visibility and foreground alpha and
     * adds the chart to the chartsComposite.
     * 
     * @param plot
     */
    private void addDialChart(final String title, final String subtitle, final double value, final Composite chartComposite) {
    	addDialChart(title, subtitle, value, chartComposite, 30, 70, 100);
    }
    
    /**
     * Adds some configuration values to the given PiePlot, like padding, line visibility and foreground alpha and
     * adds the chart to the chartsComposite.
     * 
     * @param plot
     */
    private void addDialChart(final String title, final String subtitle, final double value, final Composite chartComposite, final double low, final double middle, final double max) {
    	Display.getDefault().syncExec(new Runnable(){
    		public void run() {
		    	// clear the refresh button and title placeholder
		    	chartComposite.getChildren()[0].setVisible(false);
		    	((GridData)chartComposite.getChildren()[0].getLayoutData()).exclude = true;
		    	
		    	// determine the factor to 'zoom in'
		    	double zoominFactor = 0;
		    	String valueString = String.valueOf(value);
		    	for (int counter = 0; counter  < valueString.length(); counter++) {
		    		if (valueString.charAt(counter) == '0' || valueString.charAt(counter) == '.') {
		    			zoominFactor++;
		    		} else {
		    			break;
		    		}
		    	}
		    	
		    	zoominFactor = Math.pow(10D, zoominFactor);
		    	ValueDataset dataset = new DefaultValueDataset(value * zoominFactor);
		    	
		    	String newSubtitle = subtitle;
		    	if (zoominFactor > 1) {
		    		newSubtitle += "( / " + (int)zoominFactor + ")"; 
		    	}
		    	
		    	// DialPlot with the scale 0 - 100
				DialPlot dialplot = new DialPlot();
				dialplot.setDataset(dataset);
				dialplot.setDialFrame(new StandardDialFrame());
				dialplot.setBackground(new DialBackground());
				DialTextAnnotation dialtextannotation = new DialTextAnnotation(newSubtitle);
				dialtextannotation.setRadius(0.69999999999999996D);
				dialplot.addLayer(dialtextannotation);
				DialValueIndicator dialvalueindicator = new DialValueIndicator(0);
				dialplot.addLayer(dialvalueindicator);
				StandardDialScale standarddialscale = new StandardDialScale(0D, max, -120D, -300D, 10D, 4);
				standarddialscale.setTickRadius(0.88D);
				standarddialscale.setTickLabelOffset(0.14999999999999999D);
				dialplot.addScale(0, standarddialscale);
				dialplot.addPointer(new org.jfree.chart.plot.dial.DialPointer.Pin());
				DialCap dialcap = new DialCap();
				dialplot.setCap(dialcap);
				
				JFreeChart chart = new JFreeChart(title, dialplot);
				
				// color ranges
				StandardDialRange standarddialrange = new StandardDialRange(middle, max, Color.red);
				standarddialrange.setInnerRadius(0.52000000000000002D);
				standarddialrange.setOuterRadius(0.55000000000000004D);
				dialplot.addLayer(standarddialrange);
				StandardDialRange standarddialrange1 = new StandardDialRange(low, middle, Color.orange);
				standarddialrange1.setInnerRadius(0.52000000000000002D);
				standarddialrange1.setOuterRadius(0.55000000000000004D);
				dialplot.addLayer(standarddialrange1);
				StandardDialRange standarddialrange2 = new StandardDialRange(0, low, Color.green);
				standarddialrange2.setInnerRadius(0.52000000000000002D);
				standarddialrange2.setOuterRadius(0.55000000000000004D);
				dialplot.addLayer(standarddialrange2);
				
				chart.setBackgroundPaint(CHART_BACKGROUND);
				
				// pointer (the arrow)
				dialplot.removePointer(0);
				org.jfree.chart.plot.dial.DialPointer.Pointer pointer = new org.jfree.chart.plot.dial.DialPointer.Pointer();
				pointer.setFillPaint(Color.yellow);
				dialplot.addPointer(pointer);
   
		        // add the chart to the Composite
		        //ChartComposite frame = new ChartComposite(parent, SWT.NONE, chart, 1550, 1000, 1000, 800, 1500, 1000, true, true, true, true, true, true);
		  		ChartComposite frame = new ChartComposite(chartComposite, SWT.NONE, chart, true);
		  		GridData gridData = new GridData(GridData.FILL_BOTH);
		  		frame.setLayoutData(gridData);   
		  		
		  		chartComposite.layout();
    		}
    	});
    }
    
    
    /**
     * Executes a count query on the database and returns the result. The method takes care of closing the ResultSet and Statement.
     * 
     * @param query count query to execute
     * @param params ps parameters
     * @return count value
     */
    private Integer getCount(String query, Object ... params) {
    	Integer result = 0;
    	
    	ResultSet rs = null;
		try {
			rs = DBObject.executeQuery(query, params);
	        if (rs.next()){
	        	result = rs.getInt(1);
	        };
	        rs.close();
		} catch (DBException | SQLException e) {
			logger.error("Error while getting count for audit dashboard, query: " + query, e);
			SyncMessageDialog.showMessage(MessageDialog.ERROR, "Database error", "Error while executing query on the database: " + e.getMessage());
		} finally {
			try {
				rs.close();
			} catch (Exception e) {
				logger.warn("Could not close database connection", e);
			}
		}
		
        return result;
    }
    
    /**
     * Returns the number of records in the AclItem table. In case {@link #selectedFolder} is not null it will return the count of
     * all AclItems that are attached to a (indirect) sub file/folder of the selectedFolder.
     * 
     * @return number of AclItems
     */
    private Integer getAclCount() {
    	int aceCount = 0;
    	if (selectedFolder == null) {
    		try {
				aceCount = AclItem.getCount();
			} catch (DBException e) {
				logger.error("Could not determine the total number of ACE's.", e);
				SyncMessageDialog.showMessage(MessageDialog.ERROR, "Database error", "Error while executing query on the database: " + e.getMessage());
			}
    	} else {
    		aceCount = getCount("select count(*) from permissionanalyzer.aclitem inner join permissionanalyzer.fileinfo on aclitem.fileinfo_id = fileinfo.id where " + getSelectedFolderFilter());;
    	}
    	
    	return aceCount;
    }
    
    /**
     * Returns the number of records in the FileInfo table. In case {@link #selectedFolder} is not null it will return the count of
     * all (indirect) sub files/folders of the selectedFolder.
     * 
     * @return number of FileInfo records
     */
    private Integer getFileCount() {
    	int fileCount = 0;
    	if (selectedFolder == null) {
    		try {
				fileCount = FileInfo.getCount();
			} catch (DBException e) {
				logger.error("Could not determine the total number of files.", e);
				SyncMessageDialog.showMessage(MessageDialog.ERROR, "Database error", "Error while executing query on the database: " + e.getMessage());
			}
    	} else {
    		fileCount = getCount("select count(*) from permissionanalyzer.fileinfo where " + getSelectedFolderFilter());;
    	}
    	
    	return fileCount;
    }
    
    private String getSelectedFolderFilter() {
    	return " parent_file_ids like '" + selectedFolder.getParentFileIds() + ",%'";
    }
    
	/**
	 * Passing the focus request to the viewer's control.
	 */
	@Focus
	public void setFocus() {
		
	}
}