package com.perdemia.permissionanalyzer.application.parts;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.inject.Inject;
import javax.inject.Named;

import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MToolBarElement;
import org.eclipse.e4.ui.model.application.ui.menu.impl.ToolControlImpl;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.e4.ui.workbench.modeling.ESelectionService;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.TreeViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.TraverseEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Text;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.handlers.toolbar.ApplyFilterToolControl;
import com.perdemia.permissionanalyzer.application.jface.contentproviders.AclPartContentProvider;
import com.perdemia.permissionanalyzer.application.jface.contentproviders.EffectivePermissionsPartContentProvider;
import com.perdemia.permissionanalyzer.application.jface.contentproviders.EffectivePermissionsPartContentProviderConfig;
import com.perdemia.permissionanalyzer.application.jface.contentproviders.MainFileTreeContentProvider;
import com.perdemia.permissionanalyzer.application.jface.dialogs.MemberInfoDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SearchMembersDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.jface.labelproviders.GroupOwnershipColumnLabelProvider;
import com.perdemia.permissionanalyzer.application.jface.labelproviders.SpecialPermissionLabelProvider;
import com.perdemia.permissionanalyzer.application.preferences.PreferenceHelper;
import com.perdemia.permissionanalyzer.application.swt.DelayedKeyListenerText;
import com.perdemia.permissionanalyzer.application.swt.ExplainLabel;
import com.perdemia.permissionanalyzer.application.swt.ExplainLabel.ExplainType;
import com.perdemia.permissionanalyzer.application.util.ApplicationUtil;
import com.perdemia.permissionanalyzer.application.util.ColorFactory;
import com.perdemia.permissionanalyzer.application.util.ColorFactory.AppColor;
import com.perdemia.permissionanalyzer.application.util.Constants;
import com.perdemia.permissionanalyzer.application.util.FilterHelper;
import com.perdemia.permissionanalyzer.application.util.FilterHelper.ObjectType;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.application.util.PermissionUtil;
import com.perdemia.permissionanalyzer.data.AclItem;
import com.perdemia.permissionanalyzer.data.FileInfo;
import com.perdemia.permissionanalyzer.data.Member;
import com.perdemia.permissionanalyzer.data.PermissionLookup.SpecialPermission;

public class EffectivePermissionsPart {
	public static final String ID = "com.perdemia.permissionanalyzer.application.parts.effectivepermissions";
	private static Logger logger = LoggerFactory.getLogger(EffectivePermissionsPart.class.getName());	
	
	private TreeViewer viewer;
	private EffectivePermissionsPartContentProviderConfig contentProviderConfig = new EffectivePermissionsPartContentProviderConfig(null, true, "", false, FilterHelper.getInstance());
	private FileInfo selectedFile;
	private Text searchText;
	
	/** Controls for the ownership ratio labels */
	private Composite ownershipRatioComposite;
	private Link allMatchingMembersOnFolderLink;
	private Link allMembersOnFolderLink;
	private Label allMembersOnFolderLabel;
	
	private Link allMatchingMembersOnFolderLink2;
	private Link allMembersFromFilterLink;
	private Label allMatchingMembersOnFolderLabel2;
	
	private int paramCountOfLastFilterQuery;
	
	private static final Color greyColor = ColorFactory.getInstance().getColor(AppColor.GREY); 
	
	@Inject ESelectionService selectionService;
	@Inject EPartService partService;
	
	/**
	 * Receives selection events from the main FileInfo tree and updates the input for the {@link AclPartContentProvider}
	 */
	@Inject
	public void fileSelected(MApplication application, EModelService modelService, @Optional @Named(IServiceConstants.ACTIVE_SELECTION) IStructuredSelection selectedItem) {
		boolean fileChanged = false;
		
		// This line is necessary because when the users switches between the AclPart and TracePart this method is called
		// with an empty selectedPath
		selectedItem = (IStructuredSelection) selectionService.getSelection(FilePart.ID);
				
		if (selectedItem != null && !selectedItem.isEmpty() && selectedItem.getFirstElement() instanceof FileInfo) {
		
			// determine if the Apply Filter check button has been checked in the ToolBar
			boolean applyFilter = false;
			MPart part = partService.findPart("com.perdemia.permissionanalyzer.application.parts.effectivepermissions");
			if (part != null) {
				for (MToolBarElement element : part.getToolbar().getChildren()) {
					if (element.getElementId().equals("com.perdemia.permissionanalyzer.application.parts.effectivepermissions.toolbar.applyFilter")) {
						//if (((ToolControlImpl)element).getObject() != null) {
							ApplyFilterToolControl toolItem = (ApplyFilterToolControl) ((ToolControlImpl)element).getObject();
							if (toolItem != null) {
								applyFilter = toolItem.isChecked();
							}
						//} else {
						//	applyFilter = true;
						//}
					}
				}
			}
			
			// check on object reference of selectedFile (we do want to refresh when the filters have changed and the file id is the same!)
			fileChanged = selectedFile != selectedItem.getFirstElement() || contentProviderConfig.isApplyFilter() != applyFilter;
			
			// fill the model with the selected file
			selectedFile = (FileInfo) selectedItem.getFirstElement();
						
			contentProviderConfig.setSelectedFile(selectedFile);
			contentProviderConfig.setApplyFilter(applyFilter);
			
			if (PreferenceHelper.getInstance().isCalculateOwnershipRatioInEffectivePermissionsTab()) {
				
				// data not yet loaded by the file tree/table?
				if (!PreferenceHelper.getInstance().isCalculateOwnershipRatioInOverview()) {
					MainFileTreeContentProvider.determineOwnershipRatio(FilterHelper.getInstance(), selectedFile);;
				}
				
				if (FilterHelper.getInstance().isApplyMemberFilter()) { // member filters selected? Beside the member config filter
										
					// total ownership ratio
					allMatchingMembersOnFolderLink.setText("<a>" + String.valueOf(selectedFile.getAllMatchingMembersOnFolder()) + "</a>");
					allMembersOnFolderLink.setText("<a>" + String.valueOf(selectedFile.getAllMembersOnFolder()) + "</a>");
					allMembersOnFolderLabel.setText(" total members on the file = " + GroupOwnershipColumnLabelProvider.OWNERSHIP_RATIO_FORMAT.format(((double)selectedFile.getAllMatchingMembersOnFolder() / (double)selectedFile.getAllMembersOnFolder() * 100)) + "%");
					
					// group ownership ratio
					allMembersFromFilterLink.setText("<a>" + String.valueOf(FilterHelper.getInstance().getAllFilteredMembers()) + "</a>");
					allMatchingMembersOnFolderLink2.setText("<a>" + String.valueOf(selectedFile.getAllMatchingMembersOnFolder()) + "</a>");
					allMatchingMembersOnFolderLabel2.setText(" members found in the group filters = " + GroupOwnershipColumnLabelProvider.OWNERSHIP_RATIO_FORMAT.format(((double)selectedFile.getAllMatchingMembersOnFolder() / (double)FilterHelper.getInstance().getAllFilteredMembers() * 100)) + "%");
					
					((GridData)ownershipRatioComposite.getLayoutData()).exclude = false;
					ownershipRatioComposite.getParent().layout(true);
					ownershipRatioComposite.setVisible(true);
					ownershipRatioComposite.layout(true);
				} else {
					// hide the labels
					((GridData)ownershipRatioComposite.getLayoutData()).exclude = true;
					ownershipRatioComposite.getParent().layout(true);
					ownershipRatioComposite.setVisible(false);
				}
			}
			
		} else {
			// no file selected or found, clear the view
			contentProviderConfig.setSelectedFile(null);
			fileChanged = true;
		}
		
		if (viewer != null && !viewer.getTree().isDisposed() && fileChanged) {
			viewer.setInput(contentProviderConfig);
			viewer.refresh();
			viewer.getTree().getParent().layout(true);
			
			// auto expand the tree when the user filters on one member
			if (FilterHelper.getInstance().isFilterOnSingleMember() != null) {
				
				List<Member> directMembers = getContentProvider().getDirectMembers();
				if (!directMembers.isEmpty()) {
					viewer.expandToLevel(directMembers.get(0), 4);
				}				
			}
		}
	}
		
	@PostConstruct
	public void createComposite(Composite parent) {
		// View layout
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.verticalSpacing = 5;
		gridLayout.horizontalSpacing = 0;
		parent.setLayout(gridLayout);

		new ExplainLabel(parent, "This tab displays all the members that are found on the selected file. For each member, the effective permissions are displayed and unfolding a member will display all the applicable explicit Access Control Entries and (if applicable) members of a group. To view the inherited permissions for a particular user or group, you can put a member in the Trace tab using the context menu.", ExplainType.MEMBERS_TAB);
		
		// search box
		Composite searchComposite = new Composite(parent, SWT.NONE);
		searchComposite.setLayoutData(new GridData());
		searchComposite.setLayout(new GridLayout(2, false));
		
		searchText = DelayedKeyListenerText.create(searchComposite, SWT.BORDER, new KeyAdapter() {
			@Override
			public void keyReleased(KeyEvent e) {
				if (searchText.getText().length() > 2) {
					searchMembers();
				}
			}			
		});
		searchText.setMessage("search user or group");
		GridData gridData = new GridData();
		gridData.widthHint = 300;
		searchText.setLayoutData(gridData);
		searchText.addTraverseListener(new TraverseListener(){
			@Override
			public void keyTraversed(TraverseEvent e) {
				searchMembers();
			}
		});
			
		Button searchButton = new Button(searchComposite, SWT.PUSH);
		searchButton.setImage(ImageFactory.getInstance().getImage(AppImage.SEARCH_USER_16X16));
		searchButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				searchMembers();
			}
		});
		
		if (PreferenceHelper.getInstance().isCalculateOwnershipRatioInEffectivePermissionsTab()) {
			// composite with ownership ratio info
			ownershipRatioComposite = new Composite(parent, SWT.NONE);
			gridData = new GridData();
			gridData.heightHint = 30;
			ownershipRatioComposite.setLayoutData(gridData);
			ownershipRatioComposite.setLayout(new GridLayout(10, false));
			
			// Total ownership ratio
			Label totalOwnershipRatioLabel = new Label(ownershipRatioComposite, SWT.NONE);
			totalOwnershipRatioLabel.setText("Total ownership ratio: ");
			totalOwnershipRatioLabel.setData(Constants.CSS_CLASS_NAME_KEY, "h2");
					
			allMatchingMembersOnFolderLink = new Link(ownershipRatioComposite, SWT.NONE);
			allMatchingMembersOnFolderLink.addListener(SWT.Selection,new Listener(){
			    public void handleEvent(Event event) {
			    	if (FilterHelper.getInstance().getFilterConfig(false).isIncludeGroupMembership()) {
			    		SearchMembersDialog searchDialog = new SearchMembersDialog(Display.getCurrent().getActiveShell(), getMembersPS(0, selectedFile, true, true), paramCountOfLastFilterQuery);
			    		searchDialog.setUseSelectButton(false);
			    		searchDialog.open();
			    	} else {
			    		SearchMembersDialog dialog = new SearchMembersDialog(null, selectedFile.getDirectMembers());
			    		dialog.open();
			    	}
			    }
			});
			
			Label allMatchingMembersOnFolderLabel = new Label(ownershipRatioComposite, SWT.NONE);
			allMatchingMembersOnFolderLabel.setText(" matching members on the file   / ");
			
			allMembersOnFolderLink = new Link(ownershipRatioComposite, SWT.NONE);
			allMembersOnFolderLink.addListener(SWT.Selection,new Listener(){
			    public void handleEvent(Event event) {
			    	if (FilterHelper.getInstance().getFilterConfig(false).isIncludeGroupMembership()) {
			    		SearchMembersDialog searchDialog = new SearchMembersDialog(Display.getCurrent().getActiveShell(), getMembersPS(0, selectedFile, true, false), paramCountOfLastFilterQuery);
			    		searchDialog.setUseSelectButton(false);
			    		searchDialog.open();
			    	} else {
			    		SearchMembersDialog dialog = new SearchMembersDialog(null, selectedFile.getAllDirectMembers());
			    		dialog.open();
			    	}
			    }
			});
			
			allMembersOnFolderLabel = new Label(ownershipRatioComposite, SWT.NONE);
									
			// Group ownership ratio
			Label groupOwnershipRatioLabel = new Label(ownershipRatioComposite, SWT.NONE);
			groupOwnershipRatioLabel.setText("    Group ownership ratio: ");
			groupOwnershipRatioLabel.setData(Constants.CSS_CLASS_NAME_KEY, "h2");
			
			allMatchingMembersOnFolderLink2 = new Link(ownershipRatioComposite, SWT.NONE);
			allMatchingMembersOnFolderLink2.addListener(SWT.Selection,new Listener(){
			    public void handleEvent(Event event) {
			    	if (FilterHelper.getInstance().getFilterConfig(false).isIncludeGroupMembership()) {
			    		SearchMembersDialog searchDialog = new SearchMembersDialog(Display.getCurrent().getActiveShell(), getMembersPS(0, selectedFile, true, true), paramCountOfLastFilterQuery);
			    		searchDialog.setUseSelectButton(false);
			    		searchDialog.open();
			    	} else {
			    		SearchMembersDialog dialog = new SearchMembersDialog(null, selectedFile.getDirectMembers());
			    		dialog.open();
			    	} 
			    }
			});
			
			Label allMembersFromFilterLabel = new Label(ownershipRatioComposite, SWT.NONE);
			allMembersFromFilterLabel.setText(" matching members on the file   / ");
			
			allMembersFromFilterLink = new Link(ownershipRatioComposite, SWT.NONE);
			allMembersFromFilterLink.addListener(SWT.Selection,new Listener(){
			    public void handleEvent(Event event) {
		    		SearchMembersDialog searchDialog = new SearchMembersDialog(Display.getCurrent().getActiveShell(), getMembersPS(0, selectedFile, false, true), paramCountOfLastFilterQuery);
		    		searchDialog.setUseSelectButton(false);
		    		searchDialog.open();
			    }
			});			
			
			allMatchingMembersOnFolderLabel2 = new Label(ownershipRatioComposite, SWT.NONE);			
		}
		
		// Tree with details	
		viewer = new TreeViewer(parent, SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER | SWT.FULL_SELECTION | SWT.VIRTUAL);
		viewer.getTree().setLinesVisible(true);
		viewer.getTree().setHeaderVisible(true);
		viewer.setUseHashlookup(true);
		
		//layout data for the tree
		gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
		viewer.getTree().setLayoutData(gridData);
		
		// Member column
		TreeViewerColumn column = new TreeViewerColumn(viewer, SWT.NONE);
		column.getColumn().setText("Member");
		column.getColumn().setWidth(400);
		column.getColumn().setResizable(true);
		column.setLabelProvider(new ColumnLabelProvider() {
			
			@Override
			public String getText(Object element) {
				if (element instanceof AclItem) {
					AclItem aclItem = (AclItem) element;
					return aclItem.getMember(true).getDisplayName(true, false);
					
				} else if (element instanceof Member){
					Member member = (Member) element;
					return member.getDisplayName(true, false);
				}
				return null;
			}
			
			@Override
			public Image getImage(Object element) {
				if (element instanceof AclItem) {
					return ImageFactory.getInstance().getImage(AppImage.ACL_20X20);
				} else {
					Member member = (Member) element;
					if (member.isGroup()){
						return ImageFactory.getInstance().getImage(AppImage.GROUP_20X20);
			        } else {
			        	return ImageFactory.getInstance().getImage(AppImage.USER_20X20);
			        }
				}
			}
			
			@Override
			public Color getForeground(Object element) {
				if (element instanceof AclItem) {
					AclItem aclItem = (AclItem) element;
					if (!aclItem.isApplicableOnSelectedFolder()) {
						return greyColor;
					} else {
						return super.getForeground(element);
					}
				} else {
					return super.getForeground(element);
				}
			}
			
			@Override
			public Color getBackground(Object element) {
				if (element instanceof AclItem) {
					return ColorFactory.getInstance().getColor(AppColor.ACE_BACKGROUND_COLOR);
				} else {
					return super.getBackground(element);
				}
			}
		});

		// Permission column
		column = new TreeViewerColumn(viewer, SWT.NONE);
		column.getColumn().setText("Permission");
		column.getColumn().setWidth(200);
		column.getColumn().setResizable(true);
		column.setLabelProvider(new ColumnLabelProvider() {
			
			@Override
			public String getText(Object element) {
				if (element instanceof AclItem) {
					AclItem aclItem = (AclItem) element;
					if (aclItem.isApplicableOnSelectedFolder()) {
						return PermissionUtil.permissionsToString(aclItem, false);
					} else {
						return PermissionUtil.permissionsToString(aclItem, false) + " (n/a on this folder)";
					}
					
				} else if (element instanceof Member) {
					return PermissionUtil.effectivePermissionsToString(((Member) element).getEffectivePermissions(), selectedFile.isDirectory());
				}
				return null;
			}
			
			@Override
			public Color getForeground(Object element) {
				if (element instanceof AclItem) {
					AclItem aclItem = (AclItem) element;
					if (!aclItem.isApplicableOnSelectedFolder()) {
						return greyColor;
					} else {
						return super.getForeground(element);
					}
				} else {
					return super.getForeground(element);
				}
			}
			
			@Override
			public Color getBackground(Object element) {
				if (element instanceof AclItem) {
					return ColorFactory.getInstance().getColor(AppColor.ACE_BACKGROUND_COLOR);
				} else {
					return super.getBackground(element);
				}
			}
		});
						
		// Special permission columns
		for (final Integer permission : PermissionUtil.getCustomPermissionOrderList()) {
			//skip SYNCHRONIZE
			if (permission.intValue() != SpecialPermission.SYNCHRONIZE.ordinal()) {
				column = new TreeViewerColumn(viewer, SWT.NONE);
				column.getColumn().setWidth(30);
				column.getColumn().setResizable(false);
				column.getColumn().setToolTipText(PermissionUtil.specialPermissionToString(permission, false, null));
				column.getColumn().setImage(PermissionUtil.getSpecialPermissionImage(permission, false, true));
				column.setLabelProvider(new SpecialPermissionLabelProvider(permission, true, true));
			}
		}
		
		// Inheritance flags column
		column = new TreeViewerColumn(viewer, SWT.NONE);
		column.getColumn().setText("Inheritance flags");
		column.getColumn().setResizable(true);
		column.getColumn().setWidth(250);
		column.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				if (element instanceof AclItem) {
					AclItem aclItem = (AclItem) element;
					return PermissionUtil.aclEntryFlagsToString(aclItem, selectedFile.isDirectory());
				} else {
					return null; //Member row with effective permissions
				}
			}
			
			@Override
			public Color getForeground(Object element) {
				if (element instanceof AclItem) {
					AclItem aclItem = (AclItem) element;
					if (!aclItem.isApplicableOnSelectedFolder()) {
						return greyColor;
					} else {
						return super.getForeground(element);
					}
				} else {
					return super.getForeground(element);
				}
			}
			
			@Override
			public Color getBackground(Object element) {
				if (element instanceof AclItem) {
					return ColorFactory.getInstance().getColor(AppColor.ACE_BACKGROUND_COLOR);
				} else {
					return super.getBackground(element);
				}
			}
		});
		
		// From folder column
		column = new TreeViewerColumn(viewer, SWT.NONE);
		column.getColumn().setText("From folder");
		column.getColumn().setResizable(true);
		column.getColumn().setWidth(350);
		column.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				if (element instanceof AclItem) {
					AclItem aclItem = (AclItem) element;
					return aclItem.getFile() != null ? aclItem.getFile().getName() : "";
				} else {
					return null;
				}
			}
			
			@Override
			public Color getForeground(Object element) {
				if (element instanceof AclItem) {
					AclItem aclItem = (AclItem) element;
					if (!aclItem.isApplicableOnSelectedFolder()) {
						return greyColor;
					} else {
						return super.getForeground(element);
					}
				} else {
					return super.getForeground(element);
				}
			}
			
			@Override
			public Color getBackground(Object element) {
				if (element instanceof AclItem) {
					return ColorFactory.getInstance().getColor(AppColor.ACE_BACKGROUND_COLOR);
				} else {
					return super.getBackground(element);
				}
			}
		});
		
		viewer.setContentProvider(new EffectivePermissionsPartContentProvider(viewer));
		
		//enable tooltips for this tree viewer
		ColumnViewerToolTipSupport.enableFor(viewer);
		
		// call the listener with the current selection
		//IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
		//mainFileTreelistener.selectionChanged(null, ((FilePart)page.findView(FilePart.ID)).getTreeViewer().getSelection());
		
		//TODO use postSelectionListener for performance?
		//getSite().getWorkbenchWindow().getSelectionService().addPostSelectionListener(mainFileTreelistener);
		
		// select the first item in the tree so that this MemebersPart get refreshed through the SelectionService
		MPart part = partService.findPart(FilePart.ID);
		FilePart fileTreePart = (FilePart) part.getObject();
		selectionService.setSelection(fileTreePart.getTreeViewer().getSelection());
		
		// context menu for the Tree
		MenuManager menuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			@Override
			public void menuAboutToShow(IMenuManager manager) {
				final IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
				if (selection != null && !selection.isEmpty()) {
					// Add to Trace view action
					Action action = new Action() {
						public void run() {
							Member member;
							if (selection.getFirstElement() instanceof AclItem) {
								member = ((AclItem) selection.getFirstElement()).getMember(true);
							} else {
								member = (Member) selection.getFirstElement();
							}
							MPart part = partService.findPart(TracePart.ID);
							partService.activate(part, true);
							TracePart tracePart = (TracePart) part.getObject();
							tracePart.applySelectedMember(member);
						}
					};
					action.setText("Pin member in the Trace view");
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.TRACE_16X16));
					manager.add(action);
					
					// Show user info action
					action = new Action() {
						public void run() {
							Member member;
							if (selection.getFirstElement() instanceof AclItem) {
								member = ((AclItem) selection.getFirstElement()).getMember(true);
							} else {
								member = (Member) selection.getFirstElement();
							}
							
							MemberInfoDialog dialog = new MemberInfoDialog(null, member);
							dialog.open();
						}
					};
					action.setText("Show member info");
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.USER_INFO_16X16));
					manager.add(action);
										
					// Add as Include filter action
					action = new Action() {
						public void run() {
							Member member;
							MPart part = partService.findPart(FilterMemberPart.ID);
							FilterMemberPart filterMemberPart = (FilterMemberPart) part.getObject();
							
							// loop through all the selected items
							@SuppressWarnings("rawtypes")
							Iterator iter = selection.iterator();
							while (iter.hasNext()) {
								Object nextSelection = iter.next();
								if (nextSelection instanceof AclItem) {
									member = ((AclItem) nextSelection).getMember(true);
								} else {
									member = (Member) nextSelection;
								}
								filterMemberPart.addNewMemberFilter(member, true);
							}
						}
					};
					action.setText("Add selected members as include filter");
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.INCLUDE_GROUPS_16X16));
					action.setToolTipText("Adds the selected members as includes in the main filter to scope the data to the selected members.");
					manager.add(action);
					
					// Add as Exclude filter action
					action = new Action() {
						public void run() {
							Member member;
							MPart part = partService.findPart(FilterMemberPart.ID);
							FilterMemberPart filterMemberPart = (FilterMemberPart) part.getObject();
							
							// loop through all the selected items
							@SuppressWarnings("rawtypes")
							Iterator iter = selection.iterator();
							while (iter.hasNext()) {
								Object nextSelection = iter.next();
								if (nextSelection instanceof AclItem) {
									member = ((AclItem) nextSelection).getMember(true);
								} else {
									member = (Member) nextSelection;
								}
								filterMemberPart.addNewMemberFilter(member, false);
							}
						}
					};
					action.setText("Add selected members as exclude filter");
					action.setToolTipText("Adds the selected members as excludes in the main filter.");
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.EXCLUDE_GROUPS_16X16));
					manager.add(action);
				}
			}
		});
		
		Menu menu = menuMgr.createContextMenu(viewer.getTree());
		viewer.getTree().setMenu(menu);
	}	
	
	private void searchMembers() {
		// enter the search text in the content provider config and refresh the TreeViewer
		ApplicationUtil.setBusyCursor();
		contentProviderConfig.setMemberSearch(searchText.getText().trim());
		viewer.setInput(contentProviderConfig);
		viewer.refresh();
		viewer.getTree().getParent().layout(true);
		ApplicationUtil.setDefaultCursor();
	}
	
	/**
	 * Builds the query that is required to find members that match the selected filter criteria in the MemberFilterComposite.
	 * This method is only used when the option "Include nested group memberships" is enabled. Otherwise, we just use the direct members 
	 * from the ACE's found on the selected file.
	 * 
	 * The thing with group memberships is that we may find ACE's that don't belong to the users selected by the filter criteria! The dialog
	 * to search a member will only show members that match the filter criteria. So, instead of using the direct members from the 
	 * (filtered) ACE's on the selected file, we use the <b>groups</b> that belong to the members on the ACE's (or the members that match the
	 * filter criteria).
	 * 
	 * @return PreparedStatement with a SELECT member query
	 * @throws SQLException 
	 */
	private PreparedStatement getMembersPS(int limit, FileInfo selectedFile, boolean matchFile, boolean matchFilter) {
		StringBuilder additionalMemberFilterBuilder = new StringBuilder();
		Object[] additionalMemberFilterValues = null;
		PreparedStatement membersThatMatchFilterPS = null;
		List<Member> fileMembers;
		List<Integer> excludeMemberIds = new ArrayList<Integer>();	
		
		try {
			if (matchFile) {
				
				//if (matchFilter) {
					additionalMemberFilterBuilder.append("FILTER_MEMBERSHIP.GROUP_ID IN (");
					fileMembers = selectedFile.getDirectMembers();
					
					if (fileMembers.isEmpty()) {
						fileMembers = new ArrayList<Member>();
						fileMembers.add(new Member("dummy", -1));
					}
				
				/* instead of processing the excludes manually, we made a flag in the FilterHelper.buildQuery method
				} else {
					additionalMemberFilterBuilder.append(MessageFormat.format(FilterHelper.MEMBER_BASE_QUERY, DBObject.SCHEMA));
					additionalMemberFilterBuilder.append(" INNER JOIN PERMISSIONANALYZER.MEMBERSHIP FILTER_MEMBERSHIP ON FILTER_MEMBER.ID = FILTER_MEMBERSHIP.MEMBER_ID WHERE FILTER_MEMBERSHIP.GROUP_ID IN (");
					fileMembers = selectedFile.getAllDirectMembers();
					
					// process the exclude filters anyway (customer wants to exclude the admins from the "All members of file" count
					for (Filter filter : FilterHelper.getInstance().getMemberFilterListApplied()) {
						if (!filter.isInclude()) {
							if (filter.getType().equals(FilterType.MEMBER_ID)) {
								excludeMemberIds.add(Integer.parseInt(filter.getValue()));
								
							} else if (filter.getType().equals(FilterType.MEMBER_GROUP_ID)) {
								List<Member> members = MembershipCache.getGroupMembers(new Member("", Integer.parseInt(filter.getValue())), true, true, false);
								for (Member member : members) {
									excludeMemberIds.add(member.getId());
								}
							}
						}
					}
				}*/
					
				additionalMemberFilterValues = new Object[fileMembers.size() + excludeMemberIds.size() + 4];
				for (int counter = 0; counter < fileMembers.size(); counter++) {
					additionalMemberFilterBuilder.append("?");
					additionalMemberFilterValues[counter] = fileMembers.get(counter).getId();
					if (counter < fileMembers.size() - 1) {
						additionalMemberFilterBuilder.append(",");
					}
				}
				
				/*
				if (!matchFilter && excludeMemberIds.size() > 0) {
					// add the exclude member id's to the filter
					additionalMemberFilterBuilder.append(") AND FILTER_MEMBERSHIP.MEMBER_ID NOT IN (");
					for (int counter = 0; counter < excludeMemberIds.size(); counter++) {
						additionalMemberFilterBuilder.append("?");
						additionalMemberFilterValues[counter + fileMembers.size()] = excludeMemberIds.get(counter);
						if (counter < excludeMemberIds.size() - 1) {
							additionalMemberFilterBuilder.append(",");
						}
					}
				}*/
				
			} else {
				additionalMemberFilterValues = new Object[4];
			}
		
			// add a filter on the display_name and is_group, because this query is also used in the SearchMemberDialog
			// the is_group is added twice, when the user selects both Users and Groups to display then we 'disable' this filter by setting
			// member.is_group = true OR member.is_group = false
			if (matchFile) {
				additionalMemberFilterBuilder.append(") AND ");
			}
			additionalMemberFilterBuilder.append("(FILTER_MEMBER.NAME_LOWERCASE LIKE ? OR FILTER_MEMBER.DISPLAY_NAME_LOWERCASE LIKE ?) AND (FILTER_MEMBER.IS_GROUP = ? OR FILTER_MEMBER.IS_GROUP = ?)");
			additionalMemberFilterValues[additionalMemberFilterValues.length - 4] = "%"; // default value for the NAME_LOWERCASE compare
			additionalMemberFilterValues[additionalMemberFilterValues.length - 3] = "%"; // default value for the DISPLAY_NAME_LOWERCASE compare
			additionalMemberFilterValues[additionalMemberFilterValues.length - 2] = true; // don't filter on is_group, the column is present in the PreparedStatement because the SearchMemberDialog can filter on it
			additionalMemberFilterValues[additionalMemberFilterValues.length - 1] = false;
			
			String additionalMemberFilter = additionalMemberFilterBuilder.toString();
					
			//if (matchFilter) {
			boolean onlyUseExcludes = !matchFilter;
				membersThatMatchFilterPS = FilterHelper.getInstance().buildFilterQuery(ObjectType.MEMBER, false, onlyUseExcludes, limit, "NAME_LOWERCASE", additionalMemberFilter, additionalMemberFilterValues);
				paramCountOfLastFilterQuery = FilterHelper.getInstance().getParamCountOfLastFilterQuery();
			/*} else {
				membersThatMatchFilterPS = DBObject.getPreparedStatement(additionalMemberFilter);
				paramCountOfLastFilterQuery = additionalMemberFilterValues.length;
				for (int counter = 0; counter < additionalMemberFilterValues.length; counter++) {
					membersThatMatchFilterPS.setObject(counter + 1, additionalMemberFilterValues[counter]);
				}			
			}*/
		} catch (Exception ex) {
			logger.error("Error while getting member query for details about ownership ratio", ex);
			SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while getting member query for details about ownership ratio: " + ex.getMessage());
		}
		
		return membersThatMatchFilterPS;
	}
	
	@Focus
	public void setFocus() {
		viewer.getControl().setFocus();		
	}
	
	public AclItem getSelectedAclItem() {
		if (viewer != null && !viewer.getSelection().isEmpty()) {
			IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
			if (selection.getFirstElement() instanceof AclItem) {
				return (AclItem) selection.getFirstElement();
			} else {
				return null; //Member selected
			}
		} else {
			return null;
		}
	}

	public FileInfo getSelectedFile() {
		return selectedFile;
	}
	
	public EffectivePermissionsPartContentProvider getContentProvider() {
		return (EffectivePermissionsPartContentProvider) viewer.getContentProvider();
	}

	public EffectivePermissionsPartContentProviderConfig getContentProviderConfig() {
		return contentProviderConfig;
	}
}
