package com.perdemia.permissionanalyzer.application.parts;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.inject.Inject;

import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.model.application.ui.MDirtyable;
import org.eclipse.e4.ui.services.IStylingEngine;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.dialogs.ReportDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SaveFilterDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SearchFolderDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.WildcardInputDialog;
import com.perdemia.permissionanalyzer.application.jface.labelproviders.TableLabelProvider;
import com.perdemia.permissionanalyzer.application.util.FilterConfig;
import com.perdemia.permissionanalyzer.application.util.FilterHelper;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.Filter;
import com.perdemia.permissionanalyzer.data.Filter.FilterType;
import com.perdemia.permissionanalyzer.data.FilterSet;
import com.perdemia.permissionanalyzer.data.FilterSet.FilterSetType;
import com.perdemia.permissionanalyzer.data.Report;
import com.perdemia.permissionanalyzer.data.ReportFilterSet;

public class FilterFolderPart {
	public static final String ID = "com.perdemia.permissionanalyzer.application.parts.folderfilter";
	private static final Logger logger = LoggerFactory.getLogger(FilterFolderPart.class.getName());

	/** Controls for the saved selections */
	private Button deleteSelectionButton;
	private Combo folderSelectionCombo;
	private int folderSelectionComboIndex;
	
	/** Folder filter controls */
	private Button includeFoldersButton;
	private Button includeFilesButton;
	private Button aclInheritanceButton;
	private Button invertFoldersButton;
	private TableViewer folderFilterListViewer;
	private java.util.List<FilterSet> filterSets = new ArrayList<FilterSet>();
	
	@Inject 
	private IStylingEngine engine;

	@Inject
	private MDirtyable dirty;
	
	@PostConstruct
	public void createComposite(Composite parent) {
		ScrolledComposite scrollableComposite = new ScrolledComposite(parent, SWT.NONE | SWT.H_SCROLL | SWT.V_SCROLL);
		scrollableComposite.setMinSize(350, 500);
		scrollableComposite.setExpandHorizontal(true);
		scrollableComposite.setExpandVertical(true);
		scrollableComposite.setBackgroundMode(SWT.INHERIT_DEFAULT);
		
		final Composite mainContent = new Composite(scrollableComposite, SWT.NONE);
		mainContent.setLayout(new GridLayout(1, false));
		mainContent.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		scrollableComposite.setContent(mainContent);
		
		// Title label
		CLabel folderLabel = new CLabel(mainContent, SWT.NONE);
		folderLabel.setImage(ImageFactory.getInstance().getImage(AppImage.FOLDER_FILTER_HEADER_40X32));
		folderLabel.setText("Filter on folders");
		engine.setClassname(folderLabel, "h1");
		GridData gridData = new GridData();
		gridData.verticalAlignment = SWT.CENTER;
		folderLabel.setLayoutData(gridData);

		// ---------------------------------
		// Saved folder filter selections
		// ---------------------------------
		CLabel folderSelectionLabel = new CLabel(mainContent, SWT.NONE);
		folderSelectionLabel.setText("Saved folder selections:");
		folderSelectionLabel.setImage(ImageFactory.getInstance().getImage(AppImage.SELECTION_16X16));

		// Combo with filter selections
		Composite filterSelectionComposite = new Composite(mainContent, SWT.NONE);
		filterSelectionComposite.setLayout(new GridLayout(3, false));
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		filterSelectionComposite.setLayoutData(gridData);
		
		folderSelectionCombo = new Combo(filterSelectionComposite, SWT.READ_ONLY);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		folderSelectionCombo.setLayoutData(gridData);
		folderSelectionCombo.add("Custom");
		folderSelectionCombo.select(0);
		folderSelectionCombo.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				String filterSetName = folderSelectionCombo.getItem(folderSelectionCombo.getSelectionIndex());
				if (!filterSetName.equals("Custom")) {
					loadNewFilterSet(getSelectedFilterSet());
				} else {
					deleteSelectionButton.setEnabled(false);
					folderSelectionComboIndex = 0;
				}
			}
		});
		
		// add saved filter selections
		try {
			filterSets = FilterSet.where("filter_set_type = ?", new Object[]{FilterSet.FilterSetType.FOLDER.ordinal()});
			for (FilterSet filterSet : filterSets) {
				folderSelectionCombo.add(filterSet.getName());
			}		
		} catch (DBException ex) {
			SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while loading available filters: " + ex.getMessage()); 
		}
		
		// save button
		Button saveSelectionButton = new Button(filterSelectionComposite, SWT.PUSH);
		saveSelectionButton.setImage(ImageFactory.getInstance().getImage(AppImage.SAVE_16X16));
		saveSelectionButton.setToolTipText("Save selection");
		saveSelectionButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		saveSelectionButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				saveFilterSet();
			}
		});
		
		// Delete selection button
		deleteSelectionButton = new Button(filterSelectionComposite, SWT.PUSH);
		deleteSelectionButton.setImage(ImageFactory.getInstance().getImage(AppImage.DELETE_16X16));
		deleteSelectionButton.setToolTipText("Delete selection");
		deleteSelectionButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		deleteSelectionButton.setEnabled(false);
		deleteSelectionButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (MessageDialog.openQuestion(Display.getCurrent().getActiveShell(), "Delete folder selection", "Are you sure you want to delete the folder selection" + getSelectedFilterSet().getName() + "?")) {
					try {
						FilterSet filterSet = getSelectedFilterSet();
						
						// Is this filter set still attached to a report?
						StringBuilder reports = new StringBuilder();
						List<ReportFilterSet> reportFilterSets = ReportFilterSet.where("filterset_id = ?", new Object[]{filterSet.getId()});
						for (ReportFilterSet reportFilterSet : reportFilterSets) {
							Report report = Report.getById(reportFilterSet.getReportId());
							if (reports.length() > 0) {
								reports.append("\r\n");
							}
							reports.append(report.getName());
						}
						
						if (reports.length() > 0) {
							MessageDialog.openError(null, "Used by reports", "This filter set is being used by the following reports. Modify these reports and uncheck the filter set to copy the filters of this set to the reports.\r\n\r\n" + reports.toString());
							return;
						}
						
						FilterSet.delete(filterSet.getId());
						filterSets.remove(filterSet);
						folderSelectionCombo.remove(folderSelectionCombo.getSelectionIndex());
						resetFilters();						
					} catch (DBException ex) {
						MessageDialog.openError(null, "Error", "Error while deleting filter: " + ex.getMessage());
					}
				}
			}
		});
		
		// ---------------------------------
		// Options filter area
		// ---------------------------------		
		// selection to include users / groups
		FormToolkit toolkit = new FormToolkit(parent.getDisplay());
		Section optionsSection = new Section(mainContent, Section.TITLE_BAR | Section.TWISTIE | Section.EXPANDED);
		optionsSection.setText("Folder filter options");
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.verticalIndent = 10;
		optionsSection.setLayoutData(gridData);
		
		Composite optionsComposite = toolkit.createComposite(optionsSection, SWT.WRAP);
		GridLayout gridLayout = new GridLayout(3, false);
		gridLayout.horizontalSpacing = 20;
		optionsComposite.setLayout(gridLayout);
		optionsComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		CLabel userGroupSelectionLabel = new CLabel(optionsComposite, SWT.NONE);
		userGroupSelectionLabel.setText("Include folders/files: ");
		includeFoldersButton = new Button(optionsComposite, SWT.CHECK);
		includeFoldersButton.setImage(ImageFactory.getInstance().getImage(AppImage.FOLDER_20X20));
		includeFoldersButton.setToolTipText("Include folders");
		includeFoldersButton.addSelectionListener(new SelectionAdapter() {
		    @Override
		    public void widgetSelected(SelectionEvent e) {
		    	dirty.setDirty(true);
		    	FilterHelper.getInstance().getFilterConfig(true).setIncludeFolders(includeFoldersButton.getSelection());
		    }
	    });
		includeFoldersButton.setSelection(true);
		FilterHelper.getInstance().getFilterConfig(true).setIncludeFolders(true);
		
		includeFilesButton = new Button(optionsComposite, SWT.CHECK);
		includeFilesButton.setImage(ImageFactory.getInstance().getImage(AppImage.FILE_TXT_READ_20X20));
		includeFilesButton.setToolTipText("Include files");
		includeFilesButton.addSelectionListener(new SelectionAdapter() {
		    @Override
		    public void widgetSelected(SelectionEvent e) {
		    	dirty.setDirty(true);
		    	FilterHelper.getInstance().getFilterConfig(true).setIncludeFiles(includeFilesButton.getSelection());
		    }
	    });
		includeFilesButton.setSelection(true);
		FilterHelper.getInstance().getFilterConfig(true).setIncludeFiles(true);
		
		aclInheritanceButton = new Button(optionsComposite, SWT.CHECK);
		aclInheritanceButton.setImage(ImageFactory.getInstance().getImage(AppImage.BLOCK_ACL_INHERITANCE_24X24));
		aclInheritanceButton.setText("Only show folders/files that block ACL inheritance");
		aclInheritanceButton.setToolTipText("Only show folders/files that block ACL inheritance");
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 3;
		gridData.verticalIndent = 5;
		aclInheritanceButton.setLayoutData(gridData);
		aclInheritanceButton.addSelectionListener(new SelectionAdapter() {
		    @Override
		    public void widgetSelected(SelectionEvent e) {
		    	dirty.setDirty(true);
		    	FilterHelper.getInstance().getFilterConfig(true).setIncludeFilesThatBlockInheritance(aclInheritanceButton.getSelection());
		    }
	    });
		
		invertFoldersButton = new Button(optionsComposite, SWT.CHECK);
		invertFoldersButton.setImage(ImageFactory.getInstance().getImage(AppImage.INVERT_FOLDERS_24X24));
		invertFoldersButton.setText("Invert matching folders");
		invertFoldersButton.setToolTipText("Exclude folders that match the filter criteria. This can be used to display all the files and folders that are NOT owned by a particular group.");
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 3;
		gridData.verticalIndent = 5;
		invertFoldersButton.setLayoutData(gridData);
		invertFoldersButton.addSelectionListener(new SelectionAdapter() {
		    @Override
		    public void widgetSelected(SelectionEvent e) {
		    	dirty.setDirty(true);
		    	FilterHelper.getInstance().getFilterConfig(true).setInvertFolders(invertFoldersButton.getSelection());
		    }
	    });
		
		optionsSection.setClient(optionsComposite);
		
		// ---------------------------------
		// Folder filter list
		// ---------------------------------
		// header bar
		Composite folderHeaderBar = new Composite(mainContent, SWT.NONE);
		folderHeaderBar.setLayout(new GridLayout(4, false));
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		folderHeaderBar.setLayoutData(gridData);
		
		CLabel folderFilterListLabel = new CLabel(folderHeaderBar, SWT.NONE);
		folderFilterListLabel.setText("Include or exclude folders:");
		engine.setClassname(folderFilterListLabel, "h2");
		gridData = new GridData(SWT.LEFT, SWT.BOTTOM, true, false);
		folderFilterListLabel.setLayoutData(gridData);
		
		//search folders
		Button searchFoldersButton = new Button(folderHeaderBar, SWT.PUSH);
		searchFoldersButton.setImage(ImageFactory.getInstance().getImage(AppImage.SEARCH_16X16));
		searchFoldersButton.setToolTipText("Select folders");
		searchFoldersButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		searchFoldersButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				SearchFolderDialog searchDialog = new SearchFolderDialog(Display.getCurrent().getActiveShell(), folderFilterListViewer, true, true);
				int result = searchDialog.open();
				if (result == SearchFolderDialog.ADD_TO_INCLUDE_ID || result == SearchFolderDialog.ADD_TO_EXCLUDE_ID) {
					dirty.setDirty(true);
				}
			}
		});

		// Add wildcard value button
		Button addWildCardToFolderListButton = new Button(folderHeaderBar, SWT.PUSH);
		addWildCardToFolderListButton.setImage(ImageFactory.getInstance().getImage(AppImage.ADD_WILDCARD_16X16));
		addWildCardToFolderListButton.setToolTipText("Add wildcard search text");
		addWildCardToFolderListButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		addWildCardToFolderListButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				WildcardInputDialog inputDialog = new WildcardInputDialog(
						Display.getCurrent().getActiveShell(),
						"Add wildcard filter on folder or file name",
						"Please enter a wildcard value using the * character. Wildcard values will be used to filter on file or folder name and are not case-sensitive.\r\n\r\nExample: project* or *.exe\r\n\r\nNote that a wildcard filter has impact on the performance of the overviews. If possible use the wildcard at the end of the search text.", true);
				int result = inputDialog.open();
				if (result == WildcardInputDialog.ADD_TO_INCLUDE_ID || result == WildcardInputDialog.ADD_TO_EXCLUDE_ID){
					String valuePrefix = (inputDialog.isApplyFilterAsANDOpertator() ? FilterHelper.PREFIX_FOLDER_WILDCARD_FILTER_AFTER_OTHER_FILTERS : "") + 
							(inputDialog.isIncludeSubfolders() ? FilterHelper.PREFIX_FOLDER_WILDCARD_FILTER_INCLUDE_SUBFOLDERS : "");
					FilterHelper.getInstance().getFolderFilterList().add(new Filter(Filter.FilterType.FOLDER_NAME, result == WildcardInputDialog.ADD_TO_INCLUDE_ID, valuePrefix + inputDialog.getValue(), inputDialog.getValue()));
					folderFilterListViewer.refresh();
					dirty.setDirty(true);
				}
			}
		});
		
		// Remove from folder list
		Button removeFromFolderListButton = new Button(folderHeaderBar, SWT.PUSH);
		removeFromFolderListButton.setImage(ImageFactory.getInstance().getImage(AppImage.DELETE_16X16));
		removeFromFolderListButton.setToolTipText("Remove selected folders from the filter list");
		removeFromFolderListButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		removeFromFolderListButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				IStructuredSelection selection = (IStructuredSelection) folderFilterListViewer.getSelection();
				for (Iterator iterator = selection.iterator(); iterator.hasNext();) {
					FilterHelper.getInstance().getFolderFilterList().remove(iterator.next());
					dirty.setDirty(true);
				}
				folderFilterListViewer.refresh();
			}
		});
		
		//create a one column TableViewer so an image can be used
		folderFilterListViewer = new TableViewer(mainContent, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL | SWT.FULL_SELECTION);
		gridData = new GridData(GridData.FILL_BOTH);
		folderFilterListViewer.getTable().setLayoutData(gridData);
		
		final TableViewerColumn folderColumn = new TableViewerColumn(folderFilterListViewer, SWT.NONE);
		folderFilterListViewer.getTable().addControlListener(new ControlListener(){
			@Override
			public void controlMoved(ControlEvent e) {}

			@Override
			public void controlResized(ControlEvent e) {
				folderColumn.getColumn().setWidth(folderFilterListViewer.getTable().getSize().x-4);
			}
		});
		
		folderFilterListViewer.setContentProvider(new IStructuredContentProvider() {
			@Override
			public void dispose() {}

			@Override
			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}

			@Override
			public Object[] getElements(Object inputElement) {
				@SuppressWarnings("unchecked")
				List<Filter> filters = (List<Filter>)inputElement;
				List<Filter> result = new ArrayList<Filter>();
				for (Filter filter : filters) {
					if (filter.getType().equals(FilterType.FOLDER_NAME) || filter.getType().equals(FilterType.FOLDER_PATH)) {
						result.add(filter);
					}
				}
				return result.toArray();
			}
		});
		
		folderFilterListViewer.setLabelProvider(new TableLabelProvider(){
			@Override
			public Image getColumnImage(Object element, int columnIndex) {
				if (columnIndex == 0) {
					Filter filter = (Filter) element;
					if (filter.getType().equals(Filter.FilterType.FOLDER_NAME)) {
						if (filter.isInclude()) {
							return ImageFactory.getInstance().getImage(AppImage.INCLUDE_WILDCARD_16X16);
						} else {
							return ImageFactory.getInstance().getImage(AppImage.EXCLUDE_WILDCARD_16X16);
						}
					} else if (filter.getType().equals(Filter.FilterType.FOLDER_PATH)) {
						if (filter.isInclude()) {
							return ImageFactory.getInstance().getImage(AppImage.INCLUDE_FOLDER_16X16);
						} else {
							return ImageFactory.getInstance().getImage(AppImage.EXCLUDE_FOLDER_16X16);
						}
					}
				}
				return null;
			}

			@Override
			public String getColumnText(Object element, int columnIndex) {
				Filter filter = (Filter) element;
				switch (columnIndex) {
					case 0: 
						if (filter.getType().equals(FilterType.FOLDER_NAME)) {
							StringBuilder suffix = new StringBuilder("(");
							if (filter.getValue().contains(FilterHelper.PREFIX_FOLDER_WILDCARD_FILTER_INCLUDE_SUBFOLDERS)) {
								suffix.append("include subfolders, ");
							} else {
								suffix.append("no subfolders, ");
							}
							if (filter.getValue().contains(FilterHelper.PREFIX_FOLDER_WILDCARD_FILTER_AFTER_OTHER_FILTERS)) {
								suffix.append("apply on result of other filters)");
							} else {
								suffix.append("apply on all folders/files)");
							}					
							
							return filter.getName() + suffix.toString();
						} else {
							return filter.getName();
						}
					default: return "";
				}
			}
		});
				
		folderFilterListViewer.setInput(FilterHelper.getInstance().getFolderFilterList());

		//----------------------------------------------
		// context menu for the tables
		//----------------------------------------------
		
		// context menu for the folderFilterListViewer
		MenuManager folderMenuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
		folderMenuMgr.setRemoveAllWhenShown(true);
		folderMenuMgr.addMenuListener(new IMenuListener() {
			@Override
			public void menuAboutToShow(IMenuManager manager) {
				final IStructuredSelection selection = (IStructuredSelection) folderFilterListViewer.getSelection();
				if (selection != null && !selection.isEmpty()) {
					final Filter filter = ((Filter) selection.getFirstElement());
					if (filter.getType().equals(FilterType.FOLDER_PATH)) {
						Action action = new Action() {
							public void run() {
								Program.launch(filter.getName());
							}
						};
						action.setText("Open folder in Explorer");
						action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.FOLDER_EXPLORER_16X16));
						manager.add(action);
						
					}
				}
			}
		});
		Menu folderMenu = folderMenuMgr.createContextMenu(folderFilterListViewer.getTable());
		folderFilterListViewer.getTable().setMenu(folderMenu);
	}
	
	/**
	 * Returns the selected <code>FilterSet</code> based on the name in the dropdown box
	 * 
	 * @return selected <code>FilterSet</code>
	 * @throws DBException 
	 */
	public FilterSet getSelectedFilterSet() {
		String name = folderSelectionCombo.getItem(folderSelectionCombo.getSelectionIndex());
		
		// get the selected filter set
		for (FilterSet filterSet : filterSets) {
			if (filterSet.getName().equals(name)) {
				try {
					filterSet.loadFilters();
				} catch (DBException e) {
					logger.error("Error while loading the filters for filter set " + filterSet.getName(), e);
					SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while loading the filters for filter set " + filterSet.getName() + ": " + e.getMessage());
				}
				return filterSet;
			}
		}
		return null;
	}
	
	/** Sets the filters to their default value */
	public void resetFilters() {
		FilterHelper.getInstance().getFolderFilterList().clear();
		FilterConfig filterConfig = FilterHelper.getInstance().getFilterConfig(true);
		filterConfig.setIncludeFolders(true);
		filterConfig.setIncludeFiles(true);
		filterConfig.setIncludeFilesThatBlockInheritance(false);
		filterConfig.setInvertFolders(false);
				
		refreshFiltersInGUI();
		deleteSelectionButton.setEnabled(false);
	}
	
	/** 
	 * Load a new FilterSet in the GUI. This method is triggered by the onChange of {@link #folderSelectionCombo} or from the Load Report button in the {@link ReportDialog}.
	 * 
	 * @param filterSet FilterSet to load
	 */
	public boolean loadNewFilterSet(FilterSet filterSet) {
		boolean filtersLoaded = false;
		
		// get the selected filter set
		if (filterSet != null) {
			try {
				// get related filters
				filterSet.loadFilters();
				filtersLoaded = loadNewFilters(filterSet.getFilters());
				
			} catch (DBException ex) {
				SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while loading filters for the selected filter set: " + ex.getMessage());
			}
		}
		
		if (filtersLoaded) { // not cancelled?
			deleteSelectionButton.setEnabled(true);
			
			// select the dropdown item (this is not yet done in case the method is called from the ManageReportsHandler
			for (int counter = 0; counter < folderSelectionCombo.getItemCount(); counter++) {
				if (folderSelectionCombo.getItem(counter).equalsIgnoreCase(filterSet.getName())) {
					folderSelectionCombo.select(counter);
					folderSelectionComboIndex = counter;
					break;
				}
			}
		}
		
		return filtersLoaded;
	}
	
	/** 
	 * Load a new Filters in the GUI. This method is used by {@link #loadNewFilterSet(FilterSet)} and triggered by the Load Report button in the {@link ReportDialog}.
	 * 
	 * @param filterSet FilterSet to load
	 */
	public boolean loadNewFilters(List<Filter> filters) {
		if (dirty.isDirty() && !MessageDialog.openQuestion(Display.getCurrent().getActiveShell(), "Unsaved folder filters", "Are you sure you want to overwrite the current folder filters?")) {
			//user cancelled, select last index
			folderSelectionCombo.select(folderSelectionComboIndex);
			return false;
		} else {
			//load the new filters
			FilterHelper.getInstance().getFolderFilterList().clear();
			FilterHelper.getInstance().getFolderFilterList().addAll(filters);
			refreshFiltersInGUI();
			return true;
		}
	}
	 
	/**
	 * Updates the GUI after a new set of filters has been loaded. Either by selecting a filter set from the dropdown or by the method {@link #loadNewFilters(List)}. 
	 */
	private void refreshFiltersInGUI() {
		// refresh GUI
		folderFilterListViewer.refresh();
		FilterConfig filterConfig = FilterHelper.getInstance().getFilterConfig(true);
		
		includeFoldersButton.setSelection(filterConfig.isIncludeFolders());
		includeFilesButton.setSelection(filterConfig.isIncludeFiles());
		aclInheritanceButton.setSelection(filterConfig.isIncludeFilesThatBlockInheritance());
		invertFoldersButton.setSelection(filterConfig.isInvertFolders());
		folderSelectionCombo.select(0);		
		dirty.setDirty(false);
	}
	
	@Persist
	private void saveFilterSet() {
		try {
			int selectedIndex = -1;
			FilterSet filterSet = null;
			if (!folderSelectionCombo.getText().equals("Custom")) {
				filterSet = getSelectedFilterSet();
				selectedIndex = filterSets.indexOf(filterSet);
			}
			
			SaveFilterDialog dialog = new SaveFilterDialog(Display.getCurrent().getActiveShell(), filterSets, selectedIndex, "Folder filter sets");
			int result = dialog.open(); 
			
			if (result == IDialogConstants.OK_ID) {;
				// existing filter set is selected	
				filterSet = (FilterSet) dialog.getSelectedFilterSet();
				
				// remove all the current filters
				Filter.delete("FILTERSET_ID = ?", filterSet.getId());
				filterSet.setFilters(new ArrayList<Filter>());
				
			} else if (result == SaveFilterDialog.CREATE_ID) {
				// a new filter set is created
				filterSet = (FilterSet) dialog.getSelectedFilterSet();
				filterSet.setType(FilterSetType.FOLDER);
				filterSet.setFilters(new ArrayList<Filter>());
				filterSet.save();
				
				filterSets.add(filterSet);
				folderSelectionCombo.add(filterSet.getName());
			} else {
				//user cancelled
				return;
			}
			
			// add all the current folder filter items to the filter set
			for (Filter filter :  FilterHelper.getInstance().getFolderFilterList()) {
				//make sure to create a new filter
				filter.setId(null);
				filter.setFilterSetId(filterSet.getId());
				filter.save();
				filterSet.getFilters().add(filter);
			}
			
			selectedIndex = filterSets.indexOf(filterSet);
			folderSelectionCombo.select(selectedIndex + 1); //skip the first item "Custom"
			deleteSelectionButton.setEnabled(true);
			
			dirty.setDirty(false);
			
		} catch (DBException ex) {
			MessageDialog.openError(null, "Error", "Error while saving filter: " + ex.getMessage());
		}
	}
	
	@Focus
	public void setFocus() {
		folderFilterListViewer.getTable().setFocus();
	}
}
