package com.perdemia.permissionanalyzer.application.parts;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.inject.Inject;

import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.model.application.ui.MDirtyable;
import org.eclipse.e4.ui.services.IStylingEngine;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.dialogs.ReportDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SaveFilterDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.swt.ImageComposite;
import com.perdemia.permissionanalyzer.application.util.FilterConfig;
import com.perdemia.permissionanalyzer.application.util.FilterHelper;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.application.util.PermissionUtil;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.Filter;
import com.perdemia.permissionanalyzer.data.Filter.FilterType;
import com.perdemia.permissionanalyzer.data.FilterSet;
import com.perdemia.permissionanalyzer.data.FilterSet.FilterSetType;
import com.perdemia.permissionanalyzer.data.PermissionLookup;
import com.perdemia.permissionanalyzer.data.PermissionLookup.SimplePermission;
import com.perdemia.permissionanalyzer.data.Report;
import com.perdemia.permissionanalyzer.data.ReportFilterSet;

public class FilterPermissionPart {
	public static final String ID = "com.perdemia.permissionanalyzer.application.parts.permissionfilter";
	private static final Logger logger = LoggerFactory.getLogger(FilterPermissionPart.class.getName());
	
	/** Controls for the saved selections */
	private Button deleteSelectionButton;
	private Combo permissionSelectionCombo;
	private int permissionSelectionComboIndex;
	
	/** A list containing all the permission filters (rows with checkboxes for allow and deny). This list is used to react on selections. 
	 * For example, when the simple permission Full is selected the application selects automatically all the special permissions.
	 * 
	 *  @see #permissionMappings
	 */
	protected static List<PermissionFilterRow> permissionFilters = new ArrayList<PermissionFilterRow>();
	
	/** The index in {@link permissionFilters} where the simple permissions start */
	public final static int SIMPLE_PERMISSIONS_INDEX_OFFSET = 13;
		
	/** Permission filter controls */
	protected static Button filterAllPermissionsRadio;
	protected static Button filterAnyPermissionsRadio;
	private Button hideInheritedPermissionsButton;
	private Button includeFilterRadio;
	private Button excludeFilterRadio;	
	
	/** Checkboxes for the simple permissions (allow and deny) */
	private PermissionFilterRow fullFilter;
	private PermissionFilterRow modifyFilter;
	private PermissionFilterRow writeFilter;
	private PermissionFilterRow readExecuteFilter;
	private PermissionFilterRow listFolderContentFilter;
	private PermissionFilterRow readFilter;	
	
	/** Checkboxes for the special permissions (allow and deny) */
	private PermissionFilterRow readDataFilter;
	private PermissionFilterRow writeDataFilter;
	private PermissionFilterRow appendDataFilter;
	private PermissionFilterRow readExtAttrFilter;
	private PermissionFilterRow writeExtAttrFilter;
	private PermissionFilterRow executeFilter;
	private PermissionFilterRow deleteChildFilter;
	private PermissionFilterRow readAttrFilter;
	private PermissionFilterRow writeAttrFilter;
	private PermissionFilterRow deleteFilter;
	private PermissionFilterRow readAclFilter;
	private PermissionFilterRow writeAclFilter;
	private PermissionFilterRow writeOwnerFilter;	
		
	private java.util.List<FilterSet> filterSets = new ArrayList<FilterSet>();

	@Inject 
	private IStylingEngine engine;
	
	@Inject
	MDirtyable dirty;
	
	@PostConstruct
	public void createComposite(Composite parent) {
		ScrolledComposite scrollableComposite = new ScrolledComposite(parent, SWT.NONE | SWT.H_SCROLL | SWT.V_SCROLL);
		scrollableComposite.setMinSize(390, 900);
		scrollableComposite.setExpandHorizontal(true);
		scrollableComposite.setExpandVertical(true);
		scrollableComposite.setBackgroundMode(SWT.INHERIT_DEFAULT);
		
		Composite mainContent = new Composite(scrollableComposite, SWT.NONE);
		mainContent.setLayout(new GridLayout(1, false));
		mainContent.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		scrollableComposite.setContent(mainContent);
		
		// Title label
		CLabel permissionLabel = new CLabel(mainContent, SWT.NONE);
		permissionLabel.setImage(ImageFactory.getInstance().getImage(AppImage.PERM_FILTER_HEADER_40X32));
		permissionLabel.setText("Filter on permissions");
		//permissionLabel.setForeground(ColorFactory.getInstance().getColor(AppColor.FILTER_TITLE));
		//permissionLabel.setFont(FontFactory.getInstance().getFont(AppFont.FILTER_TITLE));
		engine.setClassname(permissionLabel, "h1");
		GridData gridData = new GridData();
		gridData.verticalAlignment = SWT.CENTER;
		permissionLabel.setLayoutData(gridData);

		// ---------------------------------
		// Saved permission filter selections
		// ---------------------------------
		CLabel permissionSelectionLabel = new CLabel(mainContent, SWT.NONE);
		permissionSelectionLabel.setText("Saved permission selections:");
		permissionSelectionLabel.setImage(ImageFactory.getInstance().getImage(AppImage.SELECTION_16X16));
		gridData = new GridData();
		permissionSelectionLabel.setLayoutData(gridData);

		// Combo with filter selections
		Composite filterSelectionComposite = new Composite(mainContent, SWT.NONE);
		filterSelectionComposite.setLayout(new GridLayout(3, false));
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		filterSelectionComposite.setLayoutData(gridData);
		
		permissionSelectionCombo = new Combo(filterSelectionComposite, SWT.READ_ONLY);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		permissionSelectionCombo.setLayoutData(gridData);
		permissionSelectionCombo.add("Custom");
		permissionSelectionCombo.select(0);
		permissionSelectionCombo.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				String filterSetName = permissionSelectionCombo.getItem(permissionSelectionCombo.getSelectionIndex());
				if (!filterSetName.equals("Custom")) {
					loadNewFilterSet(getSelectedFilterSet());
				} else {
					deleteSelectionButton.setEnabled(false);
					permissionSelectionComboIndex = 0;
				}
			}
		});
		
		// add saved filter selections
		try {
			filterSets = FilterSet.where("filter_set_type = ?", new Object[]{FilterSet.FilterSetType.PERMISSION.ordinal()});
			for (FilterSet filterSet : filterSets) {
				permissionSelectionCombo.add(filterSet.getName());
			}		
		} catch (DBException ex) {
			SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while loading available filters: " + ex.getMessage()); 
		}
		
		// save button
		Button saveSelectionButton = new Button(filterSelectionComposite, SWT.PUSH);
		saveSelectionButton.setImage(ImageFactory.getInstance().getImage(AppImage.SAVE_16X16));
		saveSelectionButton.setToolTipText("Save selection");
		saveSelectionButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		saveSelectionButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				saveFilterSet();
			}
		});
		
		// Delete selection button
		deleteSelectionButton = new Button(filterSelectionComposite, SWT.PUSH);
		deleteSelectionButton.setImage(ImageFactory.getInstance().getImage(AppImage.DELETE_16X16));
		deleteSelectionButton.setToolTipText("Delete selection");
		deleteSelectionButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		deleteSelectionButton.setEnabled(false);
		deleteSelectionButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (MessageDialog.openQuestion(Display.getCurrent().getActiveShell(), "Delete permission selection", "Are you sure you want to delete the permission selection " + getSelectedFilterSet().getName() + "?")) {
					try {
						FilterSet filterSet = getSelectedFilterSet();
						
						// Is this filter set still attached to a report?
						StringBuilder reports = new StringBuilder();
						List<ReportFilterSet> reportFilterSets = ReportFilterSet.where("filterset_id = ?", new Object[]{filterSet.getId()});
						for (ReportFilterSet reportFilterSet : reportFilterSets) {
							Report report = Report.getById(reportFilterSet.getReportId());
							if (reports.length() > 0) {
								reports.append("\r\n");
							}
							reports.append(report.getName());
						}
						
						if (reports.length() > 0) {
							MessageDialog.openError(null, "Used by reports", "This filter set is being used by the following reports. Modify these reports and uncheck the filter set to copy the filters of this set to the reports.\r\n\r\n" + reports.toString());
							return;
						}
						
						FilterSet.delete(filterSet.getId());
						filterSets.remove(filterSet);
						permissionSelectionCombo.remove(permissionSelectionCombo.getSelectionIndex());
						resetFilters();						
					} catch (DBException ex) {
						MessageDialog.openError(null, "Error", "Error while deleting filter: " + ex.getMessage());
					}
				}
			}
		});
		
		// ---------------------------------
		// Options filter area
		// ---------------------------------
		FormToolkit toolkit = new FormToolkit(parent.getDisplay());
		Section optionsSection = new Section(mainContent, Section.TITLE_BAR | Section.TWISTIE | Section.EXPANDED);
		optionsSection.setText("Permission filter options");
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.verticalIndent = 10;
		optionsSection.setLayoutData(gridData);
		
		/*
		CLabel filterOptionsLabel = new CLabel(mainContent, SWT.NONE);
		//filterOptionsLabel.setForeground(ColorFactory.getInstance().getColor(AppColor.FILTER_TITLE));
		//filterOptionsLabel.setFont(FontFactory.getInstance().getFont(AppFont.FILTER_SUBTITLE));
		filterOptionsLabel.setText("Permission filter options:");
		filterOptionsLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		engine.setClassname(filterOptionsLabel, "h2");*/
		
		Composite optionsComposite = toolkit.createComposite(optionsSection, SWT.WRAP);
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.horizontalSpacing = 20;
		optionsComposite.setLayout(gridLayout);
		optionsComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		//-----------------------------------
		// INCLUDE / EXCLUDE FILTER
		//-----------------------------------
		Composite includeExcludeFilterComposite = new Composite(optionsComposite, SWT.NONE);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.verticalIndent = -10;
		gridData.horizontalIndent = -5;
		includeExcludeFilterComposite.setLayoutData(gridData);
		includeExcludeFilterComposite.setLayout(new GridLayout(1, false));
		
		includeFilterRadio = new Button(includeExcludeFilterComposite, SWT.RADIO);
		includeFilterRadio.setText("Include selected permissions");
		includeFilterRadio.setImage(ImageFactory.getInstance().getImage(AppImage.INCLUDE_PERMISSIONS_16X16));
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		includeFilterRadio.setLayoutData(gridData);
		includeFilterRadio.setSelection(true);
		includeFilterRadio.addSelectionListener(new SelectionAdapter() {
		    @Override
		    public void widgetSelected(SelectionEvent e) {
		    	dirty.setDirty(true);
		    	FilterHelper.getInstance().getFilterConfig(true).setIncludePermissions(true);
		    }
	    });
		
		excludeFilterRadio = new Button(includeExcludeFilterComposite, SWT.RADIO);
		excludeFilterRadio.setText("Exclude selected permissions");
		excludeFilterRadio.setImage(ImageFactory.getInstance().getImage(AppImage.EXCLUDE_PERMISSIONS_16X16));
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.heightHint = 25;
		excludeFilterRadio.setLayoutData(gridData);
		excludeFilterRadio.addSelectionListener(new SelectionAdapter() {
		    @Override
		    public void widgetSelected(SelectionEvent e) {
		    	dirty.setDirty(true);
		    	FilterHelper.getInstance().getFilterConfig(true).setIncludePermissions(false);
		    }
	    });
		
		// default selection		
		FilterHelper.getInstance().getFilterConfig(true).setIncludePermissions(true);
		
		//-----------------------------------
		// FILTER TYPE COMPOSITE
		//-----------------------------------
		Composite filterTypeComposite = new Composite(optionsComposite, SWT.NONE);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalIndent = -5;
		filterTypeComposite.setLayoutData(gridData);
		filterTypeComposite.setLayout(new GridLayout(1, false));
		
		filterAnyPermissionsRadio = new Button(filterTypeComposite, SWT.RADIO);
		filterAnyPermissionsRadio.setText("Select ACE's having ANY of the selected permissions");
		filterAnyPermissionsRadio.setImage(ImageFactory.getInstance().getImage(AppImage.SELECT_PERMISSIONS_UNION_16X16));
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		//gridData.verticalIndent = 10;
		filterAnyPermissionsRadio.setLayoutData(gridData);		
		filterAnyPermissionsRadio.addSelectionListener(new SelectionAdapter() {
		    @Override
		    public void widgetSelected(SelectionEvent e) {
		    	setFilterTypeAllOrAny(false);
		    }
	    });
		
		filterAllPermissionsRadio = new Button(filterTypeComposite, SWT.RADIO);
		filterAllPermissionsRadio.setText("Select ACE's having ALL of the selected permissions");
		filterAllPermissionsRadio.setImage(ImageFactory.getInstance().getImage(AppImage.SELECT_PERMISSIONS_INTERSECT_16X16));
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		filterAllPermissionsRadio.setLayoutData(gridData);
		filterAllPermissionsRadio.addSelectionListener(new SelectionAdapter() {
		    @Override
		    public void widgetSelected(SelectionEvent e) {
		    	setFilterTypeAllOrAny(true);
		    }
	    });
		
		hideInheritedPermissionsButton = new Button(optionsComposite, SWT.CHECK);
		hideInheritedPermissionsButton.setImage(ImageFactory.getInstance().getImage(AppImage.BROWSE_16X16));
		hideInheritedPermissionsButton.setText("Hide inherited permissions");
		hideInheritedPermissionsButton.setToolTipText("Hide permissions that have been inherited from a parent folder");
		hideInheritedPermissionsButton.addSelectionListener(new SelectionAdapter() {
		    @Override
		    public void widgetSelected(SelectionEvent e) {
		    	dirty.setDirty(true);
		    	FilterHelper.getInstance().getFilterConfig(true).setHideInheritedPermissions(hideInheritedPermissionsButton.getSelection());
		    }
	    });
		
		optionsSection.setClient(optionsComposite);
		
		//-----------------------------------
		// simple permission filters header
		//-----------------------------------
		Composite permissionsComposite = new Composite(mainContent, SWT.NONE);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 3;
		gridData.verticalIndent = -10;
		permissionsComposite.setLayoutData(gridData);
		gridLayout = new GridLayout(1, false);
		gridLayout.verticalSpacing = 2;
		gridLayout.marginBottom = 10;
		permissionsComposite.setLayout(gridLayout);
		
		Composite simplePermissionsHeader = new Composite(permissionsComposite, SWT.NONE);
		simplePermissionsHeader.setLayout(new GridLayout(3, false));
		gridData = new GridData();
		gridData.heightHint = 30;
		//gridData.verticalIndent = -15;
		simplePermissionsHeader.setLayoutData(gridData);
		
		CLabel simplePermissionsLabel = new CLabel(simplePermissionsHeader, SWT.NONE);
		simplePermissionsLabel.setText("Select simple permissions:");
		//simplePermissionsLabel.setImage(ImageFactory.getInstance().getImage(AppImage.INCLUDE_PERMISSIONS_16X16));
		//simplePermissionsLabel.setFont(FontFactory.getInstance().getFont(AppFont.FILTER_SUBTITLE));
		//simplePermissionsLabel.setForeground(ColorFactory.getInstance().getColor(AppColor.FILTER_TITLE));
		gridData = new GridData();
		gridData.widthHint = 240;
		//gridData.horizontalIndent = -5;
		simplePermissionsLabel.setLayoutData(gridData);
		engine.setClassname(simplePermissionsLabel, "h2");
		
		ImageComposite simpleAllowImage = new ImageComposite(simplePermissionsHeader, ImageFactory.getInstance().getImage(AppImage.ALLOW_20X20), "Select 'allow' permissions");
		simpleAllowImage.setCursor(Display.getDefault().getSystemCursor(SWT.CURSOR_HAND));
		simpleAllowImage.addMouseListener(new MouseAdapter(){
			@Override
			public void mouseUp(MouseEvent e) {
				// invert the first checkbox
				permissionFilters.get(SIMPLE_PERMISSIONS_INDEX_OFFSET).setAllow(!permissionFilters.get(SIMPLE_PERMISSIONS_INDEX_OFFSET).isAllow(), true, true);
				
				// change the other checkboxes to meet the first one
				for (int counter = SIMPLE_PERMISSIONS_INDEX_OFFSET + 1; counter < permissionFilters.size(); counter++) {
					permissionFilters.get(counter).setAllow(permissionFilters.get(SIMPLE_PERMISSIONS_INDEX_OFFSET).isAllow(), true, true);
				}
			}
		});
		
		ImageComposite simpleDenyImage = new ImageComposite(simplePermissionsHeader, ImageFactory.getInstance().getImage(AppImage.DENY_20X20), "Select 'deny' permissions");
		((GridData)simpleDenyImage.getLayoutData()).horizontalIndent = 25;
		simpleDenyImage.setCursor(Display.getDefault().getSystemCursor(SWT.CURSOR_HAND));
		simpleDenyImage.addMouseListener(new MouseAdapter(){
			@Override
			public void mouseUp(MouseEvent e) {
				// invert the first checkbox
				permissionFilters.get(SIMPLE_PERMISSIONS_INDEX_OFFSET).setDeny(!permissionFilters.get(SIMPLE_PERMISSIONS_INDEX_OFFSET).isDeny(), true, true);
				
				// change the other checkboxes to meet the first one
				for (int counter = SIMPLE_PERMISSIONS_INDEX_OFFSET + 1; counter < permissionFilters.size(); counter++) {
					permissionFilters.get(counter).setDeny(permissionFilters.get(SIMPLE_PERMISSIONS_INDEX_OFFSET).isDeny(), true, true);
				}
			}
		});
		
		fullFilter = new PermissionFilterRow(permissionsComposite, PermissionUtil.simplePermissionToString(SimplePermission.FULL), PermissionUtil.getSimplePermissionImage(SimplePermission.FULL), true, 13, SimplePermission.FULL, this);
		modifyFilter = new PermissionFilterRow(permissionsComposite, PermissionUtil.simplePermissionToString(SimplePermission.MODIFY), PermissionUtil.getSimplePermissionImage(SimplePermission.MODIFY), true, 14, SimplePermission.MODIFY, this);
		writeFilter = new PermissionFilterRow(permissionsComposite, PermissionUtil.simplePermissionToString(SimplePermission.WRITE), PermissionUtil.getSimplePermissionImage(SimplePermission.WRITE), true, 15, SimplePermission.WRITE, this);
		readExecuteFilter = new PermissionFilterRow(permissionsComposite, PermissionUtil.simplePermissionToString(SimplePermission.READ_EXECUTE), PermissionUtil.getSimplePermissionImage(SimplePermission.READ_EXECUTE), true, 16, SimplePermission.READ_EXECUTE, this);
		listFolderContentFilter = new PermissionFilterRow(permissionsComposite, PermissionUtil.simplePermissionToString(SimplePermission.LIST_FOLDER_CONTENT), PermissionUtil.getSimplePermissionImage(SimplePermission.LIST_FOLDER_CONTENT), true, 17, SimplePermission.LIST_FOLDER_CONTENT, this);
		readFilter = new PermissionFilterRow(permissionsComposite, PermissionUtil.simplePermissionToString(SimplePermission.READ), PermissionUtil.getSimplePermissionImage(SimplePermission.READ), true, 18, SimplePermission.READ, this);
		
		//-----------------------------------
		// special permission filters header
		//-----------------------------------
		Composite specialPermissionsHeader = new Composite(permissionsComposite, SWT.NONE);
		specialPermissionsHeader.setLayout(new GridLayout(3, false));
		gridData = new GridData();
		gridData.heightHint = 30;
		gridData.verticalIndent = 10;
		specialPermissionsHeader.setLayoutData(gridData);
		
		CLabel specialPermissionsLabel = new CLabel(specialPermissionsHeader, SWT.NONE);
		specialPermissionsLabel.setText("Select special permissions:");
		//specialPermissionsLabel.setImage(ImageFactory.getInstance().getImage(AppImage.INCLUDE_PERMISSIONS_16X16));
		//specialPermissionsLabel.setFont(FontFactory.getInstance().getFont(AppFont.FILTER_SUBTITLE));
		//specialPermissionsLabel.setForeground(ColorFactory.getInstance().getColor(AppColor.FILTER_TITLE));
		gridData = new GridData();
		gridData.widthHint = 240;
		specialPermissionsLabel.setLayoutData(gridData);
		engine.setClassname(specialPermissionsLabel, "h2");
		
		ImageComposite specialAllowImage = new ImageComposite(specialPermissionsHeader, ImageFactory.getInstance().getImage(AppImage.ALLOW_20X20), "Select 'allow' permissions");
		specialAllowImage.setCursor(Display.getDefault().getSystemCursor(SWT.CURSOR_HAND));
		specialAllowImage.addMouseListener(new MouseAdapter(){
			@Override
			public void mouseUp(MouseEvent e) {
				// invert the first checkbox
				permissionFilters.get(0).setAllow(!permissionFilters.get(0).isAllow(), true, true);
				
				// change the other checkboxes to match the first one
				for (int counter = 1; counter < SIMPLE_PERMISSIONS_INDEX_OFFSET; counter++) {
					permissionFilters.get(counter).setAllow(permissionFilters.get(0).isAllow(), true, true);
				}
			}
		});
		
		ImageComposite specialDenyImage = new ImageComposite(specialPermissionsHeader, ImageFactory.getInstance().getImage(AppImage.DENY_20X20), "Select 'deny' permissions");
		((GridData)specialDenyImage.getLayoutData()).horizontalIndent = 25;
		specialDenyImage.setCursor(Display.getDefault().getSystemCursor(SWT.CURSOR_HAND));
		specialDenyImage.addMouseListener(new MouseAdapter(){
			@Override
			public void mouseUp(MouseEvent e) {
				// invert the first checkbox
				permissionFilters.get(0).setDeny(!permissionFilters.get(0).isDeny(), true, true);
				
				// change the other checkboxes to match the first one
				for (int counter = 1; counter < SIMPLE_PERMISSIONS_INDEX_OFFSET; counter++) {
					permissionFilters.get(counter).setDeny(permissionFilters.get(0).isDeny(), true, true);
				}
			}
		});
		
		readDataFilter = new PermissionFilterRow(permissionsComposite, PermissionUtil.specialPermissionToString(0, false, null), PermissionUtil.getSpecialPermissionImage(0, false, true), true, 0, null, this);
		writeDataFilter = new PermissionFilterRow(permissionsComposite, PermissionUtil.specialPermissionToString(1, false, null), PermissionUtil.getSpecialPermissionImage(1, false, true), true, 1, null, this);
		appendDataFilter = new PermissionFilterRow(permissionsComposite, PermissionUtil.specialPermissionToString(2, false, null), PermissionUtil.getSpecialPermissionImage(2, false, true), true, 2, null, this);
		readExtAttrFilter = new PermissionFilterRow(permissionsComposite, PermissionUtil.specialPermissionToString(3, false, null), PermissionUtil.getSpecialPermissionImage(3, false, true), true, 3, null, this);
		writeExtAttrFilter = new PermissionFilterRow(permissionsComposite, PermissionUtil.specialPermissionToString(4, false, null), PermissionUtil.getSpecialPermissionImage(4, false, true), true, 4, null, this);
		executeFilter = new PermissionFilterRow(permissionsComposite, PermissionUtil.specialPermissionToString(5, false, null), PermissionUtil.getSpecialPermissionImage(5, false, true), true, 5, null, this);
		deleteChildFilter = new PermissionFilterRow(permissionsComposite, PermissionUtil.specialPermissionToString(6, false, null), PermissionUtil.getSpecialPermissionImage(6, false, true), true, 6, null, this);
		readAttrFilter = new PermissionFilterRow(permissionsComposite, PermissionUtil.specialPermissionToString(7, false, null), PermissionUtil.getSpecialPermissionImage(7, false, true), true, 7, null, this);
		writeAttrFilter = new PermissionFilterRow(permissionsComposite, PermissionUtil.specialPermissionToString(8, false, null), PermissionUtil.getSpecialPermissionImage(8, false, true), true, 8, null, this);
		deleteFilter = new PermissionFilterRow(permissionsComposite, PermissionUtil.specialPermissionToString(9, false, null), PermissionUtil.getSpecialPermissionImage(9, false, true), true, 9, null, this);
		readAclFilter = new PermissionFilterRow(permissionsComposite, PermissionUtil.specialPermissionToString(10, false, null), PermissionUtil.getSpecialPermissionImage(10, false, true), true, 10, null, this);
		writeAclFilter = new PermissionFilterRow(permissionsComposite, PermissionUtil.specialPermissionToString(11, false, null), PermissionUtil.getSpecialPermissionImage(11, false, true), true, 11, null, this);
		writeOwnerFilter = new PermissionFilterRow(permissionsComposite, PermissionUtil.specialPermissionToString(12, false, null), PermissionUtil.getSpecialPermissionImage(12, false, true), true, 12, null, this);
		
		// first add the special permissions to the list, so that the index matches the permission ordinal
		permissionFilters.add(readDataFilter);
		permissionFilters.add(writeDataFilter);
		permissionFilters.add(appendDataFilter);
		permissionFilters.add(readExtAttrFilter);
		permissionFilters.add(writeExtAttrFilter);
		permissionFilters.add(executeFilter);
		permissionFilters.add(deleteChildFilter);
		permissionFilters.add(readAttrFilter);
		permissionFilters.add(writeAttrFilter);
		permissionFilters.add(deleteFilter);
		permissionFilters.add(readAclFilter);
		permissionFilters.add(writeAclFilter);
		permissionFilters.add(writeOwnerFilter);
		
		// simple permissions
		permissionFilters.add(fullFilter);
		permissionFilters.add(modifyFilter);
		permissionFilters.add(writeFilter);
		permissionFilters.add(readExecuteFilter);
		permissionFilters.add(listFolderContentFilter);
		permissionFilters.add(readFilter);
		
		// default select 'any' permission (here where the permissionFilters have been initialized) 
		filterAnyPermissionsRadio.setSelection(true);
		setFilterTypeAllOrAny(false);
		dirty.setDirty(false);
	}
	
	/**
	 * Returns the selected <code>FilterSet</code> based on the name in the dropdown box
	 * 
	 * @return selected <code>FilterSet</code>
	 */
	public FilterSet getSelectedFilterSet() {
		String name = permissionSelectionCombo.getItem(permissionSelectionCombo.getSelectionIndex());
		
		// get the selected filter set
		for (FilterSet filterSet : filterSets) {
			if (filterSet.getName().equals(name)) {
				try {
					filterSet.loadFilters();
				} catch (DBException e) {
					logger.error("Error while loading the filters for filter set " + filterSet.getName(), e);
					SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while loading the filters for filter set " + filterSet.getName() + ": " + e.getMessage());
				}
				return filterSet;
			}
		}
		return null;
	}
	
	/**
	 * Updates the GUI after a new set of filters has been loaded. Either by selecting a filter set from the dropdown or by the method {@link #loadNewFilters(List)}.
	 */
	private void refreshFiltersInGUI() {
		// clear all checkboxes
		for (PermissionFilterRow filterRow : permissionFilters) {
			filterRow.setAllow(false, false, false);
			filterRow.setDeny(false, false, false);
		}
		
		// radio buttons
		FilterConfig filterConfig = FilterHelper.getInstance().getFilterConfig(true);
		filterAnyPermissionsRadio.setSelection(!filterConfig.isAceNeedAllPermissions());
		filterAllPermissionsRadio.setSelection(filterConfig.isAceNeedAllPermissions());
		includeFilterRadio.setSelection(filterConfig.isIncludePermissions());
		excludeFilterRadio.setSelection(!filterConfig.isIncludePermissions());
		hideInheritedPermissionsButton.setSelection(filterConfig.isHideInheritedPermissions());
		
		// permission checkboxes
		for (Filter filter : FilterHelper.getInstance().getPermissionFilterList()) {
			if (filter.getType().equals(FilterType.PERMISSION_ALLOW)) {
				// checkbox for the allow column
				permissionFilters.get(Integer.parseInt(filter.getValue())).setAllow(true, false, false);
				
			} else if (filter.getType().equals(FilterType.PERMISSION_DENY)) {
				// checkbox for the deny column
				permissionFilters.get(Integer.parseInt(filter.getValue())).setDeny(true, false, false);				
			}
		}
		
		permissionSelectionCombo.select(0);
		
		dirty.setDirty(false);
	}

		
	/**
	 * Updates {@link FilterHelper#getPermissionFilterList()} and updates the filter with type {@value FilterType#FILTER_CONFIG}. This filter determines
	 * whether or not a ACE must have ALL the permissions selected or ANY of the permissions selected.
	 * 
	 * @param aceNeedsAllPermissions true in case ACE's must have all the selected permissions
	 */
	private void setFilterTypeAllOrAny(boolean aceNeedsAllPermissions) {
		dirty.setDirty(true);
		FilterHelper.getInstance().getFilterConfig(true).setAceNeedAllPermissions(aceNeedsAllPermissions);
		
		// refresh the checkboxes (simple permission may affect the special permissions)
		permissionFilters.get(0).checkboxChanged(true, permissionFilters.get(0).isAllow());
		permissionFilters.get(0).checkboxChanged(false, permissionFilters.get(0).isDeny());				
	}
	
	/** Sets all filters to their default value */
	public void resetFilters() {
		//update GUI
		permissionSelectionCombo.select(0);
		deleteSelectionButton.setEnabled(false);
		permissionSelectionComboIndex = 0;
				
		// reset radio buttons
		filterAnyPermissionsRadio.setSelection(true);
		filterAllPermissionsRadio.setSelection(false);
		includeFilterRadio.setSelection(true);
		excludeFilterRadio.setSelection(false);
		hideInheritedPermissionsButton.setSelection(false);
		
		setFilterTypeAllOrAny(false);
		FilterHelper.getInstance().getFilterConfig(true).setIncludePermissions(true);
								
		// select the permission checkboxes
		for (int counter = 0; counter < permissionFilters.size(); counter++) {
			permissionFilters.get(counter).setAllow(true, true, true);
			permissionFilters.get(counter).setDeny(true, true, true);
		}
		
		dirty.setDirty(false);
	}
	
	/** 
	 * Load a new FilterSet in the GUI. This method is triggered by the onChange of {@link #permissionSelectionCombo} or from the Load Report button in the {@link ReportDialog}.
	 * 
	 * @param filterSet FilterSet to load
	 */
	public boolean loadNewFilterSet(FilterSet filterSet) {
		boolean filtersLoaded = false;
		
		// get the selected filter set
		if (filterSet != null) {
			try {
				// get related filters
				filterSet.loadFilters();
				filtersLoaded = loadNewFilters(filterSet.getFilters());
				
			} catch (DBException ex) {
				SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while loading filters for the selected filter set: " + ex.getMessage()); 
			}
		}
		
		if (filtersLoaded) { // not cancelled?
			deleteSelectionButton.setEnabled(true);
			
			// select the dropdown item (this is not yet done in case the method is called from the ManageReportsHandler
			for (int counter = 0; counter < permissionSelectionCombo.getItemCount(); counter++) {
				if (permissionSelectionCombo.getItem(counter).equalsIgnoreCase(filterSet.getName())) {
					permissionSelectionCombo.select(counter);
					permissionSelectionComboIndex = counter;
					break;
				}
			}
		}
		
		return filtersLoaded;
	}
	
	/** 
	 * Load a new Filters in the GUI. This method is used by {@link #loadNewFilterSet(FilterSet)} and triggered by the Load Report button in the {@link ReportDialog}.
	 * 
	 * @param filterSet FilterSet to load
	 */
	public boolean loadNewFilters(List<Filter> filters) {
		if (dirty.isDirty() && !MessageDialog.openQuestion(Display.getCurrent().getActiveShell(), "Unsaved permission filters", "Are you sure you want to overwrite the current permission filters?")) {
			//user cancelled, select last index
			permissionSelectionCombo.select(permissionSelectionComboIndex);
			return false;
		} else {
			//load the new filters
			FilterHelper.getInstance().getPermissionFilterList().clear();
			FilterHelper.getInstance().getPermissionFilterList().addAll(filters);
			refreshFiltersInGUI();
			return true;
		}
	}
	
	@Persist
	private void saveFilterSet() {
		try {
			int selectedIndex = -1;
			FilterSet filterSet = null;
			if (!permissionSelectionCombo.getText().equals("Custom")) {
				filterSet = getSelectedFilterSet();
				selectedIndex = filterSets.indexOf(filterSet);
			}
			
			SaveFilterDialog dialog = new SaveFilterDialog(Display.getCurrent().getActiveShell(), filterSets, selectedIndex, "Permission filter sets");
			int result = dialog.open(); 
			
			if (result == IDialogConstants.OK_ID) {;
				// existing filter set is selected	
				filterSet = (FilterSet) dialog.getSelectedFilterSet();
				
				// remove all the current filters
				Filter.delete("FILTERSET_ID = ?", filterSet.getId());
				filterSet.setFilters(new ArrayList<Filter>());
				
			} else if (result == SaveFilterDialog.CREATE_ID) {
				// a new filter set is created
				filterSet = (FilterSet) dialog.getSelectedFilterSet();
				filterSet.setType(FilterSetType.PERMISSION);
				filterSet.setFilters(new ArrayList<Filter>());
				filterSet.save();
				
				filterSets.add(filterSet);
				permissionSelectionCombo.add(filterSet.getName());
			} else {
				// user cancelled
				return;
			}
			
			// add all the current permission filter items to the filter set
			for (Filter filter :  FilterHelper.getInstance().getPermissionFilterList()) {
				// make sure to create a new filter
				filter.setId(null);
				filter.setFilterSetId(filterSet.getId());
				filter.save();
				filterSet.getFilters().add(filter);
			}
			
			selectedIndex = filterSets.indexOf(filterSet);
			permissionSelectionCombo.select(selectedIndex + 1); //skip the first item "Custom"
			deleteSelectionButton.setEnabled(true);
			
			dirty.setDirty(false);
			
		} catch (DBException ex) {
			MessageDialog.openError(null, "Error", "Error while saving filter: " + ex.getMessage());
		}
	}
}

/**
 * Helper class that represents one row with filter controls. Such a row contains a label with the permission name, a checkbox for the allow column and a checkbox for the deny column.
 * The class adds a SelectionListener to the checkboxes to react on changes. (there's a mapping between special permissions and simple permission)
 * 
 * @see PermissionUtil#permissionMappings
 * @see PermissionUtil#SIMPLE_PERMISSION
 */
class PermissionFilterRow {
	// UI controls
	private Button allowCheckbox;
	private Button denyCheckbox;
	private int index;
	private SimplePermission simplePermission;
	private FilterPermissionPart filterPermissionPart;
	
	public PermissionFilterRow(Composite parent, String label, Image image, boolean selected, final int index, final SimplePermission simplePermission, final FilterPermissionPart filterPermissionPart) {
		this.index = index;
		this.simplePermission = simplePermission;
		this.filterPermissionPart = filterPermissionPart;
		
		Composite permissionFilterRow = new Composite(parent, SWT.NONE);
		GridData gridData = new GridData();
		gridData.heightHint = 24;
		permissionFilterRow.setLayoutData(gridData);
		GridLayout gridLayout = new GridLayout(3, false);
		gridLayout.marginHeight = 0;
		gridLayout.verticalSpacing = 0;
		permissionFilterRow.setLayout(gridLayout);
		
		CLabel permissionLabel = new CLabel(permissionFilterRow, SWT.NONE);
		permissionLabel.setImage(image);
		permissionLabel.setText(label);
		gridData = new GridData();
		gridData.widthHint = 240;
		permissionLabel.setLayoutData(gridData);
		
		allowCheckbox = new Button(permissionFilterRow, SWT.CHECK);
		allowCheckbox.setLayoutData(new GridData());
		allowCheckbox.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				checkboxChanged(true, allowCheckbox.getSelection());
				
				// update the model
				updateFilterValue(allowCheckbox.getSelection(), FilterType.PERMISSION_ALLOW);
				filterPermissionPart.dirty.setDirty(true);
			}
		});
		setAllow(selected, true, true);
		
		denyCheckbox = new Button(permissionFilterRow, SWT.CHECK);
		gridData = new GridData();
		gridData.horizontalIndent = 30;
		denyCheckbox.setLayoutData(gridData);
		denyCheckbox.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				checkboxChanged(false, denyCheckbox.getSelection());
				
				// update the model
				updateFilterValue(denyCheckbox.getSelection(), FilterType.PERMISSION_DENY);
				filterPermissionPart.dirty.setDirty(true);
			}
		});
		setDeny(selected, true, true);
	}
	
	/**
	 * Triggered when the user checks or unchecks one of the checkboxes. This method determines if other checkboxes need to change as well.
	 * 
	 * <p>For example, when the option "Select ACE's having ALL of the following permissions" has been selected then the selected
	 * simple permissions must match with the special permissions. When a ACE needs to have all special permissions
	 * then it automatically needs to have FULL permission because that's what the special permission implies.</p>	
	 * 
	 * @param allow true in case the checkbox corresponds to a allow permission. False for deny permissions.
	 * @param checked true in case the checkbox has been checked.
	 */
	public void checkboxChanged(boolean allow, boolean checked) {
		// when a simple permission checkbox is UNchecked then all the corresponding special permissions are unchecked. This does not depend
		// on the option "Select ACE's having ALL of the selected permissions", in both cases we want to uncheck the special permissions
		if (simplePermission != null && !checked) {
			for (Integer permissionFilterIndex : PermissionLookup.permissionMappings.get(simplePermission)) {
				if (allow) {
					FilterPermissionPart.permissionFilters.get(permissionFilterIndex).setAllow(false, false, true);
				} else {
					FilterPermissionPart.permissionFilters.get(permissionFilterIndex).setDeny(false, false, true);
				}
			}
		}
		
		// when the option "Select ACE's having ALL of the following permissions" has been selected then the selected
		// simple permissions must match with the special permissions. When a ACE needs to have all special permissions
		// then it automatically needs to have FULL permission because that's what the special permission implies.
		if (FilterPermissionPart.filterAllPermissionsRadio.getSelection()) {
			
			// When an ACE needs ALL the selected permissions then it is not possible to select ALLOW and DENY permissions, because an ACE
			// cannot have both type of permissions
			boolean hasAllow = false;
			boolean hasDeny = false;
			
			// explicit checked by the user?
			if (checked) {
				hasAllow = allow;
				hasDeny = !allow;
			} else {
				// possibly triggered when the user switched the option ALL permissions or ANY permissions
				// determine if both allow and deny permissions are selected
				for (PermissionFilterRow row : FilterPermissionPart.permissionFilters) {
					hasAllow = hasAllow || row.isAllow();
					hasDeny = hasDeny || row.isDeny();
				}
			}
			
			if (hasAllow) {
				// uncheck all deny permissions
				for (PermissionFilterRow row : FilterPermissionPart.permissionFilters) {
					row.setDeny(false, false, true);
				}
			} else if (hasDeny) {
				// uncheck all allow permissions
				for (PermissionFilterRow row : FilterPermissionPart.permissionFilters) {
					row.setAllow(false, false, true);
				}				
			}
			
			// simple permission checkbox? Only when the checkbox is checked, the uncheck scenario is handled above (for both ALL permissions and ANY permissions)
			if (simplePermission != null && checked) {
				for (Integer permissionFilterIndex : PermissionLookup.permissionMappings.get(simplePermission)) {
					if (allow) {
						FilterPermissionPart.permissionFilters.get(permissionFilterIndex).setAllow(true, false, true);
					} else {
						FilterPermissionPart.permissionFilters.get(permissionFilterIndex).setDeny(true, false, true);
					}
				}
			}
			
			// special permission(s) updated, check if any simple permissions are covered by the special permissions
			for (int simplePermissionIndex = 0; simplePermissionIndex < PermissionLookup.permissionMappings.size(); simplePermissionIndex++) {
				Integer[] specialPermissions = PermissionLookup.permissionMappings.get(SimplePermission.values()[simplePermissionIndex]);
				boolean simplePermissionApplicable = true;
				for (Integer specialPermissionIndex : specialPermissions) {
					if (allow) {
						simplePermissionApplicable = simplePermissionApplicable && FilterPermissionPart.permissionFilters.get(specialPermissionIndex).isAllow();
					} else {
						simplePermissionApplicable = simplePermissionApplicable && FilterPermissionPart.permissionFilters.get(specialPermissionIndex).isDeny();
					}
				}
				
				if (allow) {
					FilterPermissionPart.permissionFilters.get(FilterPermissionPart.SIMPLE_PERMISSIONS_INDEX_OFFSET + simplePermissionIndex).setAllow(simplePermissionApplicable, false, true);
				} else {
					FilterPermissionPart.permissionFilters.get(FilterPermissionPart.SIMPLE_PERMISSIONS_INDEX_OFFSET + simplePermissionIndex).setDeny(simplePermissionApplicable, false, true);
				}
			}
			
		// if an ACE may have any of the selected permissions and a special permission is CHECKED, then we must also check the simple permissions that have the
		// special permissions in them.
		} else if (FilterPermissionPart.filterAnyPermissionsRadio.getSelection() && simplePermission == null && checked) {
			
			for (int simplePermissionIndex = 0; simplePermissionIndex < PermissionLookup.permissionMappings.size(); simplePermissionIndex++) {
				Integer[] specialPermissions = PermissionLookup.permissionMappings.get(SimplePermission.values()[simplePermissionIndex]);
				for (Integer specialPermission : specialPermissions) {
					if (FilterPermissionPart.permissionFilters.get(specialPermission).isAllow()) {
						// check this simple permission, because it has this special permission which has been checked by the user
						FilterPermissionPart.permissionFilters.get(FilterPermissionPart.SIMPLE_PERMISSIONS_INDEX_OFFSET + simplePermissionIndex).setAllow(true, false, true);
					}
					
					if (FilterPermissionPart.permissionFilters.get(specialPermission).isDeny()) {
						// check this simple permission, because it has this special permission which has been checked by the user
						FilterPermissionPart.permissionFilters.get(FilterPermissionPart.SIMPLE_PERMISSIONS_INDEX_OFFSET + simplePermissionIndex).setDeny(true, false, true);
					}
				}						
			}
		}
	}
	
	/**
	 * Adds or removes the {@link Filter} object that belongs to this {@link PermissionFilterRow}.
	 * 
	 * @param selected value of the checkbox
	 * @param filterType either PERMISSION_ALLOW or PERMISSION_DENY
	 */
	private void updateFilterValue(boolean selected, FilterType filterType) {
		//update the filter value in FilterHelper, remove or add
		if (selected) {
			// add the filter for this permission
			FilterHelper.getInstance().getPermissionFilterList().add(new Filter(filterType, true, String.valueOf(index), "Index of the selected permission"));
		} else {
			// remove the filter for this permission
			Iterator<Filter> iter = FilterHelper.getInstance().getPermissionFilterList().iterator();
			while (iter.hasNext()) {
				Filter filter = iter.next();
				if (filter.getType().equals(filterType) && filter.getValue().equals(String.valueOf(index))) {
					iter.remove();
				}
			}
		}
	}
	
	public boolean isAllow() {
		return allowCheckbox.getSelection();
	}
	
	public void setAllow(boolean allow, boolean checkForOtherCheckboxes, boolean updateModel) {
		if (allowCheckbox.getSelection() != allow) {
			allowCheckbox.setSelection(allow);
			if (checkForOtherCheckboxes) {
				allowCheckbox.notifyListeners(SWT.Selection, new Event());
			} else if (updateModel) {
				// update the model
				updateFilterValue(allow, FilterType.PERMISSION_ALLOW);
			}
		}
	}
	
	public boolean isDeny() {
		return denyCheckbox.getSelection();
	}
	
	public void setDeny(boolean deny, boolean checkForOtherCheckboxes, boolean updateModel) {
		if (denyCheckbox.getSelection() != deny) {
			denyCheckbox.setSelection(deny);
			if (checkForOtherCheckboxes) {
				denyCheckbox.notifyListeners(SWT.Selection, new Event());
			} else if (updateModel) {
				// update the model
				updateFilterValue(deny, FilterType.PERMISSION_DENY);
			}
		}
	}
}