package com.perdemia.permissionanalyzer.application.parts;

import java.lang.reflect.InvocationTargetException;
import java.sql.PreparedStatement;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.inject.Inject;
import javax.inject.Named;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.e4.ui.workbench.modeling.ESelectionService;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ILazyContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Text;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.jface.dialogs.MemberInfoDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.jface.labelproviders.TableLabelProvider;
import com.perdemia.permissionanalyzer.application.swt.DelayedKeyListenerText;
import com.perdemia.permissionanalyzer.application.util.ApplicationUtil;
import com.perdemia.permissionanalyzer.application.util.Constants;
import com.perdemia.permissionanalyzer.application.util.FilterHelper;
import com.perdemia.permissionanalyzer.application.util.FilterHelper.ObjectType;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.data.AclItem;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.FileInfo;
import com.perdemia.permissionanalyzer.data.Member;

/**
 * Replaced by the {@link EffectivePermissionsPart} and {@link AllMembersPart}
 * @author Reinier
 *
 */
@Deprecated 
public class MemberScopePart {
	public static final String ID = "com.perdemia.permissionanalyzer.application.parts.memberscope";
	private static Logger logger = LoggerFactory.getLogger(MemberScopePart.class.getName());
	
	/** GUI controls for the selected file */
	private Text membersOnDirectorySearchText;
	private TableViewer membersOnDirectoryTableViewer;
	private Button linkToFolderSelectionButton;
	private Button showUsersOnDirectoryCheckbox;
	private Button showGroupsOnDirectoryCheckbox;
		
	/** GUI controls for all the members found in the results */
	private Text allMembersSearchText;
	private TableViewer allMembersTableViewer;
	private Button showAllUsersCheckbox;
	private Button showAllGroupsCheckbox;
	
	/** Data */
	private FileInfo selectedFile;
	private List<Member> membersOnSelectedFolderList;
	private List<Member> allMembersList;
	
	@Inject ESelectionService selectionService;
	@Inject EPartService partService;
	
	/**
	 * Receives selection events from the main FileInfo tree and updates the input for the membersOnDirectoryTableViewer viewer
	 */
	@Inject
	public void fileSelected(@Optional @Named(IServiceConstants.ACTIVE_SELECTION) IStructuredSelection selectedItem) {
		if (linkToFolderSelectionButton == null || linkToFolderSelectionButton.isDisposed() || linkToFolderSelectionButton.getSelection()) {
						
			if (selectedItem != null && !selectedItem.isEmpty() && selectedItem.getFirstElement() instanceof FileInfo) {
				// reset
				membersOnSelectedFolderList = null;
							
				// apparently this line is necessary because when the users switches between the AclPart and TracePart this method is called
				// with an empty selectedPath (???)
				selectedItem = (IStructuredSelection) selectionService.getSelection(FilePart.ID);
				selectedFile = (FileInfo) selectedItem.getFirstElement();
				
				if (selectedFile != null && selectedFile.getAclItems().size() > 0) {
					if (FilterHelper.getInstance().getFilterConfig(false).isIncludeGroupMembership()) {
						// use a base query to search for member because of the 'include group membership' we might want to search for other members than the ones attached to the ACL
						try {
							membersOnSelectedFolderList  = Member.getByPreparedStatement(getMembersThatMatchFilterPS(0));
						} catch (DBException ex) {
							logger.error("Error while getting member list for the selected file", ex);
							return;
						}
					} else {
						// just search through the list of members attached directly to the selected ACL
						membersOnSelectedFolderList = new ArrayList<Member>();
						for (AclItem aclItem : selectedFile.getAclItems()) {
							Member member = aclItem.getMember(true);
							
							// check if this member is not already in the list
							boolean alreadyInList = false;
							for (Member nextMember : membersOnSelectedFolderList) {
								if (nextMember.getId().equals(member.getId())) {
									alreadyInList = true;
									break;
								}
							}
							
							if (!alreadyInList) {
								membersOnSelectedFolderList.add(member);
							}
						}
					}
				}
				
				// refresh the list of members for the selected directory
				if (linkToFolderSelectionButton != null && !linkToFolderSelectionButton.isDisposed()) {
					searchMembersOnSelectedFile(true, false);
				}
			}
		}
	}
	
	@PostConstruct
	public void createComposite(Composite parent) {
		ScrolledComposite scrollableComposite = new ScrolledComposite(parent, SWT.NONE | SWT.H_SCROLL | SWT.V_SCROLL);
		scrollableComposite.setMinSize(200, 500);
		scrollableComposite.setExpandHorizontal(true);
		scrollableComposite.setExpandVertical(true);
		scrollableComposite.setBackgroundMode(SWT.INHERIT_DEFAULT);
		
		final Composite mainContent = new Composite(scrollableComposite, SWT.NONE);
		mainContent.setLayout(new GridLayout(1, false));
		mainContent.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		scrollableComposite.setContent(mainContent);
		
		// Title label
		CLabel searchScopeLabel = new CLabel(mainContent, SWT.NONE);
		searchScopeLabel.setImage(ImageFactory.getInstance().getImage(AppImage.SEARCH_USER_24X24));
		searchScopeLabel.setText("Found multiple members:");
		searchScopeLabel.setData(Constants.CSS_CLASS_NAME_KEY, "h2");
		GridData gridData = new GridData();
		gridData.verticalAlignment = SWT.CENTER;
		searchScopeLabel.setLayoutData(gridData);

		// -----------------------------------------------
		// List of members found on the current directory
		// -----------------------------------------------
		Label membersOnDirectoryLabel = new Label(mainContent, SWT.WRAP);
		membersOnDirectoryLabel.setText("Matching members on the selected file:");
		membersOnDirectoryLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
				
		// search box for members found on the selected directory
		membersOnDirectorySearchText = DelayedKeyListenerText.create(mainContent, SWT.BORDER, new KeyAdapter() {
			@Override
			public void keyReleased(KeyEvent e) {
				searchMembersOnSelectedFile(e.keyCode == 13, true);
			}			
		});
		membersOnDirectorySearchText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		membersOnDirectorySearchText.setMessage("search");
		
		// filter checkbox for users
		Composite checkboxes = new Composite(mainContent, SWT.NONE);
		GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.verticalSpacing = 10;
		checkboxes.setLayout(gridLayout);
		checkboxes.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		showUsersOnDirectoryCheckbox = new Button(checkboxes, SWT.CHECK);
		showUsersOnDirectoryCheckbox.setLayoutData(new GridData(GridData.BEGINNING));
		showUsersOnDirectoryCheckbox.setText("Show users");
		showUsersOnDirectoryCheckbox.setSelection(true);
		showUsersOnDirectoryCheckbox.setImage(ImageFactory.getInstance().getImage(AppImage.USER_20X20));
		showUsersOnDirectoryCheckbox.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				searchMembersOnSelectedFile(true, false);
			}
		});
		
		// filter checkbox for groups
		showGroupsOnDirectoryCheckbox = new Button(checkboxes, SWT.CHECK);
		showGroupsOnDirectoryCheckbox.setLayoutData(new GridData(GridData.BEGINNING));
		showGroupsOnDirectoryCheckbox.setText("Show groups");
		showGroupsOnDirectoryCheckbox.setSelection(true);
		showGroupsOnDirectoryCheckbox.setImage(ImageFactory.getInstance().getImage(AppImage.GROUP_20X20));
		showGroupsOnDirectoryCheckbox.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				searchMembersOnSelectedFile(true, false);
			}
		});
		
		linkToFolderSelectionButton = new Button(checkboxes, SWT.CHECK | SWT.WRAP);
		linkToFolderSelectionButton.setImage(ImageFactory.getInstance().getImage(AppImage.FOLDER_LINK_16X16));
		linkToFolderSelectionButton.setToolTipText("Refresh list when a folder or file is selected");
		linkToFolderSelectionButton.setText("Refresh on folder selection");
		linkToFolderSelectionButton.setSelection(true);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		linkToFolderSelectionButton.setLayoutData(gridData);
		
		//create a one column TableViewer so an image can be used
		membersOnDirectoryTableViewer = new TableViewer(mainContent, SWT.BORDER | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.VIRTUAL);
		membersOnDirectoryTableViewer.setUseHashlookup(true);
		membersOnDirectoryTableViewer.getTable().setLayoutData(new GridData(GridData.FILL_BOTH));
		
		final TableViewerColumn memberColumn = new TableViewerColumn(membersOnDirectoryTableViewer, SWT.NONE);
		membersOnDirectoryTableViewer.getTable().addControlListener(new ControlListener(){
			@Override
			public void controlMoved(ControlEvent e) {}

			@Override
			public void controlResized(ControlEvent e) {
				memberColumn.getColumn().setWidth(membersOnDirectoryTableViewer.getTable().getSize().x-4);
			}
		});
		
		membersOnDirectoryTableViewer.setContentProvider(new LazyTableContentProvider());
		membersOnDirectoryTableViewer.setLabelProvider(new MemberLabelProvider());
				
		Button addToTraceViewButton = new Button(mainContent, SWT.PUSH);
		addToTraceViewButton.setImage(ImageFactory.getInstance().getImage(AppImage.TRACE_16X16));
		addToTraceViewButton.setToolTipText("Put the selected member in the Trace view and see where the permissions for this user come from.");
		addToTraceViewButton.setText("Put the selected member in the Trace view");
		addToTraceViewButton.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		addToTraceViewButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				StructuredSelection selection = (StructuredSelection) membersOnDirectoryTableViewer.getSelection();
				if (!selection.isEmpty()) {
					MPart part = partService.findPart(TracePart.ID);
					partService.activate(part, true);
					TracePart tracePart = (TracePart) part.getObject();
					tracePart.applySelectedMember((Member)selection.getFirstElement());
				} else {
					MessageDialog.openWarning(null, "Select a member", "Please select a member in the list above.");
				}
			}
		});
		
		// ---------------------------------
		// All members found list
		// ---------------------------------
		// header bar
		Composite membersFoundHeaderBar = new Composite(mainContent, SWT.NONE);
		membersFoundHeaderBar.setLayout(new GridLayout(3, false));
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		membersFoundHeaderBar.setLayoutData(gridData);
		
		Label membersFoundLabel = new Label(membersFoundHeaderBar, SWT.WRAP);
		membersFoundLabel.setText("All members in the results:");
		membersFoundLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		// load the list with all the members found in the directory tree
		Button loadAllMembersButton = new Button(membersFoundHeaderBar, SWT.PUSH);
		loadAllMembersButton.setImage(ImageFactory.getInstance().getImage(AppImage.LOAD_LIST_16X16));
		loadAllMembersButton.setToolTipText("Load the list with all the members found in the directory tree");
		loadAllMembersButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		loadAllMembersButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				ProgressMonitorDialog progressDialog = new ProgressMonitorDialog(Display.getDefault().getActiveShell());
				try {
					progressDialog.run(true, false, new IRunnableWithProgress(){
						@Override
						public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
							try {
								monitor.beginTask("Getting all members from the results...", IProgressMonitor.UNKNOWN);
								allMembersList  = Member.getByPreparedStatement(FilterHelper.getInstance().buildFilterQuery(ObjectType.MEMBERS_IN_FILE_TREE, false, false, 0, null, null));
								
								// refresh the TableViewer
								Display.getDefault().asyncExec(new Runnable() {
									public void run() {
										searchAllMembers(true);
									}
								});
							} catch (DBException ex) {
								logger.error("Error while getting all members from the results", ex);
								SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while getting all members from the results: " + ex.getMessage());
								return;
							}
						}
					});
				} catch (InvocationTargetException | InterruptedException ex) {
					logger.error("Error while getting all members from the results", ex);
					SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while getting all members from the results: " + ex.getMessage());
					return;
				}
			}
		});
		
		// load the list with all the members found in the directory tree
		Button copyMembersToClipboardButton = new Button(membersFoundHeaderBar, SWT.PUSH);
		copyMembersToClipboardButton.setImage(ImageFactory.getInstance().getImage(AppImage.CLIPBOARD_16X16));
		copyMembersToClipboardButton.setToolTipText("Copy the list with all the members to the clipboard");
		copyMembersToClipboardButton.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
		copyMembersToClipboardButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (allMembersList != null) {
					StringBuilder membersText = new StringBuilder("Account name, Display name\r\n");
					for (Member member : allMembersList) {
						membersText.append(member.getName(true));
						membersText.append(", ");
						membersText.append(member.getDisplayName());
						membersText.append("\r\n");
					}
					Clipboard clipboard = new Clipboard(Display.getDefault());
					TextTransfer textTransfer = TextTransfer.getInstance();
			        clipboard.setContents(new String[]{membersText.toString()}, new Transfer[]{textTransfer});
			        clipboard.dispose();
			        SyncMessageDialog.showMessage(MessageDialog.INFORMATION, "Done", "Copied " + allMembersList.size() + " members to the clipboard.");
				} else {
					SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "The list with all the members has not been loaded yet, please hit the load button first.");
				}
			}
		});
				
		// search box for all members list
		allMembersSearchText = DelayedKeyListenerText.create(mainContent, SWT.BORDER, new KeyAdapter() {
			@Override
			public void keyReleased(KeyEvent e) {
				searchAllMembers(e.keyCode == 13);
			}			
		});
		allMembersSearchText.setMessage("search");
		allMembersSearchText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
					
		// filter checkbox for users
		checkboxes = new Composite(mainContent, SWT.NONE);
		checkboxes.setLayout(new GridLayout(2, false));
		checkboxes.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		showAllUsersCheckbox = new Button(checkboxes, SWT.CHECK);
		showAllUsersCheckbox.setLayoutData(new GridData(GridData.BEGINNING));
		showAllUsersCheckbox.setText("Show users");
		showAllUsersCheckbox.setSelection(true);
		showAllUsersCheckbox.setImage(ImageFactory.getInstance().getImage(AppImage.USER_20X20));
		showAllUsersCheckbox.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				searchAllMembers(true);
			}
		});
		
		// filter checkbox for groups
		showAllGroupsCheckbox = new Button(checkboxes, SWT.CHECK);
		showAllGroupsCheckbox.setLayoutData(new GridData(GridData.BEGINNING));
		showAllGroupsCheckbox.setText("Show groups");
		showAllGroupsCheckbox.setSelection(true);
		showAllGroupsCheckbox.setImage(ImageFactory.getInstance().getImage(AppImage.GROUP_20X20));
		showAllGroupsCheckbox.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				searchAllMembers(true);
			}
		});
		
		//create a one column TableViewer so an image can be used
		allMembersTableViewer = new TableViewer(mainContent, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.VIRTUAL);
		allMembersTableViewer.setUseHashlookup(true);
		gridData = new GridData(GridData.FILL_BOTH);
		allMembersTableViewer.getTable().setLayoutData(gridData);
		
		final TableViewerColumn groupColumn = new TableViewerColumn(allMembersTableViewer, SWT.NONE);
		
		allMembersTableViewer.getTable().addControlListener(new ControlListener(){
			@Override
			public void controlMoved(ControlEvent e) {}

			@Override
			public void controlResized(ControlEvent e) {
				groupColumn.getColumn().setWidth(allMembersTableViewer.getTable().getSize().x-4);
			}
		});
		
		allMembersTableViewer.setContentProvider(new LazyTableContentProvider());
		allMembersTableViewer.setLabelProvider(new MemberLabelProvider());
				
		Button addToMainFilterButton = new Button(mainContent, SWT.PUSH);
		addToMainFilterButton.setImage(ImageFactory.getInstance().getImage(AppImage.FILTER_20X20));
		addToMainFilterButton.setToolTipText("Adds the selected members as includes in the main filter to scope the data to the selected members.");
		addToMainFilterButton.setText("Add members to the main filter");
		addToMainFilterButton.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		addToMainFilterButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				StructuredSelection selection = (StructuredSelection) allMembersTableViewer.getSelection();
				if (!selection.isEmpty()) {
					MPart part = partService.findPart(FilterMemberPart.ID);
					FilterMemberPart filterMemberPart = (FilterMemberPart) part.getObject();
					
					// loop through all the selected items
					@SuppressWarnings("rawtypes")
					Iterator iter = selection.iterator();
					while (iter.hasNext()) {
						Member selectedMember = (Member)iter.next();
						filterMemberPart.addNewMemberFilter(selectedMember, true);
						allMembersList.remove(selectedMember);
						
					}
					allMembersTableViewer.refresh();
				}
			}
		});
		
		//----------------------------------------------
		// context menu for the tables
		//----------------------------------------------
		
		// context menu for the membersOnDirectoryTableViewer
		MenuManager memberMenuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
		memberMenuMgr.setRemoveAllWhenShown(true);
		memberMenuMgr.addMenuListener(new IMenuListener() {
			@Override
			public void menuAboutToShow(IMenuManager manager) {
				final IStructuredSelection selection = (IStructuredSelection) membersOnDirectoryTableViewer.getSelection();
				if (selection != null && !selection.isEmpty()) {
					final Member member = ((Member) selection.getFirstElement());
					Action action = new Action() {
						public void run() {
							MemberInfoDialog dialog = new MemberInfoDialog(null, member);
							dialog.open();
						}
					};
					action.setText("Show member info");
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.USER_INFO_16X16));
					manager.add(action);
				}
			}
		});
		Menu memberMenu = memberMenuMgr.createContextMenu(membersOnDirectoryTableViewer.getTable());
		membersOnDirectoryTableViewer.getTable().setMenu(memberMenu);
				
		// context menu for the groupFilterListViewer
		MenuManager groupMenuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
		groupMenuMgr.setRemoveAllWhenShown(true);
		groupMenuMgr.addMenuListener(new IMenuListener() {
			@Override
			public void menuAboutToShow(IMenuManager manager) {
				final IStructuredSelection selection = (IStructuredSelection) allMembersTableViewer.getSelection();
				if (selection != null && !selection.isEmpty()) {
					final Member member = ((Member) selection.getFirstElement());
					Action action = new Action() {
						public void run() {
							MemberInfoDialog dialog = new MemberInfoDialog(null, member);
							dialog.open();
						}
					};
					action.setText("Show member info");
					action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.USER_INFO_16X16));
					manager.add(action);
				}
			}
		});
		Menu groupMenu = groupMenuMgr.createContextMenu(allMembersTableViewer.getTable());
		allMembersTableViewer.getTable().setMenu(groupMenu);
		
		// intialize the list of members on the selected directory
		searchMembersOnSelectedFile(true, false);
	}
	
	/**
	 * Refreshes the list with members on the selected file, considering the search text entered by the user
	 * 
	 * @param explicitUserSearchAction true in case the user pressed the search button or ENTER in the search text box
	 */
	private void searchMembersOnSelectedFile(boolean explicitUserSearchAction, boolean showMessage) {
		if (!showUsersOnDirectoryCheckbox.getSelection() && !showGroupsOnDirectoryCheckbox.getSelection()) {
			if (explicitUserSearchAction) {
				MessageDialog.openError(Display.getCurrent().getActiveShell(), "Invalid selection", "Select at least one of the checkboxes to show users and/or groups.");
			}
			return;
		}
		
		String search = membersOnDirectorySearchText.getText().toLowerCase().replace('*', '%');
		
		// only search when the user explicit requested the search OR when the length is greater than 2 and doesn't start with % OR the search list is based on the baseList
		if (explicitUserSearchAction ||	(search.length() > 2 && !search.startsWith("%"))) {
			
			membersOnDirectoryTableViewer.getTable().clearAll();
			
			if (!search.endsWith("%")) {
				search += "%";
			}
			
			ApplicationUtil.setBusyCursor();
			List<Member> members = new ArrayList<Member>();
			
			if (membersOnSelectedFolderList != null){
				members = new ArrayList<Member>();
				search = search.replace("%", "");
				for (Member member : membersOnSelectedFolderList) {
					// match display name or member name?
					if ((member.getDisplayName(false, true).toLowerCase().contains(search.toLowerCase()) || member.getName(false).toLowerCase().contains(search.toLowerCase()))
							// filter on groups or users?
							&& ((member.isGroup() && showGroupsOnDirectoryCheckbox.getSelection()) || (!member.isGroup() && showUsersOnDirectoryCheckbox.getSelection()))) {
						members.add(member);
					}
				}
			}
			
			membersOnDirectoryTableViewer.setItemCount(members.size());
			membersOnDirectoryTableViewer.setInput(members);
							
			if (members.size() == 0 && explicitUserSearchAction && showMessage){
				MessageDialog.openInformation(Display.getCurrent().getActiveShell(), "No results", "No members found.");
			}
			
			membersOnDirectoryTableViewer.refresh();
			
			ApplicationUtil.setDefaultCursor();
		}		
	}
	
	/**
	 * Refreshes the list with all members, considering the search text entered by the user
	 * 
	 * @param explicitUserSearchAction true in case the user pressed the search button or ENTER in the search text box
	 */
	private void searchAllMembers(boolean explicitUserSearchAction) {
		if (!showAllUsersCheckbox.getSelection() && !showAllGroupsCheckbox.getSelection()) {
			if (explicitUserSearchAction) {
				MessageDialog.openError(Display.getCurrent().getActiveShell(), "Invalid selection", "Select at least one of the checkboxes to show users and/or groups.");
			}
			return;
		}
		
		String search = allMembersSearchText.getText().toLowerCase().replace('*', '%');
		
		// only search when the user explicit requested the search OR when the length is greater than 2 and doesn't start with % OR the search list is based on the baseList
		if (explicitUserSearchAction ||	(search.length() > 2 && !search.startsWith("%"))) {
			
			allMembersTableViewer.getTable().clearAll();
			
			if (!search.endsWith("%")) {
				search += "%";
			}
			
			ApplicationUtil.setBusyCursor();
			List<Member> members = new ArrayList<Member>();
			
			if (allMembersList != null){
				members = new ArrayList<Member>();
				search = search.replace("%", "");
				for (Member member : allMembersList) {
					// match display name or member name?
					if ((member.getDisplayName(false, true).toLowerCase().contains(search.toLowerCase()) || member.getName(false).toLowerCase().contains(search.toLowerCase()))
							// filter on groups or users?
							&& ((member.isGroup() && showAllGroupsCheckbox.getSelection()) || (!member.isGroup() && showAllUsersCheckbox.getSelection()))) {
						members.add(member);
					}
				}
			}
			
			allMembersTableViewer.setItemCount(members.size());
			allMembersTableViewer.setInput(members);
							
			if (members.size() == 0 && explicitUserSearchAction){
				MessageDialog.openInformation(Display.getCurrent().getActiveShell(), "No results", "No members found.");
			}
			
			allMembersTableViewer.refresh();
			
			ApplicationUtil.setDefaultCursor();
		}		
	}
	
	/**
	 * Builds the query that is required to find members that match the selected filter criteria in the MemberFilterComposite.
	 * This method is only used when the option "Include nested group memberships" is enabled. Otherwise, we just use the direct members 
	 * from the ACE's found on the selected file.
	 * 
	 * The thing with group memberships is that we may find ACE's that don't belong to the users selected by the filter criteria! The dialog
	 * to select a member for tracing will only show members that match the filter criteria. So, instead of using the direct members from the 
	 * (filtered) ACE's on the selected file, we use the <b>groups</b> that belong to the members on the ACE's (or the members that match the
	 * filter criteria).
	 * 
	 * @return PreparedStatement with a SELECT member query
	 */
	private PreparedStatement getMembersThatMatchFilterPS(int limit) throws DBException {
		StringBuilder additionalMemberFilterBuilder = new StringBuilder("FILTER_MEMBERSHIP.GROUP_ID IN (");
		Object[] additionalMemberFilterValues = new Object[selectedFile.getAclItems().size()];
		Iterator<AclItem> iter = selectedFile.getAclItems().iterator();
		int index = 0;
		while (iter.hasNext()) {
			additionalMemberFilterBuilder.append("?");
			additionalMemberFilterValues[index] = iter.next().getMemberId();
			if (iter.hasNext()) {
				additionalMemberFilterBuilder.append(",");
			}
			index++;
		}
		
		additionalMemberFilterBuilder.append(")");
		String additionalMemberFilter = additionalMemberFilterBuilder.toString();
		PreparedStatement membersThatMatchFilterPS = FilterHelper.getInstance().buildFilterQuery(ObjectType.MEMBER, false, false, limit, "NAME_LOWERCASE", additionalMemberFilter, additionalMemberFilterValues);
	
		return membersThatMatchFilterPS;
	}
	
	@Focus
	public void setFocus() {
		membersOnDirectoryTableViewer.getTable().setFocus();
	}
	
	/**
	 * Clears the TableViewer with all the members
	 */
	public void clearAllMembers() {
		if (allMembersList != null) {
			allMembersList.clear();
			allMembersTableViewer.setItemCount(0);
			allMembersTableViewer.setInput(allMembersList);
			allMembersTableViewer.refresh();
		}
	}
}

class LazyTableContentProvider implements ILazyContentProvider {
	private java.util.List<Member> searchResultList;
	private TableViewer viewer;
	
	@Override
	public void dispose() {}

	@SuppressWarnings("unchecked")
	@Override
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		this.searchResultList = (java.util.List<Member>) newInput;
		this.viewer = (TableViewer)viewer;
	}

	@Override
	public void updateElement(int index) {
		viewer.replace(searchResultList.get(index), index);				
	}
}

class MemberLabelProvider extends TableLabelProvider {
	@Override
	public Image getColumnImage(Object element, int columnIndex) {
		if (columnIndex == 0) {
			Member member = (Member) element;
			if (member.isGroup()) {
				return ImageFactory.getInstance().getImage(AppImage.GROUP_16X16);
			} else {
				return ImageFactory.getInstance().getImage(AppImage.USER_16X16);
			}
		}
		return null;
	}

	@Override
	public String getColumnText(Object element, int columnIndex) {
		Member member = (Member) element;
		switch (columnIndex) {
			case 0: return member.getName(true);
			default: return "";
		}
	}
}