package com.perdemia.permissionanalyzer.application.parts;

import java.util.List;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;

import org.eclipse.core.commands.ParameterizedCommand;
import org.eclipse.e4.core.commands.ECommandService;
import org.eclipse.e4.core.commands.EHandlerService;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.ICheckStateProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.E4LifeCycle;
import com.perdemia.permissionanalyzer.application.jface.dialogs.AlertDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.AlertDialog.AlertType;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SearchLdapDialog;
import com.perdemia.permissionanalyzer.application.jface.labelproviders.ColoredTableLabelProvider;
import com.perdemia.permissionanalyzer.application.model.LdapConnectionInfo;
import com.perdemia.permissionanalyzer.application.model.LdapOUInfo;
import com.perdemia.permissionanalyzer.application.model.LdapOUInfo.SearchObjectType;
import com.perdemia.permissionanalyzer.application.model.LdapOUInfo.SearchScope;
import com.perdemia.permissionanalyzer.application.preferences.PreferenceHelper;
import com.perdemia.permissionanalyzer.application.preferences.PreferenceListener;
import com.perdemia.permissionanalyzer.application.util.ApplicationUtil;
import com.perdemia.permissionanalyzer.application.util.ColorFactory;
import com.perdemia.permissionanalyzer.application.util.ColorFactory.AppColor;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.scanner.ldap.LdapConnector;
import com.perdemia.permissionanalyzer.scanner.ldap.LdapFactory;

public class ScanLdapPart {
	public static final String ID = "com.perdemia.permissionanalyzer.application.parts.scanldap";
	private PreferenceListener preferenceListener;
	private static Logger logger = LoggerFactory.getLogger(ScanLdapPart.class.getName());
			
	/** LDAP OU fields */
	private ComboViewer ldapConnectionCombo;
	private Text ldapOUText;
	private ComboViewer searchScopeCombo;
	private ComboViewer searchObjectTypeCombo;
	private LdapOUInfo selectedOU;
	private List<LdapOUInfo> ldapOUList;
	private CheckboxTableViewer ldapOUTableViewer;
	private Button updateLdapOUButton;

	/**
	 * Constructor, initializes the preference listener to refresh the drop down with LDAP connections in case the preferences have changed.
	 */
	public ScanLdapPart() {
		preferenceListener = new PreferenceListener() {
			@Override
			public void preferencesChanged() {
				Display.getDefault().syncExec(new Runnable(){
					public void run() {
						ldapConnectionCombo.setInput(PreferenceHelper.getInstance().getLdapConnectionsList());
						ldapConnectionCombo.getCombo().getParent().layout();
						if (ldapConnectionCombo.getCombo().getItemCount() > 0) {
							ldapConnectionCombo.setSelection(new StructuredSelection(ldapConnectionCombo.getElementAt(0)), true);
						}
					}
				});
			}
		};
		PreferenceHelper.getInstance().addListener(preferenceListener);
		ldapOUList = PreferenceHelper.getInstance().getLdapOUList();
	}	

	/**
	 * Stores the ldap container list in the preferences
	 */
	private void scanConfigChanged() {
		PreferenceHelper prefHelper = PreferenceHelper.getInstance();
		prefHelper.setLdapOUList(ldapOUList);
		prefHelper.storePreferences(false);
	}

	/**
	 * Creates a composite on the given <code>parent</code> composite to display the list of LDAP OU's to scan
	 * 
	 * @param parent
	 *            parent composite
	 */
	@PostConstruct
	public void createComposite(Composite parent, final ECommandService commandService, final EHandlerService handlerService) {
		//Composite ldapOUComposite = new Composite(parent, SWT.NONE);
		parent.setLayoutData(new GridData(GridData.FILL_BOTH));
		parent.setLayout(new GridLayout(1, false));
		parent.setBackgroundMode(SWT.INHERIT_DEFAULT);
		parent.setBackground(ColorFactory.getInstance().getColor(AppColor.PART_BACKGROUND));
	
		/*
		TitledSeparator ldapOUSeparator = new TitledSeparator(parent, SWT.NONE);
		ldapOUSeparator.setText("Users and groups");
		ldapOUSeparator.setImage(ImageFactory.getInstance().getImage(AppImage.GROUP_24X24));
		ldapOUSeparator.setSize(50, 800);
		ldapOUSeparator.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		*/
		
		Group detailsGroup = new Group(parent, SWT.NONE);
		GridData gd_detailsGroup = new GridData(GridData.FILL_HORIZONTAL);
		//gd_detailsGroup.horizontalIndent = 30;
		detailsGroup.setLayoutData(gd_detailsGroup);
		GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.horizontalSpacing = 15;
		detailsGroup.setLayout(gridLayout);

		// LDAP connection drop down
		Label directoryPathLabel = new Label(detailsGroup, SWT.NONE);
		directoryPathLabel.setText("LDAP Connection:");
		
		Composite ldapConnectionComposite = new Composite(detailsGroup, SWT.NONE);
		ldapConnectionComposite.setLayout(new GridLayout(2, false));
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalIndent = -5;
		ldapConnectionComposite.setLayoutData(gridData);
		
		ldapConnectionCombo = new ComboViewer(ldapConnectionComposite, SWT.BORDER | SWT.READ_ONLY);
		ldapConnectionCombo.getCombo().setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		ldapConnectionCombo.setContentProvider(ArrayContentProvider.getInstance());
		ldapConnectionCombo.setLabelProvider(new LabelProvider() {
			@Override
			public String getText(Object element) {
				return ((LdapConnectionInfo) element).getConnectionName();
			}
		});
		
		gridData = new GridData();
		gridData.minimumWidth = 150;
		ldapConnectionCombo.getCombo().setLayoutData(gridData);
		ldapConnectionCombo.setInput(PreferenceHelper.getInstance().getLdapConnectionsList());
		if (ldapConnectionCombo.getCombo().getItemCount() > 0) {
			ldapConnectionCombo.setSelection(new StructuredSelection(ldapConnectionCombo.getElementAt(0)), true);
		}
		
		Button openPrefsButton = new Button(ldapConnectionComposite, SWT.NONE);
		openPrefsButton.setLayoutData(new GridData());
		openPrefsButton.setImage(ImageFactory.getInstance().getImage(AppImage.CONNECTION_16X16));
		openPrefsButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				// trigger the open preference command
				ParameterizedCommand applyFilterCommand = commandService.createCommand("com.perdemia.permissionanalyzer.application.commands.openpreferences", null);
				handlerService.executeHandler(applyFilterCommand);
			}
		});

		// OU path
		Label ouPathLabel = new Label(detailsGroup, SWT.NONE);
		ouPathLabel.setText("Ldap container (base OU):");
		ouPathLabel.setToolTipText("OU to start scanning from. This value is used as 'base DN'");
		Composite browseLdapOUComposite = new Composite(detailsGroup, SWT.NONE);
		browseLdapOUComposite.setLayout(new GridLayout(2, false));
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalIndent = -5;
		browseLdapOUComposite.setLayoutData(gridData);
		ldapOUText = new Text(browseLdapOUComposite, SWT.BORDER);
		ldapOUText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		ldapOUText.setToolTipText("OU to start scanning from. This value is used as 'base DN'");
		Button browseLdapOUButton = new Button(browseLdapOUComposite, SWT.NONE);
		browseLdapOUButton.setLayoutData(new GridData());
		browseLdapOUButton.setImage(ImageFactory.getInstance().getImage(AppImage.LDAP_OU_16X16));
		browseLdapOUButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (ldapConnectionCombo.getCombo().getSelectionIndex() == -1) {
					MessageDialog.openError(null, "Select LDAP connection", "Please first select a LDAP connection from the drop down above.");
					;
					return;
				}
				LdapConnectionInfo ldapConnection = PreferenceHelper.getInstance().getLdapConnectionsList().get(ldapConnectionCombo.getCombo().getSelectionIndex());
				SearchLdapDialog searchDialog = new SearchLdapDialog(Display.getCurrent().getActiveShell(), null, true, ldapConnection, false, false);
				if (searchDialog.open() == IDialogConstants.OK_ID && searchDialog.getSelectedLdapContainer() != null) {
					ldapOUText.setText(searchDialog.getSelectedLdapContainer().getDn());
				}
			}
		});

		// Search scope drop down
		Label searchScopeLabel = new Label(detailsGroup, SWT.NONE);
		searchScopeLabel.setText("Scope:");
		searchScopeCombo = new ComboViewer(detailsGroup, SWT.BORDER | SWT.READ_ONLY);
		searchScopeCombo.setContentProvider(ArrayContentProvider.getInstance());
		searchScopeCombo.setLabelProvider(new LabelProvider() {
			@Override
			public String getText(Object element) {
				return searchScopeToString((SearchScope) element);
			}
		});
		searchScopeCombo.getCombo().setLayoutData(new GridData());
		
		// NOTE:
		// ONLY_SUBS gives an LDAP error from Active Directory:
		// LDAPSearchException(resultCode=80 (other), numEntries=0, numReferences=0, errorMessage='00000057: LdapErr: DSID-0C090797, comment: Error processing control, data 0, v2580
		// So, exclude the ONLY_SUBS scope
		LdapOUInfo.SearchScope[] scopes = new LdapOUInfo.SearchScope[3];
		scopes[0] = LdapOUInfo.SearchScope.CONTAINER_AND_SUBS;
		scopes[1] = LdapOUInfo.SearchScope.ONE_LEVEL_SUBS_NOT_CONTAINER_ITSELF;
		scopes[2] = LdapOUInfo.SearchScope.ONLY_CONTAINER;
		searchScopeCombo.setInput(scopes);
		searchScopeCombo.setSelection(new StructuredSelection(searchScopeCombo.getElementAt(0)), true);

		// Search objects drop down
		Label searchObjectsLabel = new Label(detailsGroup, SWT.NONE);
		searchObjectsLabel.setText("Object types:");
		searchObjectTypeCombo = new ComboViewer(detailsGroup, SWT.BORDER | SWT.READ_ONLY);
		searchObjectTypeCombo.setContentProvider(ArrayContentProvider.getInstance());
		searchObjectTypeCombo.setLabelProvider(new LabelProvider() {
			@Override
			public String getText(Object element) {
				return searchObjectTypeToString((SearchObjectType) element);
			}
		});
		searchObjectTypeCombo.getCombo().setLayoutData(new GridData());
		searchObjectTypeCombo.setInput(LdapOUInfo.SearchObjectType.values());
		searchObjectTypeCombo.setSelection(new StructuredSelection(searchObjectTypeCombo.getElementAt(0)), true);

		Composite detailsButtonBarComposite = new Composite(detailsGroup, SWT.NONE);
		detailsButtonBarComposite.setLayout(new GridLayout(2, false));
		GridData gd_composite = new GridData(GridData.FILL_HORIZONTAL);
		gd_composite.horizontalSpan = 2;
		detailsButtonBarComposite.setLayoutData(gd_composite);

		Button addButton = new Button(detailsButtonBarComposite, SWT.NONE);
		addButton.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, false, 1, 1));
		addButton.setText("Add");
		addButton.setImage(ImageFactory.getInstance().getImage(AppImage.ADD_16X16));
		addButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (ldapConnectionCombo.getCombo().getItemCount() == 0) {
					MessageDialog.openError(null, "No ldap connection", "There is no LDAP connection configured. Please configure at least one LDAP connection in the preferences.");
					return;
				}
				if (ldapConnectionCombo.getCombo().getSelectionIndex() == -1) {
					MessageDialog.openError(null, "No ldap connection selected", "Please select a LDAP connection.");
					return;
				}
				if (ldapOUText.getText().length() == 0) {
					MessageDialog.openError(null, "No LDAP container selected. Please select a LDAP container which will be used as 'base DN' for the scan.", "Please select a LDAP container.");
					return;
				}

				LdapOUInfo ouInfo = new LdapOUInfo();
				ouInfo.setBaseOU(ldapOUText.getText());
				ouInfo.setConnection(PreferenceHelper.getInstance().getLdapConnectionsList().get(ldapConnectionCombo.getCombo().getSelectionIndex()));
				ouInfo.setSearchObjectType((SearchObjectType)((IStructuredSelection)searchObjectTypeCombo.getSelection()).getFirstElement());
				ouInfo.setSearchScope((SearchScope)((IStructuredSelection)searchScopeCombo.getSelection()).getFirstElement());
				ouInfo.setEnabled(true);
				
				// determine the current max id
				int maxId = -1;
				for (LdapOUInfo ou : ldapOUList) {
					maxId = Math.max(maxId, ou.getLdapOUId());
				}
				ouInfo.setLdapOUId(maxId + 1);

				ldapOUList.add(ouInfo);
				scanConfigChanged();

				// reset UI fields
				clearLdapOUFields();
			}
		});

		updateLdapOUButton = new Button(detailsButtonBarComposite, SWT.NONE);
		updateLdapOUButton.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		updateLdapOUButton.setText("Update");
		updateLdapOUButton.setEnabled(false);
		updateLdapOUButton.setImage(ImageFactory.getInstance().getImage(AppImage.APPLY_16X16));
		updateLdapOUButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectedOU.setBaseOU(ldapOUText.getText());
				selectedOU.setConnection(PreferenceHelper.getInstance().getLdapConnectionsList().get(ldapConnectionCombo.getCombo().getSelectionIndex()));
				selectedOU.setSearchObjectType((SearchObjectType)((IStructuredSelection)searchObjectTypeCombo.getSelection()).getFirstElement());
				selectedOU.setSearchScope((SearchScope)((IStructuredSelection)searchScopeCombo.getSelection()).getFirstElement());
				scanConfigChanged();

				// reset UI fields
				clearLdapOUFields();
			}
		});

		ldapOUTableViewer = CheckboxTableViewer.newCheckList(parent, SWT.BORDER | SWT.FULL_SELECTION);
		Table table = ldapOUTableViewer.getTable();
		table.setLinesVisible(true);
		table.setHeaderVisible(true);
		GridData gd_table = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		//gd_table.horizontalIndent = 30;
		table.setLayoutData(gd_table);

		TableViewerColumn connectionColumn = new TableViewerColumn(ldapOUTableViewer, SWT.NONE);
		connectionColumn.getColumn().setWidth(200);
		connectionColumn.getColumn().setText("LDAP connection");

		TableViewerColumn containerColumn = new TableViewerColumn(ldapOUTableViewer, SWT.NONE);
		containerColumn.getColumn().setWidth(300);
		containerColumn.getColumn().setText("LDAP base container");

		TableViewerColumn searchScopeColumn = new TableViewerColumn(ldapOUTableViewer, SWT.NONE);
		searchScopeColumn.getColumn().setWidth(200);
		searchScopeColumn.getColumn().setText("Search scope");

		TableViewerColumn objectTypesColumn = new TableViewerColumn(ldapOUTableViewer, SWT.NONE);
		objectTypesColumn.getColumn().setWidth(150);
		objectTypesColumn.getColumn().setText("Object types");

		ldapOUTableViewer.setContentProvider(ArrayContentProvider.getInstance());
		ldapOUTableViewer.setLabelProvider(new ColoredTableLabelProvider() {
			@Override
			public String getColumnText(Object element, int columnIndex) {
				LdapOUInfo info = (LdapOUInfo) element;
				switch (columnIndex) {
				case 0:
					return info.getConnection().getConnectionName();
				case 1:
					return info.getBaseOU();
				case 2:
					return searchScopeToString(info.getSearchScope());
				case 3:
					return searchObjectTypeToString(info.getSearchObjectType());
				default:
					return "";
				}
			}

			@Override
			public Image getColumnImage(Object element, int columnIndex) {
				if (columnIndex == 0) {
					if (((LdapOUInfo)element).isEnabled()) {
						return ImageFactory.getInstance().getImage(AppImage.CONNECTION_16X16);
					} else {
						return ImageFactory.getInstance().getImage(AppImage.CONNECTION_DISABLED_16X16);
					}
				} else {
					return null;
				}
			}
			
			@Override
			public Color getForeground(Object element) {
				if (((LdapOUInfo)element).isEnabled()) {
					return null;
				} else {
					return ColorFactory.getInstance().getColor(AppColor.GREY);
				}
			}
		});

		ldapOUTableViewer.setCheckStateProvider(new ICheckStateProvider() {
			@Override
			public boolean isChecked(Object element) {
				return ((LdapOUInfo) element).isEnabled();
			}

			@Override
			public boolean isGrayed(Object element) {
				return false;
			}
		});

		ldapOUTableViewer.setInput(ldapOUList);

		ldapOUTableViewer.addCheckStateListener(new ICheckStateListener() {
			@Override
			public void checkStateChanged(CheckStateChangedEvent event) {
				((LdapOUInfo) event.getElement()).setEnabled(event.getChecked());
				PreferenceHelper.getInstance().setLdapOUList(ldapOUList);
				PreferenceHelper.getInstance().storePreferences(false);
				ldapOUTableViewer.refresh();
			}
		});

		Composite buttonBarComposite = new Composite(parent, SWT.NONE);
		buttonBarComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		buttonBarComposite.setLayout(new GridLayout(3, false));

		final Button testLdapOUButton = new Button(buttonBarComposite, SWT.PUSH);
		testLdapOUButton.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, false, 1, 1));
		testLdapOUButton.setText("Test LDAP config");
		testLdapOUButton.setEnabled(false);
		testLdapOUButton.setImage(ImageFactory.getInstance().getImage(AppImage.TEST_16X16));
		testLdapOUButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				ApplicationUtil.setBusyCursor();

				selectedOU = ldapOUList.get(ldapOUTableViewer.getTable().getSelectionIndex());

				try {
					LdapConnector connector = LdapFactory.getLdapConnector();
					boolean success = LdapFactory.openConnection(selectedOU.getConnection(), connector);
					if (success) {
						connector.testOU(selectedOU);
						connector.closeConnection();
						MessageDialog.openInformation(null, "Connection OK", "The connection to the LDAP server and the container path have been tested succesfully!");
					}
				} catch (Exception ex) {
					MessageDialog.openError(null, "Failed", "Error while testing LDAP config: " + ex.getMessage());
					logger.error("Error while testing LDAP config: " + selectedOU.toString(), ex);
					
					// reset the (wrong) password so that the user can enter another one
					if (selectedOU.getConnection().isOnlyStorePasswordInMemory()) {
						selectedOU.getConnection().setPassword("");
					}
				}

				ApplicationUtil.setDefaultCursor();
			}
		});

		final Button deleteButton = new Button(buttonBarComposite, SWT.NONE);
		deleteButton.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		deleteButton.setText("Delete");
		deleteButton.setEnabled(false);
		deleteButton.setImage(ImageFactory.getInstance().getImage(AppImage.DELETE_16X16));
		deleteButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				ldapOUList.remove(ldapOUTableViewer.getTable().getSelectionIndex());
				scanConfigChanged();

				// clear UI fields
				clearLdapOUFields();
				deleteButton.setEnabled(false);
				testLdapOUButton.setEnabled(false);
			}
		});

		ldapOUTableViewer.addSelectionChangedListener(new ISelectionChangedListener() {
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				if (ldapOUList.size() > 0 && ldapOUTableViewer.getTable().getSelectionIndex() != -1) {
					deleteButton.setEnabled(true);
					testLdapOUButton.setEnabled(true);
					
					selectedOU = ldapOUList.get(ldapOUTableViewer.getTable().getSelectionIndex());
					updateLdapOUButton.setEnabled(true);

					// update UI fields
					ldapConnectionCombo.setSelection(new StructuredSelection(selectedOU.getConnection()), true);
					ldapOUText.setText(selectedOU.getBaseOU());
					searchScopeCombo.setSelection(new StructuredSelection(selectedOU.getSearchScope()), true);
					searchObjectTypeCombo.setSelection(new StructuredSelection(selectedOU.getSearchObjectType()), true);
				}
			}
		});
		
		// wait for the perspective to show
		if (!E4LifeCycle.isSilentMode) {
			Display.getCurrent().timerExec(1000, new Runnable() {
				public void run() {
					if (ldapConnectionCombo.getCombo().getItemCount() == 0) {
						AlertDialog dialog = new AlertDialog(null, "Configure a LDAP connection", "There are no LDAP connections configured. This is required to include group memberships in the permissions overview. Do you want to configure a LDAP connection in the preferences?", AppImage.CONNECTION_48X48, true, AlertType.CONFIGURE_LDAP_CONNECTION);
						int result = dialog.open();
						if (result == IDialogConstants.YES_ID) {
							// trigger the open preference command
							ParameterizedCommand applyFilterCommand = commandService.createCommand("com.perdemia.permissionanalyzer.application.commands.openpreferences", null);
							handlerService.executeHandler(applyFilterCommand);
						}						
					}
				}
			});
		}
	}

	private void clearLdapOUFields() {
		ldapConnectionCombo.setSelection(new StructuredSelection(ldapConnectionCombo.getElementAt(0)), true);
		ldapOUText.setText("");
		searchScopeCombo.setSelection(new StructuredSelection(searchScopeCombo.getElementAt(0)), true);
		searchObjectTypeCombo.setSelection(new StructuredSelection(searchObjectTypeCombo.getElementAt(0)), true);
		ldapOUTableViewer.refresh();
		selectedOU = null;
		updateLdapOUButton.setEnabled(false);
	}

	/**
	 * Returns the text representation of a {@link LdapOUInfo#SearchScope} object.
	 * 
	 * @param searchScope
	 * @return text representation for GUI
	 */
	private String searchScopeToString(SearchScope searchScope) {
		switch (searchScope) {
		case CONTAINER_AND_SUBS:
			return "Container and sub containers";
		case ONLY_SUBS:
			return "All the sub containers, but not the container itself";
		case ONE_LEVEL_SUBS_NOT_CONTAINER_ITSELF:
			return "One level of sub containers, but not the container itself";
		case ONLY_CONTAINER:
			return "Only the container itself, not the sub containers";
		default:
			return "";
		}
	}

	/**
	 * Returns the text representation of a {@link LdapOUInfo#SearchObjectType} object.
	 * 
	 * @param searchObjectType
	 * @return text representation for GUI
	 */
	private String searchObjectTypeToString(SearchObjectType searchObjectType) {
		switch (searchObjectType) {
		case USERS_GROUPS:
			return "Users and groups";
		case USERS:
			return "Only users";
		case GROUPS:
			return "Only groups";
		default:
			return "";
		}
	}
	
	public void resetListStatus() {
		for (TableItem item : ldapOUTableViewer.getTable().getItems()) {
			item.setBackground(ColorFactory.getInstance().getColor(AppColor.WHITE));
		}
	}
	
	public void ldapOUScanned(final LdapOUInfo ouInfo, final boolean success) {
		int index = ldapOUList.indexOf(ouInfo);
		if (index != -1) {
			if (success) {
				ldapOUTableViewer.getTable().getItem(index).setBackground(ColorFactory.getInstance().getColor(AppColor.SCANNED));
			} else {
				ldapOUTableViewer.getTable().getItem(index).setBackground(ColorFactory.getInstance().getColor(AppColor.ERROR));
			}
		}
	}
	
	@Focus
	public void setFocus() {
		ldapConnectionCombo.getCombo().setFocus();
	}

	@PreDestroy
	public void preDestroy() {
		PreferenceHelper.getInstance().removeListener(preferenceListener);
	}
}
