package com.perdemia.permissionanalyzer.application.parts;

import java.sql.PreparedStatement;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.inject.Inject;
import javax.inject.Named;

import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.menu.MToolBarElement;
import org.eclipse.e4.ui.model.application.ui.menu.impl.ToolControlImpl;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.e4.ui.workbench.modeling.ESelectionService;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.TreeViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perdemia.permissionanalyzer.application.handlers.toolbar.TraceHideInheritedToolControl;
import com.perdemia.permissionanalyzer.application.jface.contentproviders.TracePartContentProvider;
import com.perdemia.permissionanalyzer.application.jface.contentproviders.TracePartContentProviderConfig;
import com.perdemia.permissionanalyzer.application.jface.dialogs.MemberInfoDialog;
import com.perdemia.permissionanalyzer.application.jface.dialogs.SyncMessageDialog;
import com.perdemia.permissionanalyzer.application.jface.labelproviders.SpecialPermissionLabelProvider;
import com.perdemia.permissionanalyzer.application.swt.ExplainLabel;
import com.perdemia.permissionanalyzer.application.swt.ExplainLabel.ExplainType;
import com.perdemia.permissionanalyzer.application.util.ColorFactory;
import com.perdemia.permissionanalyzer.application.util.ColorFactory.AppColor;
import com.perdemia.permissionanalyzer.application.util.Constants;
import com.perdemia.permissionanalyzer.application.util.FilterHelper;
import com.perdemia.permissionanalyzer.application.util.ImageFactory;
import com.perdemia.permissionanalyzer.application.util.ImageFactory.AppImage;
import com.perdemia.permissionanalyzer.application.util.PermissionUtil;
import com.perdemia.permissionanalyzer.data.AclItem;
import com.perdemia.permissionanalyzer.data.DBException;
import com.perdemia.permissionanalyzer.data.FileInfo;
import com.perdemia.permissionanalyzer.data.Member;
import com.perdemia.permissionanalyzer.data.PermissionLookup.SpecialPermission;

public class TracePart {
	private static final Logger logger = LoggerFactory.getLogger(TracePart.class.getName());
	public static final String ID = "com.perdemia.permissionanalyzer.application.parts.trace";
	
	/** UI controls */
	private TreeViewer viewer;
	private TracePartContentProviderConfig fileTraceModel = new TracePartContentProviderConfig();
	private Composite leftTopComposite;
	private Button memberInfoButton;
	private CLabel selectedMemberLabel;
	
	/** Filter data */
	private static Member selectedMember;
	private FileInfo selectedFile;
	
	@Inject ESelectionService selectionService;
	@Inject EPartService partService;
	
	/**
	 * Receives selection events from the main FileInfo tree and updates the input for the {@link TracePartContentProvider}
	 */
	@Inject
	public void fileSelected(@Optional @Named(IServiceConstants.ACTIVE_SELECTION) IStructuredSelection selectedItem) {
		// apparently this line is necessary because when the users switches between the AclPart and TracePart this method is called
		// with an empty selectedPath (???)
		selectedItem = (IStructuredSelection) selectionService.getSelection(FilePart.ID);
		boolean fileChanged = false;;
		
		if (selectedItem != null && !selectedItem.isEmpty()) {
			fileChanged = selectedFile != selectedItem.getFirstElement();
			selectedFile = (FileInfo) selectedItem.getFirstElement();
		} else {
			selectedFile = null;
		}
		
		if (selectedFile != null && selectedFile.getAclItems().size() > 0) {
			// fill the model with the selected file and its parent folders
			// the parent items are used to build to "Inherited from" folder path
			List<FileInfo> parentFiles = new ArrayList<FileInfo>();
			FileInfo parent = selectedFile;
			while (parent.getParent() != null) {
				parent = parent.getParent();
				parentFiles.add(parent);
				
				// in case this selection comes from the TableViewer (where parents are not always loaded because of the flat structure) it might be so that 
				// the ACL items of this parent have not been loaded yet
				// load the related AclItems for this FileInfo
				if (parent.getAclItems() == null) {
					try {
						List<AclItem> aclItems;
						PreparedStatement ps = FilterHelper.getInstance().buildFilterQuery(FilterHelper.ObjectType.ACL_ITEM, false, false, 0, "ACLITEM.MEMBER_ID", "ACLITEM.FILEINFO_ID = ?", parent.getId());
						//if (ps != null) {
							aclItems = AclItem.getByPreparedStatement(ps);
						//} else {
							// no filter applied, just retrieve all the ACE of this file
						//	aclItems = AclItem.where("FILEINFO_ID = ?", new Object[]{parent.getId()});
						//}
						
						parent.setAclItems(aclItems);
					} catch (DBException ex) {
						SyncMessageDialog.showMessage(MessageDialog.ERROR, "Error", "Error while retrieving ACE's for the file/folder " + parent.getName() + ": " + ex.getMessage());
					}
				}
				// process next parent
			}
			
			// determine if the Hide Inherited Permissions check button has been checked in the ToolBar
			boolean hideInheritedPermissions = false;
			MPart tracePart = partService.findPart("com.perdemia.permissionanalyzer.application.parts.trace");
			
			if (tracePart != null) {
				for (MToolBarElement element : tracePart.getToolbar().getChildren()) {
					if (element.getElementId().equals("com.perdemia.permissionanalyzer.application.parts.trace.toolbar.hideInheritedPermissions")) {
						if (((ToolControlImpl)element).getObject() != null) {
							TraceHideInheritedToolControl toolItem = (TraceHideInheritedToolControl) ((ToolControlImpl)element).getObject();
							hideInheritedPermissions = toolItem.isChecked();
						} else {
							hideInheritedPermissions = true;
						}					
					}
				}
			}
			
			fileChanged = fileChanged || fileTraceModel.isHideInheritedPermissions() != hideInheritedPermissions;
			fileTraceModel.setHideInheritedPermissions(hideInheritedPermissions);
			fileTraceModel.setSelectedFile(selectedFile);
			fileTraceModel.setParentFiles(parentFiles);
			
		} else {
			fileTraceModel.setSelectedFile(null);
			fileChanged = true;
		}
		
		if (fileChanged) {
			refreshViewer();
		}
	}
	
	@PostConstruct
	public void createComposite(Composite parent) {
		// View layout
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.verticalSpacing = 0;
		gridLayout.horizontalSpacing = 0;
		parent.setLayout(gridLayout);
		
		new ExplainLabel(parent, "This tab tab displays all permissions that apply to a specific member as well as the provenance of those permissions, e.g. through which (nested) group membership or superior directory the permissions were granted. Put a member in this tab using the context menu on the members in one of the other tabs.", ExplainType.TRACE_TAB);
		
		// Left top component
		GridData columnGridData = new GridData();
		columnGridData.verticalAlignment = SWT.BOTTOM;
		leftTopComposite = new Composite(parent, SWT.NONE);
		leftTopComposite.setLayoutData(columnGridData);
		leftTopComposite.setLayout(new GridLayout(4, false));
		
		// Member label
		Label memberLabel = new Label(leftTopComposite, SWT.NONE);
		memberLabel.setText("Show permissions for member: ");
		memberLabel.setData(Constants.CSS_CLASS_NAME_KEY, "h2");
		selectedMemberLabel = new CLabel(leftTopComposite, SWT.NONE);
		selectedMemberLabel.setLayoutData(new GridData());
		selectedMemberLabel.setText("No member selected");
				
		// Button for member info
		memberInfoButton = new Button(leftTopComposite, SWT.PUSH);
		memberInfoButton.setImage(ImageFactory.getInstance().getImage(AppImage.USER_INFO_16X16));
		memberInfoButton.setToolTipText("Show member info");
		memberInfoButton.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				MemberInfoDialog dialog = new MemberInfoDialog(null, selectedMember);
				dialog.open();
			}
		});
		memberInfoButton.setEnabled(false);
		
		// Tree with details	
		viewer = new TreeViewer(parent, SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER | SWT.FULL_SELECTION);
		viewer.getTree().setLinesVisible(true);
		viewer.getTree().setHeaderVisible(true);
		viewer.setUseHashlookup(true);
				
		//layout data for the tree
		GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
		viewer.getTree().setLayoutData(gridData);
		
		// Path column
		TreeViewerColumn column = new TreeViewerColumn(viewer, SWT.NONE);
		column.getColumn().setText("Path");
		column.getColumn().setWidth(300);
		column.getColumn().setResizable(true);
		column.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AclItem aclItem = (AclItem) element;
				return aclItem.getFile().getName();
			}
			
			@Override
			public Image getImage(Object element) {
				AclItem aclItem = (AclItem) element;
				return ImageFactory.getInstance().getImage(PermissionUtil.getFileImage(aclItem, aclItem.getFile()));
			}
			
			@Override
		    public Color getBackground(Object element) {
		        if (((AclItem) element).getMember(false) == null){
		        	return ColorFactory.getInstance().getColor(AppColor.DISABLED_ROW);
		        } else {
		        	return null;
		        }
		    }
			
			@Override
			public Color getForeground(Object element) {
				if (element instanceof AclItem) {
					AclItem aclItem = (AclItem) element;
					if (aclItem.isFlagInheritedAce()) {
						return ColorFactory.getInstance().getColor(AppColor.GREY);
					} else {
						return super.getForeground(element);
					}
				} else {
					return super.getForeground(element);
				}
			}
		});
		
		// Permission column
		column = new TreeViewerColumn(viewer, SWT.NONE);
		column.getColumn().setText("Permission");
		column.getColumn().setWidth(200);
		column.getColumn().setResizable(true);
		column.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AclItem aclItem = (AclItem) element;
				return PermissionUtil.permissionsToString(aclItem, true);
			}
						
			@Override
		    public Color getBackground(Object element) {
		        if (((AclItem) element).getMember(false) == null){
		        	return ColorFactory.getInstance().getColor(AppColor.DISABLED_ROW);
		        } else {
		        	return null;
		        }
		    }
			
			@Override
			public Color getForeground(Object element) {
				if (element instanceof AclItem) {
					AclItem aclItem = (AclItem) element;
					if (aclItem.isFlagInheritedAce()) {
						return ColorFactory.getInstance().getColor(AppColor.GREY);
					} else {
						return super.getForeground(element);
					}
				} else {
					return super.getForeground(element);
				}
			}
		});
		
		// Special permission columns
		for (final Integer permission : PermissionUtil.getCustomPermissionOrderList()) {
			//skip SYNCHRONIZE
			if (permission.intValue() != SpecialPermission.SYNCHRONIZE.ordinal()) {
				column = new TreeViewerColumn(viewer, SWT.NONE);
				column.getColumn().setWidth(30);
				column.getColumn().setResizable(false);
				column.getColumn().setToolTipText(PermissionUtil.specialPermissionToString(permission, false, null));
				column.getColumn().setImage(PermissionUtil.getSpecialPermissionImage(permission, false, true));
				column.setLabelProvider(new SpecialPermissionLabelProvider(permission, false, true));
			}
		}
		
		// Via group column
		column = new TreeViewerColumn(viewer, SWT.NONE);
		column.getColumn().setText("Via group");
		column.getColumn().setResizable(true);
		column.getColumn().setWidth(250);
		column.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AclItem aclItem = (AclItem) element;
				if (aclItem.getMember(false) != null) {
					if (aclItem.getViaMember() != null) {
						return aclItem.getViaMember().getDisplayName(true, false);
					} else {
						return "(directly)";
					}
				} else {
					return "";
				}
			}
			@Override
		    public Color getBackground(Object element) {
		        if (((AclItem) element).getMember(false) == null){
		        	return ColorFactory.getInstance().getColor(AppColor.DISABLED_ROW);
		        } else {
		        	return null;
		        }
		    }
			
			@Override
			public Color getForeground(Object element) {
				if (element instanceof AclItem) {
					AclItem aclItem = (AclItem) element;
					if (aclItem.isFlagInheritedAce()) {
						return ColorFactory.getInstance().getColor(AppColor.GREY);
					} else {
						return super.getForeground(element);
					}
				} else {
					return super.getForeground(element);
				}
			}
		});
		
		// Inheritance flags
		column = new TreeViewerColumn(viewer, SWT.NONE);
		column.getColumn().setWidth(300);
		column.getColumn().setText("Inheritance flags");
		column.getColumn().setResizable(true);
		column.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AclItem aclItem = (AclItem) element;
				if (aclItem.getMember(false) != null) {
					return PermissionUtil.aclEntryFlagsToString(aclItem, fileTraceModel.getSelectedFile().isDirectory());
				} else {
					return null;
				}
			}
			
			@Override
		    public Color getBackground(Object element) {																																																																																									
		        if (((AclItem) element).getMember(false) == null){
		        	return ColorFactory.getInstance().getColor(AppColor.DISABLED_ROW);
		        } else {
		        	return null;
		        }
		    }
			
			@Override
			public Color getForeground(Object element) {
				if (element instanceof AclItem) {
					AclItem aclItem = (AclItem) element;
					if (aclItem.isFlagInheritedAce()) {
						return ColorFactory.getInstance().getColor(AppColor.GREY);
					} else {
						return super.getForeground(element);
					}
				} else {
					return super.getForeground(element);
				}
			}
		});
		
		viewer.setContentProvider(new TracePartContentProvider());
		
		//enable tooltips for this tree viewer
		ColumnViewerToolTipSupport.enableFor(viewer);
		
		// select the first item in the tree so that the TracePart get refreshed through the SelectionService
		MPart part = partService.findPart(FilePart.ID);
		FilePart fileTreePart = (FilePart) part.getObject();
		selectionService.setSelection(fileTreePart.getTreeViewer().getSelection());
		
		applySelectedMember(selectedMember);
		// call the listener with the current selection
		//IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
		//mainFileTreelistener.selectionChanged(null, ((FilePart)page.findView(FilePart.ID)).getTreeViewer().getSelection());
				
		//TODO use postSelectionListener for performance?
		//getSite().getWorkbenchWindow().getSelectionService().addPostSelectionListener(mainFileTreelistener);
		
		// context menu for the Tree
		MenuManager menuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			@Override
			public void menuAboutToShow(IMenuManager manager) {
				final IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
				if (selection != null && !selection.isEmpty()) {					
					// Open file in explorer action
					if (((AclItem) selection.getFirstElement()).getFileId() != null) {
						Action action = new Action() {
							public void run() {
								FileInfo file = ((AclItem) selection.getFirstElement()).getFile(true);
								Program.launch(file.determineFullPath());
							}
						};
						action.setText("Open file in Explorer");
						action.setImageDescriptor(ImageFactory.getInstance().getDescriptor(AppImage.FOLDER_EXPLORER_16X16));
						manager.add(action);
					}
				}
			}
		});
		
		Menu menu = menuMgr.createContextMenu(viewer.getTree());
		viewer.getTree().setMenu(menu);
	}	
	
	public void applySelectedMember(Member member) {
		selectedMember = member;
		
		// view initialized?
		if (selectedMemberLabel != null && !selectedMemberLabel.isDisposed()) {
			if (member == null) {
				selectedMemberLabel.setText("<Select a user or group in one of the other tabs>");
				selectedMemberLabel.setImage(null);
				memberInfoButton.setEnabled(false);
			} else {
				selectedMemberLabel.setText(member.getDisplayName(true, false));
				if (member.isGroup()) {
					selectedMemberLabel.setImage(ImageFactory.getInstance().getImage(AppImage.GROUP_20X20));
				} else {
					selectedMemberLabel.setImage(ImageFactory.getInstance().getImage(AppImage.USER_20X20));
				}
				memberInfoButton.setEnabled(member.getId() > 0);
			}
			
			// save the selected member in the model of the Viewer input and refresh the viewer
			fileTraceModel.setSelectedMember(member);
			refreshViewer();
		}
	}
	
	private void refreshViewer() {
		if (viewer != null && !viewer.getTree().isDisposed()) {
			viewer.setInput(fileTraceModel);
			viewer.refresh();
			viewer.expandAll();
			viewer.getTree().getParent().layout(true);
		}
	}
	
	public static void setSelectedMember(Member member) {
		selectedMember = member;
	}
	
	public static void clearSelectedMember(){
		selectedMember = null;
	}
	
	@Focus
	public void setFocus() {
		viewer.getControl().setFocus();
	}
}
