package com.perdemia.permissionanalyzer.application.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

import org.eclipse.core.runtime.Platform;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.widgets.Display;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ApplicationUtil {
	private static Logger logger = LoggerFactory.getLogger(ApplicationUtil.class.getName());
	private static boolean scanInProgress;	
	private static final Cursor  waitCursor = Display.getDefault().getSystemCursor(SWT.CURSOR_WAIT);
	private static final Cursor  arrowCursor = Display.getDefault().getSystemCursor(SWT.CURSOR_ARROW);
	
	public static void setBusyCursor(){
		Display.getDefault().syncExec(new Runnable(){
			@Override
			public void run() {
				// might be null during application startup
				if (Display.getCurrent().getActiveShell() != null) {
					Display.getCurrent().getActiveShell().setCursor(waitCursor);
				}
			}
		});
	}
	
	public static void setDefaultCursor() {
		Display.getDefault().syncExec(new Runnable(){
			@Override
			public void run() {
				// might be null during application startup
				if (Display.getCurrent().getActiveShell() != null) {
					Display.getCurrent().getActiveShell().setCursor(arrowCursor);
				}
			}			
		});
	}
	
	/**
	 * Returns the path to the workspace directory, normally <app_dir>\workspace
	 */
	public static String getWorkspacePath() {
		File workspaceFolder = new File(getApplicationPath() + "workspace");
		if (!workspaceFolder.exists()) {
			try {
				Files.createDirectories(Paths.get(workspaceFolder.getAbsolutePath()));
			} catch (IOException e) {
				logger.error("Could not create workspace directory", e);
			}
		}
		return workspaceFolder.getAbsolutePath() + File.separator;
	}
	
	/**
	 * Returns the application directory where Permision Analyzer.exe is located
	 * 
	 * @return path
	 */
	public static String getApplicationPath() {
		String appPath = new File(Platform.getInstallLocation().getURL().getFile()).getAbsolutePath();
		
		if (!appPath.endsWith("\\") && !appPath.endsWith("/")) {
			appPath += File.separator;
		}
		
		return  appPath;
	}
	
	public static String getFileContent(String filePath) throws IOException {
		URL url = new URL("platform:/plugin/Permission_Analyzer/" + filePath);
	    InputStream inputStream = url.openConnection().getInputStream();
	    BufferedReader in = new BufferedReader(new InputStreamReader(inputStream));
	    String inputLine;
	    StringBuilder result = new StringBuilder();
	 
	    while ((inputLine = in.readLine()) != null) {
	    	result.append(inputLine + "\r\n");
	    }
	 
	    in.close();
	    
	    return result.toString();
	}

	public static boolean isScanInProgress() {
		return scanInProgress;
	}

	public static void setScanInProgress(boolean scanInProgress) {
		ApplicationUtil.scanInProgress = scanInProgress;
	}
}
