package com.perdemia.permissionanalyzer.application.util;

import org.eclipse.jface.resource.FontRegistry;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.widgets.Display;

public class FontFactory {
	/** The singleton instance. */
	private static FontFactory instance;
	
	/** The font registry. */
	private FontRegistry fontRegistry;
	
	
	/** The enum that defines all the fonts. */
	public static enum AppFont {
		FILTER_TITLE, FILTER_SUBTITLE
	}
	
	/**
	 * Disables public constructor (Singleton pattern).
	 */
	private FontFactory(){
		fontRegistry = new FontRegistry(Display.getCurrent());
	    
	    fontRegistry.put(AppFont.FILTER_TITLE.name(), new FontData[]{new FontData("Arial", 11, SWT.BOLD)} );
	    fontRegistry.put(AppFont.FILTER_SUBTITLE.name(), new FontData[]{new FontData("Arial", 9, SWT.BOLD)} );
	};
	
	/**
	 * Controls the access to this class (Singleton pattern)
	 * 
	 * @return Instance of this class. If it doesn't exist, it will be created.
	 */
	public static FontFactory getInstance(){
		if (instance == null) {
			instance = new FontFactory();
		}
		return instance;
	}

	/**
	 * Gets a font from the registry based on the given <code>font</code> enum.
	 * 
	 * @param font enum value
	 * @return Font instance
	 */
	public Font getFont(AppFont font) {
		return fontRegistry.get(font.name());
	}
}
