package com.perdemia.permissionanalyzer.data;

import java.util.List;
import java.util.Map;

import com.perdemia.permissionanalyzer.application.util.PermissionUtil;

/**
 * Contains a Map of effective permissions and their corresponding AclItems. Besides it indicates if all the effective permissions
 * share the same properties like allow/deny, FileInherit flag and if they're all generic permissions.
 * 
 * This object is not stored in the database, but it is used in the {@link FileTreeView}.
 * 
 * @author Reinier
 * @see {@link PermissionUtil#determineEffectivePermissions(java.util.List, FileInfo)}
 *
 */
public class EffectivePermissions {
	/** Map of effective permission values and their corresponding AclItem */
	private Map<Integer, AclItem> permissions;
	
	/** Indicates whether or not all the effective permissions have deny or allow. Null means that the effective permissions are a combination of deny and allow */
	private Boolean deny;
	
	/** Indicates if all the effective permissions have the flag FILE_INHERIT enabled */
	private boolean flagFileInherit;
	
	/** Indicates if all the permission have one of the generic permissions */
	private boolean genericAll;
	private boolean genericExecute;
	private boolean genericWrite;
	private boolean genericRead;
	
	public Map<Integer, AclItem> getPermissions() {
		return permissions;
	}
	public void setPermissions(Map<Integer, AclItem> permissions) {
		this.permissions = permissions;
	}
	public Boolean getDeny() {
		return deny;
	}
	public void setDeny(Boolean deny) {
		this.deny = deny;
	}
	public boolean isFlagFileInherit() {
		return flagFileInherit;
	}
	public void setFlagFileInherit(boolean flagFileInherit) {
		this.flagFileInherit = flagFileInherit;
	}
	public boolean isGenericAll() {
		return genericAll;
	}
	public void setGenericAll(boolean genericAll) {
		this.genericAll = genericAll;
	}
	public boolean isGenericExecute() {
		return genericExecute;
	}
	public void setGenericExecute(boolean genericExecute) {
		this.genericExecute = genericExecute;
	}
	public boolean isGenericWrite() {
		return genericWrite;
	}
	public void setGenericWrite(boolean genericWrite) {
		this.genericWrite = genericWrite;
	}
	public boolean isGenericRead() {
		return genericRead;
	}
	public void setGenericRead(boolean genericRead) {
		this.genericRead = genericRead;
	}
	
	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("EffectivePermissions [permissions=");
		builder.append(permissions);
		builder.append(", deny=");
		builder.append(deny);
		builder.append(", flagFileInherit=");
		builder.append(flagFileInherit);
		builder.append(", genericAll=");
		builder.append(genericAll);
		builder.append(", genericExecute=");
		builder.append(genericExecute);
		builder.append(", genericWrite=");
		builder.append(genericWrite);
		builder.append(", genericRead=");
		builder.append(genericRead);
		builder.append("]");
		return builder.toString();
	}
}
