package com.perdemia.permissionanalyzer.data;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class FilterSet extends DBObject {
private final static Logger logger = LoggerFactory.getLogger(FilterSet.class.getName());
	
	/** Table attributes */
	private final static String[] tableAttributes = new String[]{"NAME", "FILTER_SET_TYPE"};
	
	/** Table fields */
	private String name;
	private FilterSetType type;
		
	public static enum FilterSetType {
		MEMBER, PERMISSION, FOLDER
	}
		
	/** Helper field, not saved in DB */
	private List<Filter> filters;
	
	/**
	 * The last ID that was used for an INSERT of this class in the database. If 0 is returned then the
	 * {@link DBObject#insert()} method will retrieve the max ID from the database.
	 */
	private static int lastId;
	
	public FilterSet() {}	

	public FilterSet(String name, FilterSetType type) {
		this.name = name;
		this.type = type;
	}
		
	//========================================
	// GENERAL DB ACTIONS
	//========================================
	
	public static FilterSet getById(Integer id) throws DBException {
		return (FilterSet) getById(FilterSet.class, id);
	}
		
	@SuppressWarnings("unchecked")
	public static List<FilterSet> where(String whereClause, Object[] parameters) throws DBException {
		return where(FilterSet.class, whereClause, parameters, null, 0);
	}
	
	@SuppressWarnings("unchecked")
	public static List<FilterSet> where(String whereClause, Object[] parameters, String orderBy) throws DBException {
		return where(FilterSet.class, whereClause, parameters, orderBy, 0);
	}
	
	@SuppressWarnings("unchecked")
	public static List<FilterSet> where(String whereClause, Object[] parameters, String orderBy, int limit) throws DBException {
		return where(FilterSet.class, whereClause, parameters, orderBy, limit);
	}
	
	public static FilterSet findFirst(String whereClause, Object... parameters) throws DBException {
		return (FilterSet) findFirst(FilterSet.class, whereClause, parameters);
	}
	
	public static int delete(Integer id) throws DBException {
		// delete corresponding filters
		// the cascade delete on the foreign key doesn't work on MSSQL
		List<Filter> filters = Filter.where("FILTERSET_ID = ?", new Object[]{id});
		for (Filter filter : filters) {
			filter.delete();
		}
		return delete(FilterSet.class, id);
	}
	
	public static int delete(String whereClause, Object...parameters) throws DBException {
		return delete(FilterSet.class, whereClause, parameters);
	}
	
	@Override
	protected String[] getTableAttributes(){
		return tableAttributes;
	}
	
	@Override
	protected void buildObject(ResultSet rs) throws SQLException {
		id = (rs.getInt("id"));
		name = rs.getString(tableAttributes[0]);
		type = FilterSetType.values()[rs.getInt(tableAttributes[1])];
	}
	
	@Override
	protected void setQueryParameters(PreparedStatement ps) throws SQLException {
		ps.setString(1, name);
		ps.setInt(2, type != null ? type.ordinal() : null);
	}

	// ========================================
	// HELPER METHODS
	// ========================================

	public void loadFilters() throws DBException {
		//if (filters == null) {
			filters = Filter.where("filterset_id = ?", new Object[]{id});
		//}
	}
	
	//========================================
	// GETTERS AND SETTERS
	//========================================
	
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public FilterSetType getType() {
		return type;
	}

	public void setType(FilterSetType type) {
		this.type = type;
	}

	public List<Filter> getFilters() {
		return filters;
	}

	public void setFilters(List<Filter> filters) {
		this.filters = filters;
	}

	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("FilterSet [name=");
		builder.append(name);
		builder.append(", type=");
		builder.append(type);
		builder.append(", filters=");
		builder.append(filters);
		builder.append(", id=");
		builder.append(id);
		builder.append("]");
		return builder.toString();
	}
	
	@Override
	public int getLastId() {
		return lastId;
	}

	@Override
	public void setLastId(int lastId) {
		FilterSet.lastId = lastId;
	}	
}
