package com.perdemia.permissionanalyzer.data;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ReportFilterSet extends DBObject {
	private final static Logger logger = LoggerFactory.getLogger(ReportFilterSet.class.getName());
		
	/** Table attributes */
	private final static String[] tableAttributes = new String[]{"REPORT_ID", "FILTERSET_ID"};
	
	/** Table fields */
	private int reportId;
	private Integer filterSetId;
	
	/**
	 * The last ID that was used for an INSERT of this class in the database. If 0 is returned then the
	 * {@link DBObject#insert()} method will retrieve the max ID from the database.
	 */
	private static int lastId;
	
	public ReportFilterSet(){}
		
	public ReportFilterSet(int reportId, Integer filterSetId) {
		this.reportId = reportId;
		this.filterSetId = filterSetId;
	}
	
	//========================================
	// GENERAL DB ACTIONS
	//========================================
	
	public static ReportFilterSet getById(Integer id) throws DBException {
		return (ReportFilterSet) getById(ReportFilterSet.class, id);
	}
		
	@SuppressWarnings("unchecked")
	public static List<ReportFilterSet> where(String whereClause, Object[] parameters) throws DBException {
		return where(ReportFilterSet.class, whereClause, parameters, null, 0);
	}
	
	@SuppressWarnings("unchecked")
	public static List<ReportFilterSet> where(String whereClause, Object[] parameters, String orderBy) throws DBException {
		return where(ReportFilterSet.class, whereClause, parameters, orderBy, 0);
	}
	
	@SuppressWarnings("unchecked")
	public static List<ReportFilterSet> where(String whereClause, Object[] parameters, String orderBy, int limit) throws DBException {
		return where(ReportFilterSet.class, whereClause, parameters, orderBy, limit);
	}
	
	public static ReportFilterSet findFirst(String whereClause, Object... parameters) throws DBException {
		return (ReportFilterSet) findFirst(ReportFilterSet.class, whereClause, parameters);
	}
	
	public static int delete(Integer id) throws DBException {
		return delete(ReportFilterSet.class, id);
	}
	
	public static int delete(String whereClause, Object...parameters) throws DBException {
		return delete(ReportFilterSet.class, whereClause, parameters);
	}
	
	@Override
	protected String[] getTableAttributes(){
		return tableAttributes;
	}
	
	@Override
	protected void buildObject(ResultSet rs) throws SQLException {
		id = (rs.getInt("id"));
		reportId = rs.getInt(tableAttributes[0]);
		filterSetId = getNullableInteger(rs, tableAttributes[1]);
	}
	
	@Override
	protected void setQueryParameters(PreparedStatement ps) throws SQLException {
		ps.setInt(1, reportId);	
		setNullableInteger(ps, 2, filterSetId);
	}
	
	//========================================
	// GETTERS AND SETTERS
	//========================================
	
	public int getReportId() {
		return reportId;
	}

	public void setReportId(int reportId) {
		this.reportId = reportId;
	}

	public Integer getFilterSetId() {
		return filterSetId;
	}

	public void setFilterSetId(Integer filterSetId) {
		this.filterSetId = filterSetId;
	}
	
	@Override
	public int getLastId() {
		return lastId;
	}

	@Override
	public void setLastId(int lastId) {
		ReportFilterSet.lastId = lastId;
	}	
}
